#!/usr/bin/env python3
"""
Global Minimization Algorithm modified for SWW
by Javantea
Nov 9, 2015
Based on Global Minimization Algorithm
by Javantea
Nov 26, 2014 - Dec 1, 2014
Based on Convert PDB to Spherical Coordinates Oct 28, 2014 - Nov 26, 2014
Based on metropolis_min1.py by Javantea, July 17-22, 2014
Based on s_4atom.py by Javantea, June 24, 2014
Based on basinhopping_min1.py by Javantea, July 23, 2014

TODO: Random selection.

10k 25 nodes:
Step 1: 47.033 seconds
Step 2: 62.653 seconds
10k 50 nodes:
Step 1: 176.698 seconds # double nodes, 3.7x time
Step 2: 251.426 seconds # double nodes, 4x time
100k 20 nodes:
Step 1: 320.358 seconds
Step 2: 44.233 seconds
100k 25 nodes:
Step 1: 473.610 seconds # 10x steps, 5x time
Step 2: 72.284 seconds
100k 50 nodes:
Step 1: 1739.802 # 2x nodes, 3.7x time
Step 2: 255.959 # 2x nodes, 3.5x time

Is it exponential or is it polynomial? Need at least 10 data points. Let's do it...

Definitely a bug...


TODO:
There are tons of skips to be done here. For one, loop detection. See loops1.py
"""
from __future__ import print_function
import os
from math import pi
import random
import time
import numpy as np
from scipy.optimize import minimize
from sww_optimize1 import LJ_Potential, V_SmallWideWorld, getNonConns, createMap4, ConnList
import mapmake1
tau = 2*pi

class degreesOfFreedom:
	def __init__(self, map1, r0=40, verbose=False):
		"""
		Find out how many degrees of freedom a 2D graph has.
		"""
		self.map1 = map1
		self.r0 = r0
		self.verbose = verbose
		self.configurations = [(r0, 0),
			(r0, pi),
			(r0, pi*0.5),
			(r0, pi*1.5)]
		
		self.configuration_names = ['x', '-x', 'y', '-y']
		
		self.polarCoords, self.atom_name_hash = map1.toPolar(verbose)
		# TODO: Loop detection, loops are not active.
		
		#num_spc = len(molecule.x0)
		self.num_active = 0
		self.activity = []
		for i, node in enumerate(map1.points):
			if len(node.conns) <= 1:
				# This is a hydrogen or something like it. Not active.
				self.activity.append(False)
				continue
			#end if
			conn_conn = 0
			#invalid = False
			for x in node.conns:
				# x is the index that the atom is connected to.
				if len(x.conns) == 1:
					# hydrogen doesn't count
					#activity.append(False)
					#invalid = True
					continue
				#end if
				conn_conn += 1
			#next x
			# Allow the above break to cause continue in this loop.
			#if invalid:
			#	continue
			#end if
			if conn_conn <= 1:
				# This atom is connected to hydrogens and one active 
				# atom, therefore it is not active.
				self.activity.append(False)
				continue
			#end if
			self.activity.append(True)
			self.num_active += 1
		#next i, atom
		
		#print("Active members in the molecule:", num_active)
		#print("Activity is %i long. molecule.x0 is %i, molecule.atoms is %i" % 
		#	(len(activity), len(molecule.x0), len(molecule.atoms)))
		
		# TODO: Which value is reverse for each configuration?
		# Messy? Or easy? Gimme a visual!
		# We could just leave the inactive ones where they lay...
		# But that's not rotating with the rest of the structure.
		
		# Our algorithm will stop long before 5**num_active
		# Using random configurations seems like a good idea.
		self.num_confs = len(self.configurations)
		self.totalEnd = self.num_confs ** self.num_active
		print("Configuration space ", self.totalEnd)
	#end def degreesOfFreedom(map1, r0=40, verbose=False)
	
	def range(self, start=0, end=None):
		if end is None or end > self.totalEnd:
			end = self.totalEnd
		#end if
		for i in range(start, end):
			out_i = []
			b = i
			for j, v in enumerate(self.activity):
				if v:
					out_i.append(self.configurations[b % self.num_confs])
					b //= self.num_confs
				else:
					out_i.append(self.polarCoords[j])
					#out_i.append(configurations[0])
				#end if
			#next v
			
			#print(out_i)
			yield out_i
		#next i
		
	#end def range(start=0, end=None)
	
	def random(self, count):
		if count is None:
			count = 10000
		#end if
		for i in range(count):
			out_i = []
			b = random.randint(0, self.totalEnd)
			for j, v in enumerate(self.activity):
				if v:
					out_i.append(self.configurations[b % self.num_confs])
					b //= self.num_confs
				else:
					out_i.append(self.polarCoords[j])
					#out_i.append(configurations[0])
				#end if
			#next v
			
			#print(out_i)
			yield out_i
		#next i
		
	#end def random(count)
#end class degreesOfFreedom

def usage():
	"Print the usage info."
	print(main.__doc__)
#end def usage()

def main():
	"""Usage: python3 algorithm1.py [-v] [-h] [-d] [-s start] [-e end] [filename.json] [...]
	Runs AltSci Global Minimization Algorithm on a JSON file.
	-v | --verbose Print the name combinations that produce the dr
	-h | --help    Print this help.
	-d | --data    Write a dat file as output instead of a array. Dat files 
	               are very easy to parse with numpy or gnuplot.
	-s | --start   Configuration to start with. Default: 0
	-e | --end     Configuration to end with. Default: num_confs ** num_active
	-n | --nodes   How many nodes in the backbone [default 20]
	-c | --conns   How many connections between the backbone [default 4]
	-b | --branches How many branches to add to the backbone [default 0]
	-r | --random  Use a random seed instead of a deterministic seed id
	-s | --shuffle Pick configurations randomly to improve likelihood of finding a solution
	"""
	import sys
	import getopt
	
	# Deterministic
	mapmake1.seed(12345)
	
	# Do not switch to scientific notation even though you enjoy it.
	# We're trying to do science here!
	np.set_printoptions(suppress=True)

	filenames = ['']
	verbose = False
	datOutput = False
	#if len(sys.argv) > 1:
	#	filename = sys.argv[1]
	#end if
	start = 0
	end = None
	actualRandom = False
	shuffle = False
	nodes = 20
	conns = 4
	branches = 0
	#steps = 20
	gtol = 1e-2
	try:
		flags, extra_args = getopt.getopt(sys.argv[1:], 'vhds:e:n:c:b:rug:', 
		['verbose', 'help', 'data', 'start=', 'end=', 'nodes=', 'conns=', 
		'branches=', 'random', 'shuffle', 'gtol='])
	except getopt.GetoptError as goe:
		print(goe)
		return 1
	#end try
	for flag in flags:
		if flag[0] == '-v' or flag[0] == '--verbose':
			verbose = True
		if flag[0] == '-d' or flag[0] == '--data':
			datOutput = True
		elif flag[0] == '-h' or flag[0] == '--help':
			usage()
			return 1
		elif flag[0] == '-s' or flag[0] == '--start':
			start = int(flag[1])
		elif flag[0] == '-e' or flag[0] == '--end':
			end = int(flag[1])
		elif flag[0] == '-n' or flag[0] == '--nodes':
			nodes = int(flag[1])
		elif flag[0] == '-c' or flag[0] == '--conns':
			conns = int(flag[1])
		elif flag[0] == '-b' or flag[0] == '--branches':
			branches = int(flag[1])
		elif flag[0] == '-r' or flag[0] == '--random':
			actualRandom = True
		elif flag[0] == '-u' or flag[0] == '--shuffle':
			shuffle = True
		elif flag[0] == '-g' or flag[0] == '--gtol':
			gtol = float(flag[1])
		#end if
	#next flag
	
	if extra_args != []:
		filenames = extra_args
	#end if
	
	lj_conf = LJ_Potential()
	
	# Necessary for the functions we call, optional for mapmake1.
	mapmake1.loadPolar()
	b0 = 40
	
	for filename in filenames:
		if filename != '':
			base_filename = filename.split(os.sep)[-1]
			map1 = mapmake1.Map()
			map1.loadJson(open(filename, 'r').read())
		else:
			# They didn't give a filename must want us to generate one.
			seed = 9999
			if actualRandom:
				# TODO: Reduces the entropy to very low.
				seed = mapmake1.randint(0, 0xffffffff)
			#end if
			# Don't sort, we'll use loop sort.
			map1 = createMap4(nodes, conns, branches, seed, False)
			map1.sortLinear()
			map1.sortLoop()
			map1.centerBound()
			map1.desc = "SWW Optimize map with {} nodes and {} conns [results]".format(nodes, conns)
			# FIXME: rare overwrites
			base_filename = 'algorithm1_%i' % random.SystemRandom().randint(0, 0xffffffff)
		#end if
		x0, conns = map1.getX0Conns(True)
		conns_cpp = ConnList(conns)
		#print(x0)
		if verbose:
			print("atom conns:", conns)
		#end if
		nonConns = ConnList(getNonConns(conns, True))
		V_init = V_SmallWideWorld(x0, conns_cpp, b0, lj_conf, nonConns)

		# Not necessary to get spherical coordinates of the original coordinates.
		#output = [spherical2.cartesianToSpherical(v) for v in molecule.x0]
		#print("Spherical coordinates of atoms:")
		#print(output)
		
		# This may be used eventually
		#sphericalCoords, atom_name_hash = molecule.toSpherical(verbose)
		
		minimizer_opts = {"method": "L-BFGS-B", 'args':(conns_cpp, b0, lj_conf, nonConns), 'options':{'gtol':gtol, 'eps':1e-4}}

		# Reduced number of possible configurations based on Degrees of Freedom
		configs = degreesOfFreedom(map1, verbose=verbose)
		if shuffle:
			configs_gen = configs.random(end)
		else:
			configs_gen = configs.range(start, end)
		#end if
		i = 0
		start_t = time.time()
		V = []
		t = []
		for configuration in configs_gen:
			# I was thinking about doing this, but it's easier to not.
			#cart_i = toCartesian(configuration, verbose)
			#  Cartesian in this context means dr = r2-r1. This 
			# must be used to generate atom positions r2 and r1.
			#print(configuration)
			map1.setCartesian(configuration)
			# Gives much lower numbers because overlap is common.
			#print(mapmake1.randint(0, 100000))
			map1.randomize(8)
			map1.centerBound()
			x_i, tmp = map1.getX0Conns(True)
			V_i = V_SmallWideWorld(x_i, conns_cpp, b0, lj_conf, nonConns)
			V.append(V_i)
			t.append(time.time() - start_t)
			#print(i, V_i)
			#open(base_filename.replace('.json', '') + ('.conf%i.json' % i), 'w').write(map1.saveJson())
			i += 1
		#next configuration
		minV = min(V)
		min_i = V.index(minV)
		print("Initial Potential:", V_init)
		print("Minimum Potential:", minV, min_i)
		print("Algorithm Step 1: %3.3f seconds" % (time.time() - start_t))
		if datOutput:
			datFilename_i = 0
			while os.path.exists('algorithm1_Vt%i.dat' % datFilename_i):
				datFilename_i += 1
			#loop
			t_min_dat = [t[0]]
			V_min_dat = [V[0]]
			curr_V_min = V[0]
			for t_i, V_i in zip(t, V):
				if V_i < curr_V_min:
					t_min_dat.append(t_i)
					V_min_dat.append(V_i)
					curr_V_min = V_i
				#end if
			#next t_i, V_i
			with open('algorithm1_Vt%i.dat' % datFilename_i, 'w') as dat_file:
				dat_file.write('\n'.join(['%3.7f %3.7f' % x for x in zip(t_min_dat, V_min_dat)]))
			#end with
			print('algorithm1_Vt%i.dat' % datFilename_i)
		#end if
		
		start2_t = time.time()
		# Restore the position before you run degreesOfFreedom because it uses spherical Coordinates.
		#print(x0, x0.reshape(len(map1.points), 2))
		map1.setPos(x0.reshape(len(map1.points), 2))
		#print([str(p) for p in map1.points])
		
		# Save the minimum of course!
		for configuration in configs.range(min_i, min_i+1):
			# I was thinking about doing this, but it's easier to not.
			#cart_i = toCartesian(configuration, verbose)
			#  Cartesian in this context means dr = r2-r1. This 
			# must be used to generate atom positions r2 and r1.
			#print(configuration)
			# Prime the random number generator, cheaper than keeping x0, trust me.
			mapmake1.seed(12345)
			for i in range(min_i):
				#print(mapmake1.randint(0, 100000))
				map1.randomize(8)
			#next i
			map1.setCartesian(configuration)
			# Gives much lower numbers because overlap is common.
			#print(mapmake1.randint(0, 100000))
			map1.randomize(8)
			map1.centerBound()
			x_i, tmp = map1.getX0Conns(True)
			V_i = V_SmallWideWorld(x_i, conns_cpp, b0, lj_conf, nonConns)
			print('Potential Recalc:', min_i, V_i)
			open(base_filename.replace('.json', '') + ('.conf%i.json' % min_i), 'w').write(map1.saveJson())
			open(base_filename.replace('.json', '') + ('.conf%i.svg' % min_i), 'w').write(map1.drawSvg())
			res = minimize(V_SmallWideWorld, x_i, **minimizer_opts)
			print(res)
			Vmin = V_SmallWideWorld(res.x, conns_cpp, b0, lj_conf, nonConns)
			print('Local Minimized:', Vmin)
			m = np.matrixlib.matrix(res.x)
			m = m.reshape(len(map1.points), 2)
			map1.setPos([np.array(xa)[0] for xa in m])
			map1.centerBound()
			open(base_filename.replace('.json', '') + ('.conf%imin.json' % min_i), 'w').write(map1.saveJson())
			open(base_filename.replace('.json', '') + ('.conf%imin.svg' % min_i), 'w').write(map1.drawSvg())
		#next configuration
	#next filename
	print("Algorithm Step 2: %3.3f seconds" % (time.time() - start2_t))
	
	"""
	Using minimize:
	x0 = np.array([0.,0.,0., 0.,0.,0.99])
	conns = [[1], [0]]
	xmin = scipy.optimize.minimize(algorithm1.V_molecular_mult, x0, method='L-BFGS-B', args=(conns,))
	v = xmin.x
	v[3] -= v[0]
	v[4] -= v[1]
	v[5] -= v[2]
	v[0] = 0
	v[1] = 0
	v[2] = 0
	print(v)
	[ 0.          0.          0.          0.          0.          0.97718752]
	This shows us that the optimal distance between two connected items in 
	the potential (as it currently is) is 0.97718752 angstroms.
	In an optimized methane molecule, the distance is usually 1.0 angstroms.
	"""
	
	# return the last in case it's being called by a tester.
	return map1
#end def main()

if __name__ == '__main__':
	main()
#end if
