
/*
 * Stack isFunction from Underscore.js.
 */
function isFunction(obj)
{
  return !!(obj && obj.constructor && obj.call && obj.apply);
}

/**
class Storage(object):
    """
    Class used to store the lowest energy structure
    """
    **/
    //def __init__(this, minres):
function Storage(minres) {
    if (this._add) this._add(minres);
}
new Storage({});

Storage.prototype._add = function(minres)
{
    this.minres = minres;
    // Copy the array.
    this.minres.x = minres.x.slice(0);
}
Storage.prototype.update = function(minres)
{
    if (minres.fun < this.minres.fun) {
        this._add(minres);
        return true;
    } else {
        return false;
    }
}

Storage.prototype.get_lowest = function()
{
        return this.minres;
}

/**
class BasinHoppingRunner(object):
    """This class implements the core of the basinhopping algorithm.

    x0 : ndarray
        The starting coordinates.
    minimizer : callable
        The local minimizer, with signature ``result = minimizer(x)``.
        The return value is an `optimize.OptimizeResult` object.
    step_taking : callable
        This function displaces the coordinates randomly.  Signature should
        be ``x_new = step_taking(x)``.  Note that `x` may be modified in-place.
    accept_tests : list of callables
        To each test is passed the kwargs `f_new`, `x_new`, `f_old` and
        `x_old`.  These tests will be used to judge whether or not to accept
        the step.  The acceptable return values are True, False, or ``"force
        accept"``.  If the latter, then this will override any other tests in
        order to accept the step.  This can be used, for example, to forcefully
        escape from a local minimum that ``basinhopping`` is trapped in.
    disp : bool, optional
        Display status messages.

    """*/
function BasinHoppingRunner(x0, minimizer, step_taking, accept_tests, disp)
{
    //def __init__(this, x0, minimizer, step_taking, accept_tests, disp=False):
    if (disp == undefined) { disp = false; }
    // Copy the array.
    this.x = x0.slice(0);
    this.minimizer = minimizer;
    this.step_taking = step_taking;
    this.accept_tests = accept_tests;
    this.disp = disp;

    this.nstep = 0;

    //# initialize return object
    this.res = {}; //scipy.optimize.OptimizeResult();
    this.res.minimization_failures = 0;

    //# do initial minimization
    var minres = minimizer(this.x);
    if (!minres.success) {
        this.res.minimization_failures++;
        if (this.disp) {
            debug("warning: basinhopping: local minimization failure");
        }
    }
    // Copy the array.
    this.x = minres.x.slice(0);
    this.energy = minres.fun;
    if (this.disp) {
        debug("basinhopping step " + this.nstep + ": f " + this.energy);
    }

    //# initialize storage class
    this.storage = new Storage(minres);

    if (minres.nfev != undefined)
        this.res.nfev = minres.nfev;
    if (minres.njev != undefined)
        this.res.njev = minres.njev;
    if (minres.nhev != undefined)
        this.res.nhev = minres.nhev;
}
new BasinHoppingRunner([0], function(){ return {success:true, x:[0]} }, function(){}, [], false);

/**
 * Do one monte carlo iteration
    Randomly displace the coordinates, minimize, and decide whether
    or not to accept the new coordinates.
 **/
BasinHoppingRunner.prototype._monte_carlo_step = function()
{
    //# Take a random step.  Make a copy of x because the step_taking
    //# algorithm might change x in place
    var x_after_step = np.copy(this.x);
    x_after_step = this.step_taking(x_after_step);

    //# do a local minimization
    var minres = this.minimizer(x_after_step);
    var x_after_quench = minres.x;
    var energy_after_quench = minres.fun;
    if (!minres.success) {
        this.res.minimization_failures++;
        if (this.disp) {
            debug("warning: basinhopping: local minimization failure");
        }
    }

    if (minres.nfev != undefined)
        this.res.nfev = minres.nfev;
    if (minres.njev != undefined)
        this.res.njev = minres.njev;
    if (minres.nhev != undefined)
        this.res.nhev = minres.nhev;

    //# accept the move based on this.accept_tests. If any test is false,
    //# than reject the step.  If any test returns the special value, the
    //# string 'force accept', accept the step regardless.  This can be used
    //# to forcefully escape from a local minimum if normal basin hopping
    //# steps are not sufficient.
    var accept = true;
    var i;
    //for test in this.accept_tests:
    for(i = 0; i < this.accept_tests.length; i++) {
        var test = this.accept_tests[i];
        // f_new=energy_after_quench, x_new=x_after_quench, f_old=this.energy, x_old=this.x
        testres = test(energy_after_quench, x_after_quench, this.energy, this.x);
        if (typeof testres == 'boolean') {
            if (!testres) {
                accept = false;
            }
        } else if (typeof testres == 'string') {
            if (testres == "force accept") {
                accept = true;
                break;
            } else {
                throw Exception("accept test must return bool or string " +
                                    "'force accept'. Type is " + (typeof testres));
            }
        } else {
            throw Exception("accept test must return bool or string " +
                                "'force accept'. Type is " + (typeof testres));
        }
    }

    //# Report the result of the acceptance test to the take step class.
    //# This is for adaptive step taking
    if (this.step_taking.report != undefined) {
        this.step_taking.report(accept, {f_new:energy_after_quench,
                                x_new:x_after_quench, f_old:this.energy,
                                x_old:this.x});
    }

    return [accept, minres];
}

/**
 * Do one cycle of the basinhopping algorithm
 **/
BasinHoppingRunner.prototype.one_cycle = function()
{
        this.nstep++;
        new_global_min = false;

        accept, minres = this._monte_carlo_step()

        if (accept) {
            this.energy = minres.fun;
            // Copy the array
            this.x = minres.x.slice(0);
            new_global_min = this.storage.update(minres);
        }

        //# print some information
        if (this.disp) {
            this.print_report(minres.fun, accept);
            if (new_global_min) {
                debug("found new global minimum on step "+this.nstep+" with function"+
                      " value " + this.energy);
            }
        }

        //# save some variables as BasinHoppingRunner attributes
        this.xtrial = minres.x;
        this.energy_trial = minres.fun;
        this.accept = accept;

        return new_global_min;
}
/**
 * print a status update
 **/
BasinHoppingRunner.prototype.print_report = function(energy_trial, accept)
{
    var minres = this.storage.get_lowest();
    debug("basinhopping step "+this.nstep+": f "+this.energy+" trial_f "+
        energy_trial+" accepted "+accept+" lowest_f " + minres.fun);
}

/**
class AdaptiveStepsize(object):
    Class to implement adaptive stepsize.

    This class wraps the step taking class and modifies the stepsize to
    ensure the true acceptance rate is as close as possible to the target.

    Parameters
    ----------
    takestep : callable
        The step taking routine.  Must contain modifiable attribute
        takestep.stepsize
    accept_rate : float, optional
        The target step acceptance rate
    interval : int, optional
        Interval for how often to update the stepsize
    factor : float, optional
        The step size is multiplied or divided by this factor upon each
        update.
    verbose : bool, optional
        Print information about each update

**/
function AdaptiveStepsize(takestep, accept_rate, interval, factor, verbose)
{
    if (accept_rate == undefined) { accept_rate = +0.5; }
    if (interval == undefined) { interval = 50|0; }
    if (factor == undefined) { factor = +0.9; }
    if (verbose == undefined) { verbose = true; }
    //def __init__(this, takestep, accept_rate=0.5, interval=50, factor=0.9,
    //             verbose=True):
    this.takestep = takestep;
    this.target_accept_rate = accept_rate;
    this.interval = interval;
    this.factor = factor;
    this.verbose = verbose;

    this.nstep = 0;
    this.nstep_tot = 0;
    this.naccept = 0;
}
new AdaptiveStepsize(null, 0.5, 50, 0.9, true);

AdaptiveStepsize.prototype.call = function(x)
{
    return this.take_step(x);
}

AdaptiveStepsize.prototype._adjust_step_size = function()
{
    old_stepsize = this.takestep.stepsize;
    accept_rate = this.naccept / this.nstep;
    if (accept_rate > this.target_accept_rate) {
        //#We're accepting too many steps.  This generally means we're
        //#trapped in a basin.  Take bigger steps
        this.takestep.stepsize /= this.factor;
    } else {
        //#We're not accepting enough steps.  Take smaller steps
        this.takestep.stepsize *= this.factor;
    }
    if (this.verbose) {
        debug("adaptive stepsize: acceptance rate "+accept_rate+" target "+this.target_accept_rate+" new "+
                "stepsize "+this.takestep.stepsize+" old stepsize " + old_stepsize);
    }
}

AdaptiveStepsize.prototype.take_step = function(x)
{
    this.nstep += 1;
    this.nstep_tot += 1;
    if (this.nstep % this.interval == 0) {
        this._adjust_step_size();
    }
    return this.takestep(x);
}

/**
 * called by basinhopping to report the result of the step
 **/
AdaptiveStepsize.prototype.report = function(accept, kwargs)
{
    if (accept) {
        this.naccept++;
    }
}

/**
class RandomDisplacement(object):
    """
    Add a random displacement of maximum size, stepsize, to the coordinates

    update x inplace
    """
**/
function RandomDisplacement(stepsize)
{
    //def __init__(this, stepsize=0.5):
    if (stepsize == undefined) { stepsize = +0.5; }
    this.stepsize = stepsize;
}
new RandomDisplacement(0.5);

RandomDisplacement.prototype.call = function(x)
{
    //def __call__(this, x):
    // FIXME: Use uniform random instead of JavaScript random.
    // TODO: Do x no matter what shape it is.
    // Array operation
    //x += np.random.uniform(-this.stepsize, this.stepsize, np.shape(x))
    var i;
    for (i = 0; i < x.length; i++) {
        x[i] += (Math.random() - 0.5) * this.stepsize * 2;
    }
    return x;
}

/**
 * class MinimizerWrapper(object):
 * wrap a minimizer function as a minimizer class
 **/
function MinimizerWrapper(minimizer, func, kwargs)
{
    //def __init__(this, minimizer, func=None, **kwargs):
    if (func == undefined) { func = null; }
    if (kwargs == undefined) { kwargs = null; }
    this.minimizer = minimizer;
    this.func = func;
    this.kwargs = kwargs;
}
new MinimizerWrapper(null, null, null);

MinimizerWrapper.prototype.call = function(x0)
{
    // How do you minimize without a function? Only if the minimizer has the function built-in.
    if (this.func == null) {
        var cargs = [x0];
        cargs.concat(this.kwargs);
        return this.minimizer.apply(cargs);
    } else {
        var cargs = [this.func, x0];
        cargs.concat(this.kwargs);
        return this.minimizer.apply(cargs);
    }
}

/*
    basinhopping(func, x0, niter=100, T=1.0, stepsize=0.5,
                 minimizer_kwargs=null, take_step=null, accept_test=null,
                 callback=null, interval=50, disp=False, niter_success=null)
    Find the global minimum of a function using the basin-hopping algorithm

    Parameters
    ----------
    func : callable ``f(x, *args)``
        Function to be optimized.  ``args`` can be passed as an optional item
        in the dict ``minimizer_kwargs``
    x0 : ndarray
        Initial guess.
    niter : integer, optional
        The number of basin hopping iterations
    T : float, optional
        The "temperature" parameter for the accept or reject criterion.  Higher
        "temperatures" mean that larger jumps in function value will be
        accepted.  For best results ``T`` should be comparable to the
        separation
        (in function value) between local minima.
    stepsize : float, optional
        initial step size for use in the random displacement.
    minimizer_kwargs : dict, optional
        Extra keyword arguments to be passed to the minimizer
        ``scipy.optimize.minimize()`` Some important options could be:

            method : str
                The minimization method (e.g. ``"L-BFGS-B"``)
            args : tuple
                Extra arguments passed to the objective function (``func``) and
                its derivatives (Jacobian, Hessian).

    take_step : callable ``take_step(x)``, optional
        Replace the default step taking routine with this routine.  The default
        step taking routine is a random displacement of the coordinates, but
        other step taking algorithms may be better for some systems.
        ``take_step`` can optionally have the attribute ``take_step.stepsize``.
        If this attribute exists, then ``basinhopping`` will adjust
        ``take_step.stepsize`` in order to try to optimize the global minimum
        search.
    accept_test : callable, ``accept_test(f_new=f_new, x_new=x_new, f_old=fold, x_old=x_old)``, optional
        Define a test which will be used to judge whether or not to accept the
        step.  This will be used in addition to the Metropolis test based on
        "temperature" ``T``.  The acceptable return values are True,
        False, or ``"force accept"``.  If the latter, then this will
        override any other tests in order to accept the step.  This can be
        used, for example, to forcefully escape from a local minimum that
        ``basinhopping`` is trapped in.
    callback : callable, ``callback(x, f, accept)``, optional
        A callback function which will be called for all minima found.  ``x``
        and ``f`` are the coordinates and function value of the trial minimum,
        and ``accept`` is whether or not that minimum was accepted.  This can be
        used, for example, to save the lowest N minima found.  Also,
        ``callback`` can be used to specify a user defined stop criterion by
        optionally returning True to stop the ``basinhopping`` routine.
    interval : integer, optional
        interval for how often to update the ``stepsize``
    disp : bool, optional
        Set to True to print status messages
    niter_success : integer, optional
        Stop the run if the global minimum candidate remains the same for this
        number of iterations.


    Returns
    -------
    res : OptimizeResult
        The optimization result represented as a ``OptimizeResult`` object.  Important
        attributes are: ``x`` the solution array, ``fun`` the value of the
        function at the solution, and ``message`` which describes the cause of
        the termination. The ``OptimzeResult`` object returned by the selected
        minimizer at the lowest minimum is also contained within this object
        and can be accessed through the ``lowest_optimization_result`` attribute.
        See `OptimizeResult` for a description of other attributes.

    See Also
    --------
    minimize :
        The local minimization function called once for each basinhopping step.
        ``minimizer_kwargs`` is passed to this routine.

    Notes
    -----
    Basin-hopping is a stochastic algorithm which attempts to find the global
    minimum of a smooth scalar function of one or more variables [1]_ [2]_ [3]_
    [4]_.  The algorithm in its current form was described by David Wales and
    Jonathan Doye [2]_ http://www-wales.ch.cam.ac.uk/.

    The algorithm is iterative with each cycle composed of the following
    features

    1) random perturbation of the coordinates

    2) local minimization

    3) accept or reject the new coordinates based on the minimized function
       value

    The acceptance test used here is the Metropolis criterion of standard Monte
    Carlo algorithms, although there are many other possibilities [3]_.

    This global minimization method has been shown to be extremely efficient
    for a wide variety of problems in physics and chemistry.  It is
    particularly useful when the function has many minima separated by large
    barriers. See the Cambridge Cluster Database
    http://www-wales.ch.cam.ac.uk/CCD.html for databases of molecular systems
    that have been optimized primarily using basin-hopping.  This database
    includes minimization problems exceeding 300 degrees of freedom.

    See the free software program GMIN (http://www-wales.ch.cam.ac.uk/GMIN) for
    a Fortran implementation of basin-hopping.  This implementation has many
    different variations of the procedure described above, including more
    advanced step taking algorithms and alternate acceptance criterion.

    For stochastic global optimization there is no way to determine if the true
    global minimum has actually been found. Instead, as a consistency check,
    the algorithm can be run from a number of different random starting points
    to ensure the lowest minimum found in each example has converged to the
    global minimum.  For this reason ``basinhopping`` will by default simply
    run for the number of iterations ``niter`` and return the lowest minimum
    found.  It is left to the user to ensure that this is in fact the global
    minimum.

    Choosing ``stepsize``:  This is a crucial parameter in ``basinhopping`` and
    depends on the problem being solved.  Ideally it should be comparable to
    the typical separation between local minima of the function being
    optimized.  ``basinhopping`` will, by default, adjust ``stepsize`` to find
    an optimal value, but this may take many iterations.  You will get quicker
    results if you set a sensible value for ``stepsize``.

    Choosing ``T``: The parameter ``T`` is the temperature used in the
    metropolis criterion.  Basinhopping steps are accepted with probability
    ``1`` if ``func(xnew) < func(xold)``, or otherwise with probability::

        exp( -(func(xnew) - func(xold)) / T )

    So, for best results, ``T`` should to be comparable to the typical
    difference in function values between local minima.

    .. versionadded:: 0.12.0

    References
    ----------
    .. [1] Wales, David J. 2003, Energy Landscapes, Cambridge University Press,
        Cambridge, UK.
    .. [2] Wales, D J, and Doye J P K, Global Optimization by Basin-Hopping and
        the Lowest Energy Structures of Lennard-Jones Clusters Containing up to
        110 Atoms.  Journal of Physical Chemistry A, 1997, 101, 5111.
    .. [3] Li, Z. and Scheraga, H. A., Monte Carlo-minimization approach to the
        multiple-minima problem in protein folding, Proc. Natl. Acad. Sci. USA,
        1987, 84, 6611.
    .. [4] Wales, D. J. and Scheraga, H. A., Global optimization of clusters,
        crystals, and biomolecules, Science, 1999, 285, 1368.

    Examples
    --------
    The following example is a one-dimensional minimization problem,  with many
    local minima superimposed on a parabola.

    >>> from scipy.optimize import basinhopping
    >>> func = lambda x: np.cos(14.5 * x - 0.3) + (x + 0.2) * x
    >>> x0=[1.]

    Basinhopping, internally, uses a local minimization algorithm.  We will use
    the parameter ``minimizer_kwargs`` to tell basinhopping which algorithm to
    use and how to set up that minimizer.  This parameter will be passed to
    ``scipy.optimize.minimize()``.

    >>> minimizer_kwargs = {"method": "BFGS"}
    >>> ret = basinhopping(func, x0, minimizer_kwargs=minimizer_kwargs,
    ...                    niter=200)
    >>> print("global minimum: x = %.4f, f(x0) = %.4f" % (ret.x, ret.fun))
    global minimum: x = -0.1951, f(x0) = -1.0009

    Next consider a two-dimensional minimization problem. Also, this time we
    will use gradient information to significantly speed up the search.

    >>> def func2d(x):
    ...     f = np.cos(14.5 * x[0] - 0.3) + (x[1] + 0.2) * x[1] + (x[0] +
    ...                                                            0.2) * x[0]
    ...     df = np.zeros(2)
    ...     df[0] = -14.5 * np.sin(14.5 * x[0] - 0.3) + 2. * x[0] + 0.2
    ...     df[1] = 2. * x[1] + 0.2
    ...     return f, df

    We'll also use a different local minimization algorithm.  Also we must tell
    the minimizer that our function returns both energy and gradient (jacobian)

    >>> minimizer_kwargs = {"method":"L-BFGS-B", "jac":True}
    >>> x0 = [1.0, 1.0]
    >>> ret = basinhopping(func2d, x0, minimizer_kwargs=minimizer_kwargs,
    ...                    niter=200)
    >>> print("global minimum: x = [%.4f, %.4f], f(x0) = %.4f" % (ret.x[0],
    ...                                                           ret.x[1],
    ...                                                           ret.fun))
    global minimum: x = [-0.1951, -0.1000], f(x0) = -1.0109


    Here is an example using a custom step taking routine.  Imagine you want
    the first coordinate to take larger steps then the rest of the coordinates.
    This can be implemented like so:

    >>> class MyTakeStep(object):
    ...    def __init__(this, stepsize=0.5):
    ...        this.stepsize = stepsize
    ...    def __call__(this, x):
    ...        s = this.stepsize
    ...        x[0] += np.random.uniform(-2.*s, 2.*s)
    ...        x[1:] += np.random.uniform(-s, s, x[1:].shape)
    ...        return x

    Since ``MyTakeStep.stepsize`` exists basinhopping will adjust the magnitude
    of ``stepsize`` to optimize the search.  We'll use the same 2-D function as
    before

    >>> mytakestep = MyTakeStep()
    >>> ret = basinhopping(func2d, x0, minimizer_kwargs=minimizer_kwargs,
    ...                    niter=200, take_step=mytakestep)
    >>> print("global minimum: x = [%.4f, %.4f], f(x0) = %.4f" % (ret.x[0],
    ...                                                           ret.x[1],
    ...                                                           ret.fun))
    global minimum: x = [-0.1951, -0.1000], f(x0) = -1.0109


    Now let's do an example using a custom callback function which prints the
    value of every minimum found

    >>> def print_fun(x, f, accepted):
    ...         print("at minimum %.4f accepted %d" % (f, int(accepted)))

    We'll run it for only 10 basinhopping steps this time.

    >>> np.random.seed(1)
    >>> ret = basinhopping(func2d, x0, minimizer_kwargs=minimizer_kwargs,
    ...                    niter=10, callback=print_fun)
    at minimum 0.4159 accepted 1
    at minimum -0.9073 accepted 1
    at minimum -0.1021 accepted 1
    at minimum -0.1021 accepted 1
    at minimum 0.9102 accepted 1
    at minimum 0.9102 accepted 1
    at minimum 2.2945 accepted 0
    at minimum -0.1021 accepted 1
    at minimum -1.0109 accepted 1
    at minimum -1.0109 accepted 1


    The minimum at -1.0109 is actually the global minimum, found already on the
    8th iteration.

    Now let's implement bounds on the problem using a custom ``accept_test``:

    >>> class MyBounds(object):
    ...     def __init__(this, xmax=[1.1,1.1], xmin=[-1.1,-1.1] ):
    ...         this.xmax = np.array(xmax)
    ...         this.xmin = np.array(xmin)
    ...     def __call__(this, **kwargs):
    ...         x = kwargs["x_new"]
    ...         tmax = bool(np.all(x <= this.xmax))
    ...         tmin = bool(np.all(x >= this.xmin))
    ...         return tmax and tmin

    >>> mybounds = MyBounds()
    >>> ret = basinhopping(func2d, x0, minimizer_kwargs=minimizer_kwargs,
    ...                    niter=10, accept_test=mybounds)

    */
function basinhopping(func, x0, niter, T, stepsize,
                 minimizer_kwargs, take_step, accept_test,
                 callback, interval, disp, niter_success)
{
    if(niter == undefined) { niter = 100|0; }
    if(T == undefined) { T = +1.0; }
    if(stepsize == undefined) { stepsize = +0.5; }
    if(minimizer_kwargs == undefined) { minimizer_kwargs = null; }
    if(take_step == undefined) { take_step = null; }
    if(accept_test == undefined) { accept_test = null; }
    if(callback == undefined) { callback = null; }
    if(interval == undefined) { interval = 50|0; }
    if(disp == undefined) { disp = false; }
    if(niter_success == undefined) { niter_success = null; }
    //x0 = np.array(x0)

    //# set up minimizer
    if (minimizer_kwargs === null) {
        minimizer_kwargs = [];
    }
    // TODO: implement minimize and other local minimization functions.
    var minimize = _minimize_lbfgsb;
    var minw_list = [minimize, func];
    minw_list.concat(minimizer_kwargs);
    var wrapped_minimizer = new MinimizerWrapper.apply(minw_list);

    //# set up step taking algorithm
    if (take_step != null) {
        if (!isFunction(take_step)) {
            throw Exception("take_step must be callable");
        }
        //# if take_step.stepsize exists then use AdaptiveStepsize to control
        //# take_step.stepsize
        if (take_step.stepsize != undefined) {
            take_step_wrapped = new AdaptiveStepsize(take_step, 0.5, interval,
                                                 0.9, disp);
        } else {
            take_step_wrapped = take_step;
        }
    } else {
        //# use default
        var displace = RandomDisplacement(stepsize);
        take_step_wrapped = new AdaptiveStepsize(displace, 0.5, interval,
                                             0.9, disp);
    }

    //# set up accept tests
    if (accept_test != null) {
        if (!isFunction(accept_test)) {
            throw Exception("accept_test must be callable");
        }
        accept_tests = [accept_test];
    } else {
        accept_tests = [];
    }
    //# use default
    var metropolis = MetropolisObject(T);
    accept_tests.push(metropolis.call);

    if (niter_success === null) {
        niter_success = niter + 2;
    }

    var bh = new BasinHoppingRunner(x0, wrapped_minimizer, take_step_wrapped,
                            accept_tests, disp);

    //# start main iteration loop
    var count = 0;
    var message = ["requested number of basinhopping iterations completed successfully"];
    for (i = 0; i < niter; i++) {
        var new_global_min = bh.one_cycle();

        if (isFunction(callback)) {
            //# should we pass a copy of x?
            var val = callback(bh.xtrial, bh.energy_trial, bh.accept);
            if (val != null) {
                if (val) {
                    message = ["callback function requested stop early by returning True"];
                    break;
                }
            }
        }

        count++;
        if (new_global_min) {
            count = 0;
        } else if (count > niter_success) {
            message = ["success condition satisfied"];
            break;
        }
    }

    //# prepare return object
    var res = bh.res;
    res.lowest_optimization_result = bh.storage.get_lowest();
    res.x = np.copy(res.lowest_optimization_result.x);
    res.fun = res.lowest_optimization_result.fun;
    res.message = message;
    res.nit = i + 1;
    return res;
}
