/*
 * Small Wide World
 * by Joel R. Voss aka. Javantea
 * Copyright 2007 AltSci Concepts
 * June 28, 2007
 *
 * Port to C from Python.
 */

#include <vector>
#include <string>
using std::vector;
using std::string;

class Point;

class Vector2 {
	public:
	int x, y;
	
	Vector2 operator -(Vector2 v);
	Vector2 operator +(Vector2 v);
	Vector2 operator *(float f);
	Vector2 operator /(float f);
	Vector2 &operator +=(Vector2 v);
	Vector2 &operator -=(Vector2 v);
	Vector2 &operator *=(float f);
	Vector2 &operator /=(float f);
	float length();
	string str();
	
	Vector2();
	Vector2(int x, int y);
	Vector2(Point *p);
	~Vector2();
};

class Point : public Vector2 {
	public:
	string name;
	string color;
	vector<Point *> conns;
	
	void setColor(string new_color);
	void addConn(Point *point, bool reConn=true);
	string str();
	
	Point(string init_name, int init_x, int init_y, vector<Point *> *init_conns = NULL, string init_color="#0000ff");
	Point();
	~Point();
	
};

typedef vector<Point> Points;
typedef vector<Point *> Pointps;


class Map {
	public:
	string name;
	Points points;
	int width;
	int height;
	int line_width;
	Point *selectedPoint;
	
	void addPoint(Point point);
	void addPoints(Points add_points);
	void randomize(int amount=180);
	void click(int x, int y);
	bool drag(int x, int y);
	void sort1();
	void sort2(float dist1=70, float f=0.5);
	void sort3();
	void sort4();
	void center();
	void squash();
	string str();
	string drawSvg();
	
	Map(Points *init_points, string init_name, int init_width, int init_height, int init_line_width);
	~Map();
	
};

typedef unsigned long uint32;
