/*
 * Small Wide World
 * by Joel R. Voss aka. Javantea
 * Copyright 2007 AltSci Concepts
 * June 28, 2007
 *
 * Port to C++ from Python.
 */

/*
Map maker
by Javantea
April 26, 2007
Ported to C++: June 28, 2007

Description:
An idea to make maps.
1: Place all points at the center.
2: Move each point randomly.
3: Move each point closer to it's connected points.
4: Goto 2
5: Profit!

TODO: Lots of conversion.
*/

//from random import randint, seed
//from math import sqrt
#include <cmath>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <cstring>
#include <cstdlib>
#include <set>
#include <iomanip>
#include <cstdint>
#include <map>
#include "sww.h"

using std::cout;
using std::cerr;
using std::endl;
using std::stringstream;

int randint(int min, int max)
{
	// FIXME: this randint function looks very suspicious.
	uint64_t r = rand();
	//cout << "r = " << r << ", RM = " << RAND_MAX << ", max=" << max << ", min=" << min << endl;
	float f = (float)r / (float)RAND_MAX;
	//cout << "f = " << f << endl;
	return (int)((float)min + f * (float)(max - min));
}

/*
Base class for all maps (friend, object, route).
Write the hard math and physics here once and use it by calling once.
*/
// points=None, width=640, height=480, line_width=2
Map::Map(Points *init_points, int init_width, int init_height, int init_line_width, string init_line_color)
{
	if(init_points != NULL)
	{
		points = *init_points;
	}
	//# There's a bug in python where if I do points=[] above, it 
	//# thinks it's a static when it's really really not.
	//if(points == None) self.points = [];
	name = "Small Wide World Map";
	desc = "Test of drawing a network map with svg";
	width  = init_width;
	height = init_height;
	line_width = init_line_width;
	line_color = init_line_color;
	selectedPoint = NULL;
}

Map::~Map()
{
	selectedPoint = NULL;
}

void Map::addPoint(Point point)
{
	points.push_back(point);
}

void Map::addPoints(Points &add_points)
{
	size_t i;
	for(i = 0; i < add_points.size(); i++)
	{
		//cout << add_points[i].drawSvg(line_width) << endl;
		points.push_back(add_points[i]);
	}
}

// amount = 180
void Map::randomize(int amount)
{
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		points[i].x += randint(-amount, amount);
		points[i].y += randint(-amount, amount);
	}
	//#next point
}
//#end void randomize()

bool Map::drag(int x, int y)
{
	if(selectedPoint == NULL)
	{
		return false;
	}
	//#end if
	Vector2 r(x, y);
	float d_len = (r - (Vector2)selectedPoint).length();
	if(d_len > 10)
	{
		selectedPoint->x = x;
		selectedPoint->y = y;
		return true;
	}
	//#end if
	return false;
}
//#end void drag(x, y)

void Map::click(int x, int y)
{
	Vector2 cp(x, y);
	float min_dr = -1.0;
	Point *min_dr_p = NULL;
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		Vector2 dr = cp - points[i];
		float dr_len = dr.length();
		if((min_dr < 0) || (dr_len < min_dr))
		{
			min_dr = dr_len;
			min_dr_p = &points[i];
		}
		//#end if
	}
	//#next point
	if(min_dr_p != NULL)
	{
		cout << "Setting color!" << endl;
		min_dr_p->setColor("#cc0000");
		if(selectedPoint != NULL)
		{
			selectedPoint->setColor("#0000ff");
		}
		//#end if
		selectedPoint = min_dr_p;
	}
	//#end if
}
//#end void click(x, y)

void Map::sort1()
{
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		size_t j;
		for(j = 0; j < points[i].conns.size(); j++)
		{
			Vector2 dr = *(points[i].conns[j]) - points[i];
			//#(dx, dy) = (conn.x - point.x, conn.y - point.y)
			//points[i] += dr * 0.4;
			points[i].x += dr.x * 0.4;
			points[i].y += dr.y * 0.4;
		}
		//#next conn
	}
	//#next point
}
//#end void sort1()

// dist1 = 70, f = 0.5
void Map::sort2(float dist1, float f)
{
	/*
	Use springs to decide the position of each point.
	*/
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		size_t j;
		for(j = 0; j < points[i].conns.size(); j++)
		{
			//#(dx, dy) = (conn.x - point.x, conn.y - point.y)
			Vector2 dr = *(points[i].conns[j]) - points[i];
			//#dr_len = sqrt(dx*dx + dy*dy)
			float dr_len = dr.length();
			//#print '	dr:', dr
			if(dr_len == 0.0)
			{
				points[i].x += 5;
				points[i].y += 5;
				dr_len = sqrt(5*5 + 5*5);
				//#print 'zero dr is now:', dr
			}
			//#end if
			//#print 'non-zero dr:', dr
			float m = 1;
			float a_len = (f/m) * (dr_len - dist1);
			//#dr_hat = (dx/dr, dy/dr)
			Vector2 dr_hat = dr / dr_len;
			//#a = (a_len*dr_hat[0], a_len*dr_hat[1]/m)
			Vector2 a = dr_hat * (a_len / m);
			float dt = 0.6;
			//#dp = (0.5*dt*dt*a[0], 0.5*dt*dt*a[1])
			Vector2 dp = a * (0.5*dt*dt);
			points[i].x += dp.x;
			points[i].y += dp.y;
		}
		//#next conn
	}
	//#next point
}
//#end void sort2()

void Map::sort3()
{
	/*
	Use springs to decide the position of each point.
	*/
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		if(points[i].conns.size() <= 1) continue;
		//#avgx = 0
		//#avgy = 0
		Vector2 avg = Vector2(0, 0);
		size_t j;
		for(j = 0; j < points[i].conns.size(); j++)
		{
			//#avgx += conn.x
			//#avgy += conn.y
			avg += *(points[i].conns[j]);
		}
		//#next conn
		//#point.x = avgx / len(point.conns)
		//#point.y = avgy / len(point.conns)
		Vector2 p = avg / points[i].conns.size();
		points[i].x = p.x;
		points[i].y = p.y;
	}
	//#next point
}
//#end void sort3()

void Map::sort4()
{
	/*
	Use a style of reverse attack on non-conns that are close.
	*/
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		size_t j;
		for(j = 0; j < points.size(); j++)
		{
			// nonconn
			//if(points[j] in point.conns)
			Points::iterator it = points.end(); //find(points.begin(), points.end(), points[j]);
			if(it != points.end())
			{
				continue;
			}
			//#end if
			Vector2 dr = points[i] - points[j];
			float dr_len = dr.length();
			//#print 'dr_len =', dr_len
			if(dr_len < 50)
			{
				if(dr_len == 0)
				{
					dr = Vector2(1, 1);
					dr_len = sqrt(2);
				}
				//#end if
				dr *= 4.0 / dr_len;
				points[i].x += dr.x;
				points[i].y += dr.y;
			}
			//#end if
		}
		//#next nonconn
	}
	//#next point
}
//#end void sort4()

void Map::sortBranch(float r0, bool *knownPos)
{
	/*"""
	void sortBranch(r0=40, knownPos=NULL)
	A brilliant sorting method that works well with branches but not loops.
	r0 is the length of the bonds set by the algorithm.
	knownPos is a list of booleans that tells us whether point i is given or should be computed.
	roundTry is required for loops. This can be optimized greatly.
	FIXME: Starfish where node 5 is the central node.
	Fixable by picking knownPos based on number of connections.
	TODO: Find other bugs.
	"""*/
	bool checkEverything = true;
	size_t nodes = points.size();
	if(nodes < 1)
	{
		//# Congrats, you're guaranteed to be sorted.
		return;
	} //#end if
	//# This is our algorithm... Written on the whiteboard last night. Understood many years ago but unable to write the code. Until now!
	if(knownPos == NULL) {
		knownPos = new bool[nodes];
		memset(knownPos, 0, sizeof(bool) * nodes);
		knownPos[0] = true;
		//# We don't need to do expensive checks if we have no knowns. *shrug*
		checkEverything = false;
		points[0].x = width / 2;
		points[0].y = height / 2;
	} //#end if
	//# v1 in spherical coordinates.
	//#v1 = (r1, theta1) == (r1*cos(theta1), r1*sin(theta1))
	Vector2 v1(r0, 0);
	//# -v1 in spherical coordinates
	//#neg_v1 = (r0, v1[1] + pi)
	bool fillReady = false;
	
	size_t roundTry;
	for(roundTry = 0; roundTry < nodes; roundTry++) {
		//#print("fillReady?", fillReady)
		bool knownDone = true;
		size_t i;
		for(i = 0; i < nodes; i++) {
			//#print(points[i].name, [c.name for c in points[i].conns])
			vector<size_t> givens;
			vector<size_t> unknowns;
			//# Set the position of its connections.
			Pointps::iterator conn;
			for(conn = points[i].conns.begin(); conn != points[i].conns.end(); ++conn) {
				size_t j = (*conn) - &(*points.begin());
				if(knownPos[j]) {
					givens.push_back(j);
					continue;
				} //#end if
				unknowns.push_back(j);
			} //#next conn
			if(knownPos[i]) {
				//# First solve the easy cases, then work your way up.
				if(unknowns.size() == 0) {
					//# Everything is known, we're good.
					//#print('Yay', points[i].x, points[i].y)
					continue;
				} //#end if
				if(givens.size() == 0) {
					//# All unknowns, so use the simplex method.
					size_t n_conns = unknowns.size();
					float theta0 = TAU / n_conns; //# tau = 2*pi
					int x = 0;
					vector<size_t>::iterator j;
					for(j = unknowns.begin(); j != unknowns.end(); ++j) {
						//# Spherical coordinates
						float theta_j = v1.y + (theta0 * (x+1));
						//#v_j = (r0, theta_j)
						points[*j].x = points[i].x + (r0*cos(theta_j));
						points[*j].y = points[i].y + (r0*sin(theta_j));
						knownPos[*j] = true;
						knownDone = false;
						//#print('Simple', x, j, points[j].x, points[j].y, theta_j)
						x++;
					} //#next j
					continue;
				} //#end if
				if(givens.size() == 1) {
					//# Very easy, create a v1 and do the same as above
					Vector2 v1i_cartesian = points[i] - points[givens[0]];
					float v1i_theta = atan2(v1i_cartesian.y, v1i_cartesian.x);
					//#v1i_spherical = (v1i_cartesian.length(), v1i_theta)
					//#neg_v1i_spherical = (v1i_cartesian.length(), v1i_theta+pi)
					size_t n_conns = unknowns.size() + 1;
					float theta0 = TAU / n_conns; //# tau = 2*pi
					int x = 0;
					vector<size_t>::iterator j;
					for(j = unknowns.begin(); j != unknowns.end(); ++j) {
						//# Spherical coordinates
						float theta_j;
						if(n_conns == 2) {
							theta_j = v1i_theta;
						} else {
							theta_j = v1i_theta+PI + (theta0 * (x+1));
						} //#end if
						//#v_j = (r0, theta_j)
						points[*j].x = points[i].x + (r0*cos(theta_j));
						points[*j].y = points[i].y + (r0*sin(theta_j));
						knownPos[*j] = true;
						knownDone = false;
						//#print('Simple1', x, j, points[j].x, points[j].y, theta_j, 'v1it', v1i_theta)
						x++;
					} //#next j
				} //#end if
				//#print("Not sure what to do for: given: %i, unknown: %i" % (givens.size(), unknowns.size()))
				//# Take the list of givens and create a perfect placement for our node
			} else {
				//# Don't use v1 until all known nodes are figured out.
				//# Fixes sortBranch_fail1.json
				if(not fillReady) { continue; }
				//#print("Not known pos", i, knownPos[i])
				if(givens.size() == 0) {
					//# We don't know any of our neighbors yet, nope.
					continue;
				} //#end if
				if(givens.size() == 1) {
					//# Use v1 to choose our position
					knownPos[i] = true;
					size_t j = givens[0];
					//#print(i, 'using v_%i' % j)
					float theta_j = v1.y;
					points[i].x = points[j].x + (r0 * cos(theta_j));
					points[i].y = points[j].y + (r0 * sin(theta_j));
					if(checkEverything) {
						//# Expensive check for overlapping points, stops early when no overlap occurs.
						//# Worst case scenario, 3*known points
						float thetas[] = {v1.y + (float)PI, v1.y + (float)(PI*0.5), v1.y + (float)(PI*1.5)};
						size_t theta_j_new_index;
						for(theta_j_new_index = 0; theta_j_new_index < sizeof(thetas)/sizeof(float); theta_j_new_index++) {
							float theta_j_new = thetas[theta_j_new_index];
							bool isOverlapped = false;
							size_t k;
							for(k = 0; k < nodes; k++) {
								if(k == i) { continue; }
								if(knownPos[k] == false) { continue; }
								Vector2 dr = points[i] - points[k];
								if(dr.lensq() < SMALL) {
									//#print("Found overlap, it's time to fix.")
									//# We're on top of something known, move.
									points[i].x = points[j].x + (r0 * cos(theta_j_new));
									points[i].y = points[j].y + (r0 * sin(theta_j_new));
									isOverlapped = true;
									break;
								} //#end if
							} //#next k, p
							//# No overlap found, this is good enough.
							if(!isOverlapped) { break; }
						} //#next theta_j_new
					} //#end if
					continue;
				} //#end if
				//#print('avg')
				//# Average of its peers
				Vector2 totalPos;
				size_t j;
				for(j = 0; j < givens.size(); j++) {
					totalPos += points[givens[j]];
				} //#next j
				Vector2 avg(totalPos.x / givens.size(), totalPos.x / givens.size());
				points[i].x = avg.x;
				points[i].y = avg.y;
				knownPos[i] = true;
			} //#end if
			//if(not (false in knownPos))
			if(std::find(knownPos, knownPos+nodes, false) == false) { break; }
		} //#next i
		if(std::find(knownPos, knownPos+nodes, false) == false) { break; }
		//# If knownDone is still true at this point, we're ready to fill in the rest of the values.
		if(knownDone) { fillReady = true; }
	} //#next roundTry
	if(std::find(knownPos, knownPos+nodes, false)) {
		cout << "Took " << roundTry + 1 << " tries, " << points.size() << " nodes" << endl;
	} //#end if
} //#end def sortBranch(r0=40, knownPos=NULL)

void Map::sortLinear(float b0)
{
	/*"""
	sortLinear(b0=40)
	Draws the "backbone" in a line.
	TODO: Test and Understand everything that you need to understand for backbone.
	FIXED: loopsort1_5.json backbone issue with a trivial graph.
	FIXME: python3 ../loopsort1.py -n 10 -c 2 puts c and d in the same place.
	"""*/
	//# Polar coordinates to the rescue!
	std::vector<PolarVector> v;
	//v = [(b0, 0) for i in points]
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		v.push_back(PolarVector(b0, 0));
	}
	setCartesian(&v);
} //#end def sortLinear(b0=40)

void Map::sortCircle(float b0)
{
	/*"""
	sortCircle(b0=40)
	Draws the "backbone" in a circle.
	Sort in a weird way. Doesn't work for long (or even reasonably sized) graphs.
	"""*/
	//# Polar coordinates to the rescue!
	//loadPolar()
	//#theta0 = tau / 12//# 30 degrees
	float theta0 = TAU / 9; //# 40 degrees
	Connectivity rev = getPolarConn();
	//#v = [(b0, theta0 * i) for i in range(points.size())]
	std::vector<PolarVector> v;
	//bonds = defaultdict(int)
	std::map<int, int> bonds;
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		bonds[i] = 0;
	}
	Connectivity::iterator rev_i;
	for(rev_i = rev.begin(); rev_i != rev.end(); ++rev_i) {
		int i = rev_i->a;
		int i_x = rev_i->parent;
		//#print('{0} -- {1} {2}'.format(points[i].name, points[i_x].name, (i_x + bonds[i_x])))
		v.push_back(PolarVector(b0, theta0 * ((-0.004 * i_x * i_x) + i_x + bonds[i_x])));
		//# Don't put two nodes in the same place hmmm
		if(i != i_x) { bonds[i_x]++; }
	} //#next i
	setCartesian(&v);
} //#end def sortCircle(b0=40)

void Map::sortSpiral(float b0)
{
	/*"""
	Draws the "backbone" in a spiral.
	Sort in a novel way. Works for backbone only and loops well, but not branches.
	TODO: Test and Understand everything that you need to understand for backbone.
	"""*/
	//# Use grid because we have the infrastructure.
	size_t rows = points.size();
	size_t cols = rows;
	std::vector <Pointps> grid;
	size_t i, j;
	for(i = 0; i < rows; i++) {
		Pointps vcols;
		for(j = 0; j < cols; j++) {
			vcols.push_back(NULL);
		}
		grid.push_back(vcols);
	} //#next i
	// equivalent to cols // 2, rows // 2
	int pos[] = {(int)cols >> 1, (int)rows >> 1};
	//#grid[pos[1]][pos[0]] = points[0]
	int directions[][2] = {{1, 0}, {0, 1}, {-1, 0}, {0, -1}};
	//#dir_names = 'right down left up'.split(' ')
	int currDirection = 0;
	Points::iterator p;
	//for(p in points) {
	for(p = points.begin(); p != points.end(); ++p) {
		//#print(p.name)
		//bool foundSpot = false;
		while(true) {
			if(grid[pos[1]][pos[0]] == NULL) {
				grid[pos[1]][pos[0]] = &(*p);
				//foundSpot = true;
				break;
			} //#end if
			//# Move in the current direction.
			//#print(dir_names[currDirection])
			pos[0] = pos[0] + directions[currDirection][0];
			pos[1] = pos[1] + directions[currDirection][1];
		} //#loop
		//# Move in the current direction until we can turn.
		while(true) {
			pos[0] = pos[0] + directions[currDirection][0];
			pos[1] = pos[1] + directions[currDirection][1];
			if(grid[pos[1]][pos[0]] == NULL) {
				break;
			} //#end if
			//#print(dir_names[currDirection], 'because', grid[pos[1]][pos[0]].name)
		} //#loop
		//# If we can turn, turn.
		int nextDirection = (currDirection + 1) % 4;
		int peekPos[] = {pos[0] + directions[nextDirection][0], pos[1] + directions[nextDirection][1]};
		if(grid[peekPos[1]][peekPos[0]] == NULL) {
			currDirection = nextDirection;
			//#print(dir_names[currDirection], 'turn')
		} //#end if
	} //#next i
	//# TODO: Shrink the grid if necessary.
	gridToPos(&grid, b0);
	centerBound();
} //#end def sortSpiral(b0=40)

void Map::sortLattice(float b0)
{
	/*"""
	Sort almost perfectly for completely unconnected nodes.
	TODO: take connections into account.
	TODO: Make rows and cols variable.
	TODO: Limited sortLattice using knownPos
	"""*/
	size_t nodes = points.size();
	size_t side_len = ceil(sqrt(nodes));
	cout << "side_len: " << side_len << ", nodes: " << nodes << endl;
	if(side_len < 2) {
		//# We don't want your tiny 1 node graph at this point.
		return;
	} //#end if
	Vector2 row_pos(0, 0);
	size_t i = 0;
	//# Reverse direction with each row for better performance.
	//# --->
	//# <---
	//# --->
	int mul = -1;
	size_t row, col;
	for(row = 0; row < side_len; row++) {
		size_t q = side_len;
		if(i + q > nodes) { q = nodes - i; }
		if(mul == 1) {
			mul = -1;
			row_pos.x += (side_len - 1) * b0;
		} else {
			mul = 1;
			row_pos.x -= (side_len - 1) * b0;
		} //#end if
		for(col = 0; col < q; col++) {
			Point &p = points[i];
			cout << "i: " << i << endl;
			cout << "row_pos.x + (mul * col * b0): " << row_pos.x << " + (" << mul << " * " << col << " * " << b0 << ")" << endl;
			// Since mul and col are size_t, they must be converted for this to work.
			p.x = row_pos.x + (((float)mul) * ((float)col) * b0);
			p.y = row_pos.y;
			cout << "p.x: " << p.x << endl;
			cout << "p.y: " << p.y << endl;
			i++;
		} //#next col
		row_pos.x += b0 * -0.5;
		row_pos.y += b0 * 0.8660254037844386;
	} //#next row
} //#end def sortLattice(b0=40)

MapGroup *Map::addNodeToGroup(Point *p, Pointps *left, Pointps *right, std::vector<MapGroup> *groups, MapGroup *currGroup)
{
	/*"""
	Static utility function, finds the right group and adds to right and left bracket functions.
	You are part of a group if you have two connections to the group.
	This function doesn't cover the case of the left bracket. That's handled in the calling function.
	currGroup is an optional attribute designed to optimize which groups we have to check.
	"""*/
	// TODO: C++ version is completely untested!
	//# TODO: improve this
	std::set<MapGroup *> myGroups;
	std::vector<MapGroup> *checkGroups;
	MapGroup *myGroup = NULL;
	if(currGroup) {
		checkGroups = new std::vector<MapGroup>();
		checkGroups->push_back(*currGroup);
	} else {
		checkGroups = groups;
	} //#end if
	Pointps::iterator q;
	//for(q in p.conns) {
	for(q = p->conns.begin(); q != p->conns.end(); ++q) {
		std::vector<MapGroup>::iterator group;
		for(group = checkGroups->begin(); group != checkGroups->end(); ++group) {
			//# Can't add anything to a leaf.
			if(group->finished) { continue; }
			//if(q in group.points) {
			if(find(group->points.begin(), group->points.end(), *q) != group->points.end()) {
				//# I may belong to this group I suppose...
				//if(group in myGroups) {
				if(find(myGroups.begin(), myGroups.end(), &(*group)) != myGroups.end()) {
					//# We found our group.
					myGroup = &(*group);
					break;
				} else {
					myGroups.insert(&(*group));
				} //#end if
			} //#end if
		} //#next group
		if(myGroup != NULL) { break; }
	} //#next q
	if(currGroup)
	{
		delete checkGroups;
		checkGroups = NULL;
	}
	if(myGroup == NULL && myGroups.size() != 0) {
		//# Assume we're in the first group. Not very good idea.
		myGroup = *(myGroups.begin());
	} //#end if
	if(myGroup == NULL) {
		//# First member creates the group
		myGroup = new MapGroup();
		myGroup->addPoint(p);
		groups->push_back(*myGroup);
		if(left->size() == 0) {
			//# Likely Left bracket
			myGroup->left_bracket = p;
		} else {
			//# Likely Right bracket
			myGroup->right_bracket = p;
		} //#end if
	} else {
		//# Add to existing group
		myGroup->addPoint(p);
		if(left->size() == 0 && myGroup->left_bracket == NULL) {
			//# Likely Left bracket
			myGroup->left_bracket = p;
		} else if(right->size() == 0 && myGroup->right_bracket == NULL) {
			//# Likely Right bracket
			myGroup->right_bracket = p;
		} //#end if
	} //#end if
	return myGroup;
} //#end def addNodeToGroup(p, left, right, groups, currGroup)

std::vector<MapGroup> Map::groupNodes()
{
	/*"""
	Requires sortLinear and sortFix to be called first.
	Returns a list of group objects:
		a list of nodes,
		a left bracket,
		a right bracket,
		a finished boolean
	This allows the organization much easier using a minor modification of sortLoop.
	Look at me algorithming.
	"""*/
	std::vector<MapGroup> groups;
	MapGroup *currGroup = NULL;
	MapGroup newGroup;
	
	Points::iterator p; 
	//for(p in points) {
	for(p = points.begin(); p != points.end(); ++p) {
		Pointps left;
		Pointps right;
		Pointps::iterator q;
		//for(q in p.conns) {
		for(q = p->conns.begin(); q != p->conns.end(); ++q) {
			if((*q)->x < p->x) {
				left.push_back(*q);
			} else {
				right.push_back(*q);
			} //#end if
		} //#next q
		if(left.size() == 0 || right.size() == 0) {
			if(left.size() == 1 || right.size() == 1) {
				//# This is a leaf
				//#print('leaf', p.name)
				MapGroup leafGroup;
				leafGroup.leaf(&(*p));
				groups.push_back(leafGroup);
			} else {
				//# This is a node from a group
				//#print('node from a group', p.name)
				currGroup = addNodeToGroup(&(*p), &left, &right, &groups, currGroup);
			} //#end if
		} else {
			//# left and right have something in them.
			if(left.size() == 1 || right.size() == 1) {
				//# We've got a bracket
				if(left.size() == 1) {
					//# Possible left bracket
					//# Check to see if I'm the start of the new group.
					Pointps lpoints;
					Pointps rpoints;
					Points::iterator r;
					//for(q in points) {
					for(r = points.begin(); r != points.end(); ++r) {
						//# Don't count yourself
						if(&(*r) == &(*p)) { continue; }
						if(r->x < p->x) {
							lpoints.push_back(&*r);
						} else {
							rpoints.push_back(&*r);
						} //#end if
					} //#next q
					//# Expensive on large maps for for for for.
					bool newGroupYN = true;
					Pointps::iterator s;
					Pointps::iterator t;
					//for(q in lpoints) {
					for(s = lpoints.begin(); s != lpoints.end(); ++s) {
						//for(r in q.conns) {
						for(t = (*s)->conns.begin(); t != (*s)->conns.end(); ++t) {
							//if(r in rpoints) {
							if(std::find(rpoints.begin(), rpoints.end(), *t) != rpoints.end()) {
								newGroupYN = false;
								break;
							} //#end if
						} //#next r
						if(newGroupYN == false) { break; }
					} //#next q
					if(newGroupYN) {
						//#print('left bracket', p.name)
						newGroup.addPoint(&*p);
						groups.push_back(newGroup);
						currGroup = &newGroup;
					} else {
						//#print('node from group 3', p.name)
						currGroup = addNodeToGroup(&*p, &left, &right, &groups, currGroup);
					} //#end if
				} else {
					//# Possible right bracket
					//#print('right bracket', p.name)
					currGroup = addNodeToGroup(&*p, &left, &right, &groups, currGroup);
				} //#end if
			} else {
				//# It's a group node apparently
				//#print('node from group 2', p.name)
				currGroup = addNodeToGroup(&*p, &left, &right, &groups, currGroup);
			} //#end if
		} //#end if
	} //#next p
	
	//# Fix the problem with node 0 in one group and the rest in another.
	std::vector<MapGroup>::iterator group;
	size_t j;
	for(group = groups.begin(); group != groups.end(); ++group) {
		if(group->points.size() > 2) { continue; }
		//# One or two nodes in a group on their own.
		bool foundConcat = false;
		size_t foundConcat_i = 0;
		for(j = 0; j < groups.size(); j++) {
			if(&(*group) == &(groups[j])) { continue; }
			//for(p in group.points) {
			Pointps::iterator v;
			for(v = group->points.begin(); v != group->points.end(); ++v) {
				int conns_in_G = 0;
				Pointps::iterator w;
				//for(q in p.conns) {
				for(w = (*v)->conns.begin(); w != (*v)->conns.end(); ++w) {
					if(std::find(groups[j].points.begin(), groups[j].points.end(), *w) != groups[j].points.end()) {
						conns_in_G++;
					} //#end if
				} //#next q
				if(conns_in_G >= 2) {
					foundConcat = true;
					foundConcat_i = j;
					break;
				} //#end if
			} //#next p
			if(foundConcat) { break; }
		} //#next j
		if(foundConcat) {
			//# Move all group members to the found group.
			//groups[foundConcat_i].points.extend(group.points);
			groups[foundConcat_i].points.insert(groups[foundConcat_i].points.end(), group->points.begin(), group->points.end());
			group->points.clear();
		} //#end if
	} //#next group
	
	//# Fix the problem of loop_bug3.json where 0 is conn to 5 6
	//# Looking for group A is small and connected to group B and C.
	//# Group B is small and connected to C also.
	std::vector<std::vector<bool> > groupsConns;
	size_t i, k;
	for(i = 0; i < groups.size(); i++) {
		MapGroup &group_i = groups[i];
		if(group_i.points.size() > 2) {
			//# Have to put something here for consistency.
			groupsConns.push_back(std::vector<bool>());
			continue;
		} //#end if
		//# One or two nodes in a group on their own.
		//# TODO: We can skip half of these.
		//groupConn = [false for j in range(groups.size())];
		std::vector<bool> groupConn;
		for(j = 0; j < groups.size(); j++) {
			groupConn.push_back(false);
		}
		for(j = 0; j < groups.size(); j++) {
			if(i == j) { continue; }
			for(k = 0; k < group_i.points.size(); k++) {
				Point *x = group_i.points[k];
				int conns_in_G = 0;
				Pointps::iterator u;
				//for(q in p.conns) {
				for(u = x->conns.begin(); u != x->conns.end(); ++u) {
					if(std::find(groups[j].points.begin(), groups[j].points.end(), *u) != groups[j].points.end()) {
						conns_in_G++;
						break;
					} //#end if
				} //#next q
				//#debug("conns in G: " + conns_in_G + " " + k);
				if(conns_in_G > 0) {
					groupConn[j] = true;
					break;
				} //#end if
			} //#next p
		} //#next j
		bool foundConcatYN = false;
		size_t foundConcat = 0;
		size_t foundConcat2 = 0;
		//# look for A in groupsConns.
		for(j = 0; j < groupsConns.size(); j++) {
			//#// We are connected to them.
			if(groupConn[j]) {
				//#// Look for the group we are both connected to.
				for(k = 0; k < groups.size(); k++) {
					//#print(groupConn[k], groupsConns[j])
					if(groupConn[k] && groupsConns[j].size() > k && groupsConns[j][k]) {
						//#// We have found a loop, add ourselves and them.
						foundConcatYN = true;
						foundConcat = k;
						foundConcat2 = j;
						break;
					} //#end if
				} //#next k
			} //#end if
		} //#next j
		//#debug("Concat 2 " + foundConcat + " " + i);
		if(foundConcatYN) {
			//#// Move all group members to the found group.
			//groups[foundConcat].points.extend(group_i.points);
			groups[foundConcat].points.insert(groups[foundConcat].points.end(), group_i.points.begin(), group_i.points.end());
			group_i.points.clear();
			MapGroup &group2 = groups[foundConcat2];
			//groups[foundConcat].points.extend(group2.points);
			groups[foundConcat].points.insert(groups[foundConcat].points.end(), group2.points.begin(), group2.points.end());
			group2.points.clear();
			//#// Have to put something here for consistency.
			groupsConns.push_back(std::vector<bool>());
		} else {
			groupsConns.push_back(groupConn);
		} //#end if
	} //#next i, group
	
	//# FIXME: when a group consumes a second group, cull doesn't work.
	//# python3 ../loopsort1.py -x 26 -n 20 -c 2
	//# Fix the bug with groups consuming more nodes than just their group.
	//# python3 ../loopsort1.py -x 2 -n 20 -c 1
	Pointps newGroups;
	std::vector<MapGroup>::iterator group_j;
	for(group_j = groups.begin(); group_j != groups.end(); ++group_j) {
		Pointps nodesOrphaned = group_j->cull();
		//newGroups.extend(nodesOrphaned);
		newGroups.insert(newGroups.end(), nodesOrphaned.begin(), nodesOrphaned.end());
	} //#next group
	Pointps::iterator group_p;
	for(group_p = newGroups.begin(); group_p != newGroups.end(); ++group_p) {
		MapGroup stemGroup;
		stemGroup.addPoint(*group_p);
		groups.push_back(stemGroup);
	} //#next p
	//removeGroups = [];
	//# Filter any blank groups.
	// FIXME: Learn to filter.
	//groups = [g for g in groups if g.points.size() > 0];
	//# Sort by minimum x value.
	// FIXME: Learn to sort.
	//groups.sort(key=lambda g:g.minX());
	return groups;
} //#end def groupNodes()
/*
	
	
	def sortFix(self, b0=40):
		"""
		Fix the problem with sortLinear above
		../loopsort1.py -n 10 -c 2 puts c and d in the same place.
		This fixes d but not j. An obvious problem is square instead of ngon, 
		but with the ngon it's broken putting it all the way over to the right.
		That's because it's seeing i-j being long and thinking that i is e. 
		This can benefit from grouping, but is it the only solution? Seems legit.
		
		"""
		for(p in points) {
			for(q in p.conns) {
				drx = abs(p.x - q.x)
				if(drx < SMALL) {
					//# Two connected items are on top of one another, should we fix it?
					//#print("Should we fix?", p.name, '--', q.name)
					//# Let's move q and everything to the right to the right.
					q.x += b0
					for(r in points) {
						if(r.x > p.x) {
							//#print("moving %s to the right" % r.name)
							r.x += b0
						} //#end if
					} //#next r
					//# Q got moved twice, so we fix that here. Don't think you can fix this.
					q.x -= b0
				} //#end if
			} //#next q
		} //#next p
	} //#end def sortFix(b0=40)
	

	def pointToIndex(self):
		//# Reverse point to index.
		mapPointToIndex = {}
		for(i, p in enumerate(points)) {
			mapPointToIndex[p] = i
		} //#next i, p
		return mapPointToIndex
	} //#end def pointToIndex()
	
	def solveDoubleLoop(self, group, inBetweensSet, ungrouped, knownPos, level=0):
		"""
		Solve double diamond problem and so forth.
		a--b--c--d--e--f--g
		a--d
		d--g
		As you can see, there is just one group in this graph.
		"""
		//# If this turns up more groups than the original, this is a cull bug we can fix.
		//# This seems strange, but believe me it works in all known problem cases.
		groups = self.groupNodes()
		for(group_new in groups) {
			print('\t', ', '.join([str(p.name) for p in group_new.points]))
		} //#next group
		equal_groups = false
		group_picked = NULL
		first_ungrouped = list(ungrouped)[0]
		for(group_new in groups) {
			if(first_ungrouped in group_new.points) {
				//# This is our group, so check to see if it is equal to group.
				if(group_new.points.size() == group.points.size()) {
					//# Same size, must be same data.
					equal_groups = true
					group_picked = group_new
				} else {
					//# Different group.
					equal_groups = false
					group_picked = group_new
				} //#end if
				break
			} //#end if
		} //#next group
		mapPointToIndex = self.pointToIndex()
		if(equal_groups) {
			//# double diamond problem
			//# given a b c, find d, then sortGroup?
			connectors = set()
			for(x in ungrouped) {
				for(v in inBetweensSet) {
					if(v in x.conns) {
						//# this is d.
						connectors.add(v)
					} //#end if
				} //#next v
			} //#next x
			print('Found %i connectors' % connectors.size())
			if(connectors.size() <= 2) {
				//# This almost solves double_diamond.json, but the problem is overlap.
				//# This is a singly or doubly connected thing, easy, just sort group.
				unGroupedGroup = MapGroup()
				//# Use d as the known, this is bound to cause problems, but...
				//# This is a trick to get consistency for python3 ../loopsort1.py -x 31 -n 20 -c 2
				connectors_sorted = list(connectors)
				connectors_sorted.sort(key=lambda x: x.x)
				unGroupedGroup.points = connectors_sorted + list(ungrouped)
				ungrouped_nodes = self.sortGroup(unGroupedGroup, knownPos, mapPointToIndex, 40, level + 1)
			} //#end if
		} else {
			//# We lucked out, it's a cull problem
			//# This is a very cheap algorithm I assume.
			//# python3 ../loopsort1.py -x 42 -n 20 -c 2
			groups_solve = set()
			for(x in ungrouped) {
				for(group_new in groups) {
					if(x in group_new.points) {
						groups_solve.add(group_new)
					} //#end if
				} //#next group_new
			} //#next x
			//#print('To solve:')
			ungrouped_nodes = 0
			for(group_new in groups_solve) {
				//#print('\t', ', '.join([str(p.name) for p in group_new.points]))
				if(group_new.points.size() == 1) {
					continue
				else if(group_new.points.size() == 2) {
					continue
				} //#end if
				ungrouped_nodes += self.sortGroup(group_new, knownPos, mapPointToIndex, 40, level + 1)
			} //#next group_new
		} //#end if
		
	} //#end def solveDoubleLoop(group, inBetweensSet, ungrouped, level=0)

	def sortLoop(self, b0=40, level=0):
		"""
		Start small, more and more because it will improve the speed of algorithm and optimize.
		Assumes that sortLinear has already been called.
		Uses sortBranch in combination with sortLoop to fix branches.
		"""
		groups = self.groupNodes()
		for(group in groups) {
			print('\t', ', '.join([str(p.name) for p in group.points]))
		} //#next group
		
		knownPos = [false for i in points]
		//# Reverse point to index dictionary.
		mapPointToIndex = self.pointToIndex()
		//#print([x.name for x in points[5].conns])
		//# TODO: optimize this double loop if it becomes an issue (so far not)
		//# FIXME: currX is not threadsafe but who would run two sorts at the same time?
		self.currX = NULL
		//#addX = 0
		ungrouped_nodes = 0
		for(group in groups) {
			if(group.points.size() < 1) {
				continue
			else if(group.points.size() == 1) {
				//# Single node groups are leaves or stems. For example: a--b--c--d--e
				p = group.points[0]
				if(self.currX == NULL) { self.currX = p.x
				//#addX += self.currX - p.x
				//#print('addX', p.name, addX)
				p.x = self.currX
				//#print('self.currX', p.name, self.currX)
				self.currX += b0
				//# TODO: May or may not be necessary for(leaves) {
				//#knownPos[mapPointToIndex[p]] = true
				continue
			else if(group.points.size() == 2) {
				//# A two point group? That's an error.
				print("Warning: Groups of two are pretty rare. One might say they shouldn't exist.")
				p = group.points[0]
				if(self.currX == NULL) { self.currX = p.x
				//#addX += self.currX - p.x
				p.x = self.currX
				//#print('self.currX', p.name, self.currX)
				self.currX += b0
				q = group.points[1]
				//#addX += self.currX - q.x
				q.x = self.currX
				self.currX += b0
				//# TODO: May or may not be necessary for(leaves) {
				//#knownPos[mapPointToIndex[p]] = true
				//#knownPos[mapPointToIndex[q]] = true
				continue
			} //#end if
			ungrouped_nodes += self.sortGroup(group, knownPos, mapPointToIndex, b0)
		} //#next group
		//# If we know nothing, then just run sortBranch normally.
		if(knownPos == [false for i in points]) {
			knownPos = NULL
		} //#end if
		self.sortBranch(b0, knownPos)
		if(ungrouped_nodes > 1 and level < 1) {
			//# One expensive not yet working solution is to repeat sort by loop.
			//#self.sortLoop(b0, level+1)
			pass
		} //#end if
	} //#end def sortLoop(b0=40, level=0)
	
	def sortGroup(self, group, knownPos, mapPointToIndex, b0=40, level=0):
		"""
		Chooses the position of a group based on shape.
		returns the number of nodes in the group that were not positioned.
		//# Pentagon and up
		//# Pentagon: python3 loopsort1.py -x 8
		//# loopsort1_162.json
		//# Hexagon: python3 loopsort1.py -x 9
		//# loopsort1_163.json
		//# Septagon: python3 loopsort1.py -x 7
		//# loopsort1_164.json
		//# Octagon: python3 loopsort1.py -x 12
		//# loopsort1_167.json
		//# Octdecagon: python3 loopsort1.py -x 10
		//# loopsort1_165.json
		//# The first one goes to 180 - 360/shape. cos((pi-(2*pi/shape))), sin((pi-(2*pi/shape)))

		//# 108°
		theta1 = (pi-(2*pi/shape))
		pos1 = (cos(theta1), sin(theta1))
		//# 72°
		theta2a = pi - theta1
		//# 18°
		theta2b = (pi*0.5) - theta2a
		//# 108+18-90 = 36°, we could just remove pi*0.5 from theta2b if we wanted to.
		theta2b = theta2b + theta1 - (pi*0.5)
		pos2a = (cos(theta2b), sin(theta2b))
		pos2 = (pos1[0]+pos2a[0], pos1[0]+pos2a[0])
		"""
		//# TODO: Ton of little stuff here..
		p, q = NULL, NULL
		if(group.left_bracket and group.right_bracket) {
			print('lb rb', end=" ")
			p, q = group.left_bracket, group.right_bracket
			if(q not in p.conns) {
				//# Find the correct q in p.conns
				best_q = NULL
				maxConns = 0
				for(q_test in p.conns) {
					if(q_test.x > p.x and q_test.conns.size() > maxConns) {
						maxConns = q_test.conns.size()
						best_q = q_test
					} //#end if
				} //#next q_test
				q = best_q
				if(q == NULL) {
					print("Error: Unable to find a valid q 1")
					q = group.right_bracket
				} //#end if
			} //#end if
			print('p=%s, q=%s' % (p.name, q.name))
		} //#end if

		//# Find p and q amongst the group. Naturally the furthest left.
		if(group.left_bracket) {
			print('lb', end=" ")
			p = group.left_bracket
		} else {
			print('*shrug*', end=" ")
			p = group.points[0]
		} //#end if
		"""q = group.points[1]
		if(p == q) {
			q = group.points[0]
			if(p == q) {
				print("Error: bug in inBetweens code")
			} //#end if
		} //#end if"""
		//#if(q not in p.conns) {
		//# Find the correct q in p.conns
		best_q = NULL
		maxConns = 0
		for(q_test in p.conns) {
			//#print(q_test.name, 'has', q_test.conns.size(), 'conns')
			//# We don't want any other groups in our group.
			if(q_test not in group.points) { continue
			if(q_test.conns.size() > maxConns) {
				maxConns = q_test.conns.size()
				best_q = q_test
			} //#end if
		} //#next q_test
		q = best_q
		if(q == NULL) {
			print("Error: Unable to find a valid q 2", group.right_bracket)
			q = group.right_bracket
			return 0
		} //#end if
		} //#end if
		//#print('p=%s, q=%s' % (p.name, q.name))

		//# Find p, q, and inBetweens correctly!
		inBetweens = self.findRoute(p, q)
		
		if(inBetweens.size() > group.points.size()) {
			//# We're using parts of a connected group, verify that we're using 
			//# the least possible of it.
			inBetweensSet = set(inBetweens)
			groupSet = set(group.points)
			diff = inBetweensSet.difference(groupSet)
			found_unconn = []
			for(v in diff) {
				connected = false
				for(q in groupSet) {
					if(q in v.conns) {
						connected = true
						break
					} //#end if
				} //#next q
				if(not connected) {
					found_unconn.push_back(v)
					//#break
				} //#end if
			} //#next v
			if(found_unconn.size() > 0) {
				print("Found unconnected nodes:", [v.name for v in found_unconn])
				//# Find a route that avoids one of the unconnected.
				inBetweens_test = self.findRoute(p, q, [found_unconn[0]])
				if(inBetweens_test != NULL) {
					inBetweens_testSet = set(inBetweens_test)
					diff_test = inBetweens_testSet.difference(groupSet)
					//# Choose the shorter of the two:
					if(diff_test.size() < diff.size()) {
						print("Found a shorter route:", [v.name for v in inBetweens], [v.name for v in inBetweens_test])
						inBetweens = inBetweens_test
					} //#end if
				} //#end if
			} //#end if
		} //#end if
		
		if(self.currX == NULL) { self.currX = p.x
		//#p.x = self.currX
		//#print('self.currX', p.name, self.currX)
		
		//#TODO: Fix addX here.
		//#addX += self.currX - p.x
		//# There's always two nodes inline, the rest are perpendicular.
		self.currX += b0 * 2
		//# This can be fixed by setting the shape based on len of inBetweens, I suppose.
		//#if(inBetweens.size() != (shape)) {
		//#	print("inBetweens are different size.", ', '.join([str(x) for x in inBetweens]))
		//#	continue
		} //#end if
		shape = inBetweens.size()
		//# Simple bug to fix.
		if(shape == 1) {
			print("findRoute bug", [v.name for v in group.points])
			return 0
		} //#end if
		print("Picked ngon %i %s" % (shape, p.name), [v.name for v in inBetweens], [v.name for v in group.points])
		//# Using this method make a loop over shape for each vertex
		prevPos = (p.x, p.y)
		pos = []
		theta_inner = (2*pi/shape)
		for(i = 0; i < 1, shape); i++) {
			theta_i = pi-(theta_inner*i)
			pos_i = (b0*cos(theta_i), b0*sin(theta_i))
			pos.push_back((pos_i[0] + prevPos[0], pos_i[1] + prevPos[1]))
			prevPos = pos[-1]
		} //#next i
		//# Skip p.
		for(i, x in enumerate(inBetweens[1) {]):
			x.x = pos[i][0]
			x.y = pos[i][1]
		} //#next i, x
		//# Move all the others left shape - 2.
		"""
		dx = b0 * (shape - 2)
		further = q
		if(q.x < p.x) { further = p
		for(ttr in points) {
			if(ttr in inBetweens) { continue
			if(ttr.x >= further.x) {
				ttr.x -= dx
			} //#end if
		} //#next ttr
		"""
		knownPos[mapPointToIndex[p]] = true
		knownPos[mapPointToIndex[q]] = true
		for(inBetween in inBetweens) {
			knownPos[mapPointToIndex[inBetween]] = true
			//# Solve the moving issue.
			//#inBetween.x += addX
		} //#next inBetween
		
		//# Detect multiple loops in a single group
		ungrouped_nodes = 0
		//# level < 1 needed to avoid infinite recursion
		if(inBetweens.size() < group.points.size() and level < 1) {
			inBetweensSet = set(inBetweens)
			groupSet = set(group.points)
			ungrouped = groupSet.difference(inBetweensSet)
			print('Ungrouped:', ', '.join([x.name for x in ungrouped]))
			self.solveDoubleLoop(group, inBetweensSet, ungrouped, knownPos, level)
			ungrouped_nodes = ungrouped.size()
		} //#end if
		return ungrouped_nodes
	} //#end def sortGroup(group, knownPos, mapPointToIndex, b0=40, level=0)
	
	def toGrid(self, width=40):
		"""
		Unlike sortGrid, toGrid uses input positions to decide which cell to place 
		nodes in. It returns a grid (array of array of points and/or arrays).
		TODO: Improve the quality of position picking.
		"""
		r = self.bounds()
		dx = r[2] - r[0]
		dy = r[3] - r[1]
		cols = ceil(2 * dx / width)
		rows = ceil(2 * dy / width)
		nearby = [[0, 1], [1, 1], [1, 0], [0, -1], [-1, 0], [-1, 1], [-1, -1], [1, -1]]
		grid = []
		for(i = 0; i < rows); i++) {
			grid.push_back([NULL] * cols)
		} //#next i
		min_x = r[0]
		min_y = r[1]
		for(p in points) {
			row = round((p.y - min_y) / width)
			col = round((p.x - min_x) / width)
			//# Check for a nearby empty spot.
			found = false
			for(near in nearby) {
				if(row+near[1] < 0) { continue
				if(col+near[0] < 0) { continue
				if(grid[row+near[1]][col+near[0]] == NULL) {
					grid[row+near[1]][col+near[0]] = p
					found = true
					break
				} //#end if
			} //#next near
			if(not found) {
				if(grid[row][col]) {
					if(type(grid[row][col]) == list) {
						grid[row][col].push_back(p)
					} else {
						grid[row][col] = [grid[row][col], p]
					} //#end if
				} else {
					grid[row][col] = p
				} //#end if
			} //#end if
		} //#next p
		//# Should this be optional? Perhaps.
		gridToPos(grid, width)
		return grid
	} //#end def toGrid(width=40)

	def sortGrid(self, width=40, rearrange=true):
		"""
		Create an NxM grid, place one node per cell. This simplifies the branch sort algorithm into a lattice instead of a trigonometry problem.
		"""
		nodes = points.size()
		if(nodes <= 1) {
			//# Congratulations, you're guaranteed to be sorted.
			return
		} //#end if
		//# Allocating all at the start will become a mess with large values of num_points, so we make it extensible.
		knownPos = [false] * nodes
		grid_rows = 2
		grid_cols = 2
		grid = []
		for(i = 0; i < grid_rows); i++) {
			grid.push_back([NULL] * grid_cols)
		} //#next i
		//#print(grid)
		grid[0][0] = points[0]
		knownPos[0] = true
		//# This is bound to change because of extensible. That sucks.
		pointPosMap = {0: [0, 0]}
		//# FIXME: Not sure what the right number is, need some cheap boolean detector.
		for(roundTry = 0; roundTry < nodes); roundTry++) {
			//#print("fillReady?", fillReady)
			knownDone = true
			for(i = 0; i < nodes); i++) {
				givens = []
				unknowns = []
				//# Set the position of its connections.
				for(conn in points[i].conns) {
					j = points.index(conn)
					if(knownPos[j]) {
						givens.push_back(j)
						continue
					} //#end if
					unknowns.push_back(j)
				} //#next conn
				if(knownPos[i]) {
					if(unknowns.size() == 0) {
						//# Everything is known, we're good.
						//#print('Yay', map1.points[i].x, map1.points[i].y)
						continue
					} //#end if
					//#print('sorting from', map1.points[i].name)
					j = 0
					//# Put the unknowns on the grid near the position of the point.
					pos_i = pointPosMap[i]
					//#print(pointPosMap, i)
					left = [pos_i[0]-1, pos_i[1]]
					right = [pos_i[0]+1, pos_i[1]]
					up = [pos_i[0], pos_i[1]-1]
					down = [pos_i[0], pos_i[1]+1]
					ul = [pos_i[0]-1, pos_i[1]-1]
					ur = [pos_i[0]+1, pos_i[1]-1]
					dl = [pos_i[0]-1, pos_i[1]+1]
					dr = [pos_i[0]+1, pos_i[1]+1]
					change = [0, 0]
					for(p in [left, right, up, down, ul, ur, dl, dr]) {
						//# Move the p where it should go based on additions.
						p[0] += change[0]
						p[1] += change[1]
						//#print('try', p)
						dp = NULL
						if(p[0] + change[0] < 0) {
							//# available, but we will need to extend left, maybe up or down
							//# Extend left
							//#print(map1.points[unknowns[j]].name, end=' ')
							//#print('extend left', end='')
							for(k = 0; k < grid_rows); k++) {
								grid[k] = [NULL] + grid[k]
							} //#next k
							grid_cols += 1
							dp = [1, 0]
							change[0] += 1
							if(p[1] < 0) {
								//# Extend up
								//#print(', extend up', end='')
								grid.insert(0, [NULL] * grid_cols)
								grid_rows += 1
								dp[1] = 1
								change[1] += 1
							} //#end if
							if(p[1] > grid_rows) {
								//# Extend down
								//#print(', extend down', end='')
								grid.push_back([NULL] * grid_cols)
								grid_rows += 1
							} //#end if
							//#print()
							grid[p[1]+dp[1]][p[0]+dp[0]] = points[unknowns[j]]
							pointPosMap[unknowns[j]] = [p[0], p[1]]
							knownPos[unknowns[j]] = true
							//#print("set %s to %r" % (map1.points[unknowns[j]].name, p))
							j += 1
							if(j >= unknowns.size()) {
								break
							} //#end if
						else if(p[0] >= grid_cols) {
							//# available, but we will need to extend right
							//# Extend right
							//#print(map1.points[unknowns[j]].name, end=' ')
							//#print('extend right', end='')
							for(k = 0; k < grid_rows); k++) {
								grid[k] += [NULL]
							} //#next k
							grid_cols += 1
							dp = [0, 0]
							if(p[1] < 0) {
								//# Extend up
								//#print(', extend up', end='')
								grid.insert(0, [NULL] * grid_cols)
								grid_rows += 1
								dp[1] = 1
								change[1] += 1
							} //#end if
							if(p[1] > grid_rows) {
								//# Extend down
								//#print(', extend down', end='')
								grid.push_back([NULL] * grid_cols)
								grid_rows += 1
							} //#end if
							//#print()
							grid[p[1]+dp[1]][p[0]+dp[0]] = points[unknowns[j]]
							pointPosMap[unknowns[j]] = [p[0], p[1]]
							knownPos[unknowns[j]] = true
							//#print("set %s to %r" % (map1.points[unknowns[j]].name, p))
							j += 1
							if(j >= unknowns.size()) {
								break
							} //#end if
						else if(p[1] < 0) {
							//# available, but we will need to extend up
							//# Extend up
							//#print(map1.points[unknowns[j]].name, end=' ')
							//#print('extend up')
							grid.insert(0, [NULL] * grid_cols)
							grid_rows += 1
							dp = [0, 1]
							change[1] += 1
							grid[p[1]+dp[1]][p[0]+dp[0]] = points[unknowns[j]]
							pointPosMap[unknowns[j]] = [p[0], p[1]]
							knownPos[unknowns[j]] = true
							//#print("set %s to %r" % (map1.points[unknowns[j]].name, p))
							j += 1
							if(j >= unknowns.size()) {
								break
							} //#end if
						else if(p[1] >= grid_rows) {
							//# available, but we will need to extend down
							//# Extend down
							//#print(map1.points[unknowns[j]].name, end=' ')
							//#print('extend down')
							grid.push_back([NULL] * grid_cols)
							grid_rows += 1
							grid[p[1]][p[0]] = points[unknowns[j]]
							pointPosMap[unknowns[j]] = [p[0], p[1]]
							knownPos[unknowns[j]] = true
							//#print("set %s to %r" % (map1.points[unknowns[j]].name, p))
							j += 1
							if(j >= unknowns.size()) {
								break
							} //#end if
						else if(grid[p[1]][p[0]] == NULL) {
							//#print(map1.points[unknowns[j]].name, end=' ')
							//#print('in spot')
							//# available, put it there.
							grid[p[1]][p[0]] = points[unknowns[j]]
							pointPosMap[unknowns[j]] = [p[0], p[1]]
							knownPos[unknowns[j]] = true
							//#print("set %s to %r" % (map1.points[unknowns[j]].name, p))
							j += 1
							if(j >= unknowns.size()) {
								break
							} //#end if
						} //#end if
						//# Fix the pointPosMap using generated dp. 
						//# TODO: do this after the loop for optimization.
						if(dp and dp != [0, 0]) {
							//#print("Fix ppm using", dp)
							for(v in pointPosMap) {
								pointPosMap[v][0] += dp[0]
								pointPosMap[v][1] += dp[1]
							} //#next v
							dp = NULL
						} //#end if
					} //#next p
					//# Fix the pointPosMap using generated dp. 
					//# NOTE: this is a minor hack. See above the same code. This is here because of the breaks.
					if(dp and dp != [0, 0]) {
						//#print("Fix ppm 2 using", dp)
						for(v in pointPosMap) {
							pointPosMap[v][0] += dp[0]
							pointPosMap[v][1] += dp[1]
						} //#next v
					} //#end if
					//#print(pointPosMap)
				} else {
					//# Deal with other things later
					pass
				} //#end if
			} //#next i
			if(not (false in knownPos)) {
				break
			} //#end if
		} //#next roundTry
		print("%i rounds" % (roundTry + 1))
		if(false in knownPos) {
			//#print(knownPos)
			//#print([points[i].name for i in range(knownPos.size()) if not knownPos[i]])
			//# TODO: Try to put the unset nodes in slots that are empty near their connections.
			for(i = 0; i < knownPos.size()); i++) {
				if(knownPos[i]) { continue
				foundPos = false
				for(y = 0; y < grid.size()); y++) {
					for(x = 0; x < len(grid[0])); x++) {
						if(grid[y][x] == NULL) {
							grid[y][x] = points[i]
							pointPosMap[i] = [x, y]
							//#print("set %s to %r" % (map1.points[i].name, (x, y)))
							foundPos = true
							break
						} //#end if
					} //#next x
					if(foundPos) { break
				} //#next y
				if(not foundPos) {
					//# We need to extend, which is almost never. euler's identity repros this.
					grid.push_back([NULL] * grid_cols)
					grid_rows += 1
					grid[-1][0] = points[i]
					pointPosMap[i] = [grid_rows - 1, 0]
				} //#end if
			} //#next i
		} //#end if
		
		if(rearrange and pointPosMap.size() < points.size()) {
			print("Error: pointPosMap is broken, we can't rearrange.")
			rearrange = false
		} //#end if
		if(rearrange) {
			//# Improve the potential dramatically at a very high cost.
			//#for(i = 0; i < 10); i++) {
			//#	if(not rearrangeGrid(map1, grid, pointPosMap, 40)) {
			//#		break
			//#	} //#end if
			} //#next i
			self.rearrangeGrid(grid, pointPosMap, 40)
		} //#end if
		//# Then set the points' positions.
		gridToPos(grid, width)
	} //#end def sortGrid(width=40, rearrange=true)
	
	def rearrangeGrid(self, grid, pointPosMap, width=40, nonConns=NULL):
		"""
		Reduce potential using algorithm designed last night.
		TODO: auto loop sort of groups on the grid
		Let's solve grid1_128331410_1009.json
		Repro:
		python3 grid1.py -n 16 -c 2 -b 0 -x 3033139000
		Returns true on improvement, false when there is no improvement.
		You do not need to run gridToPos before or after rearrangeGrid, but if you 
		don't and rearrangeGrid returns false, then you will have map1 back as it 
		was, thus the code at the bottom of sortGrid is _correct_ so long as this 
		is true.
		The only bug to be fixed for this graph is adding another row when necessary and putting a node there.
		This algorithm doesn't attempt that.
		FIXME: This is slow even with caching. We must do whatever we can to 
		improve the speed of this as well as deciding not to use it.
		"""
		//# TODO: a strange circular dependency, not good, eventually just incorporate.
		global V_Harmonic, V_Harmonic_Angle
		from sww_optimize1 import LJ_Potential, V_SmallWideWorld, V_Harmonic, V_Harmonic_Angle
		b0 = 40
		lj_conf = LJ_Potential()
		gridToPos(grid, width)
		x0, conns = self.getX0Conns()
		if(nonConns is NULL) {
			nonConns = getNonConns(conns, true)
		} //#end if
		V_init = V_SmallWideWorld(x0, conns, b0, lj_conf, nonConns)
		//#print("V_init:", V_init)
		nodes = points.size()
		minV = V_init
		x_out = self.getX0Conns()[0]
		best_x = x_out
		ret = false
		
		for(roundTry = 0; roundTry < 10); roundTry++) {
			madeImprovement = false
			for(i = 0; i < nodes); i++) {
				//# The cache is made up of the potential minus this node's contribution.
				V_cache = PotentialCache(0, i)
				V_node_i = V_SmallWideWorld_Cache(x_out, conns, b0, lj_conf, nonConns, V_cache)
				V_cache.PotentialA = minV - V_node_i
				pos_i = pointPosMap[i]
				for(conn in points[i].conns) {
					j = points.index(conn)
					pos_j = pointPosMap[j]
					dx = pos_i[0] - pos_j[0]
					dy = pos_i[1] - pos_j[1]
					dr_sq = (dx * dx) + (dy * dy)
					if(dr_sq <= 1) {
						//# Don't worry about this bond. It's fine.
						continue
					} //#end if
					//# Get a list of positions we can pick for this one...
					possiblePos = getOptionsList(pos_i, pos_j)
					//#print(pos_i, pos_j, possiblePos)
					
					best_pos = NULL
					valid_positions = 0
					for(pos in possiblePos) {
						if(pos[1] >= grid.size() or pos[0] >= len(grid[0])) { continue
						if(pos[1] < 0 or pos[0] < 0) { continue
						//# Don't put two in the same place.
						if(grid[pos[1]][pos[0]] != NULL) { continue
						valid_positions += 1
						x_out[2 * i] = width * pos[0]
						x_out[2 * i + 1] = width * pos[1]
						//#V_i = V_SmallWideWorld(x_out, conns, b0, lj_conf, nonConns)
						V_i = V_SmallWideWorld_Cache(x_out, conns, b0, lj_conf, nonConns, V_cache)
						if(V_i < minV) {
							//#print("Improved V_i: %f when %s moved to %r" % (V_i, map1.points[i].name, pos))
							minV = V_i
							best_x = x_out[:]
							best_pos = pos
						} //#end if
					} //#next pos
					dx = abs(pos_i[0] - pos_j[0])
					dy = abs(pos_i[1] - pos_j[1])
					if(valid_positions == 0) {
						possiblePos = []
						if(dx == 0) {
							//# Try above and below, assuming we can.
							possiblePos = [[pos_j[0] - 1, pos_j[1]], [pos_j[0] + 1, pos_j[1]]]
							if(pos_j[0] == 0) {
								possiblePos.pop(0)
								//#print("Can't go up.")
							else if(pos_j[0] >= grid.size()) {
								possiblePos.pop(1)
								//#print("Can't go down.")
							} //#end if
						} //#end if
						if(dy == 0) {
							//# Try above and below, assuming we can.
							possiblePos = [[pos_j[0], pos_j[1] - 1], [pos_j[0], pos_j[1] + 1]]
							if(pos_j[1] == 0) {
								possiblePos.pop(0)
								//#print("Can't go up.")
							else if(pos_j[1] >= grid.size()) {
								possiblePos.pop(1)
								//#print("Can't go down.")
							} //#end if
						} //#end if
						//#print('Trying %s with' % points[i].name, possiblePos, pos_i, pos_j)
						for(pos in possiblePos) {
							if(grid.size() <= pos[1]) { continue
							if(len(grid[0]) <= pos[0]) { continue
							//# Don't put two in the same place.
							if(grid[pos[1]][pos[0]] != NULL) { continue
							valid_positions += 1
							x_out[2 * i] = width * pos[0]
							x_out[2 * i + 1] = width * pos[1]
							//#V_i = V_SmallWideWorld(x_out, conns, b0, lj_conf, nonConns)
							V_i = V_SmallWideWorld_Cache(x_out, conns, b0, lj_conf, nonConns, V_cache)
							if(V_i < minV) {
								//#print("Improved V_i: %f when %s moved to %r" % (V_i, map1.points[i].name, pos))
								minV = V_i
								best_x = x_out[:]
								best_pos = pos
							} //#end if
						} //#next pos
						//#print("Got %i valid positions." % valid_positions)
					} //#end if
					if(best_pos is not NULL) {
						//# Change the map
						m = np.matrixlib.matrix(best_x)
						m = m.reshape(points.size(), 2)
						self.setPos([np.array(xa)[0] for xa in m])
						//# Change the grid
						//#print("name: %s pos %r" % (map1.points[i].name, pointPosMap[i]))
						//#print('removing %s from grid pos %r' % (grid[pointPosMap[i][1]][pointPosMap[i][0]].name, pointPosMap[i]))
						grid[pointPosMap[i][1]][pointPosMap[i][0]] = NULL
						pointPosMap[i] = best_pos
						grid[best_pos[1]][best_pos[0]] = points[i]
						//#print('added %s from grid pos %r' % (grid[best_pos[1]][best_pos[0]].name, best_pos))
						ret = true
						madeImprovement = true
					} else {
						//# Reset x_out
						x_out = best_x
					} //#end if
				} //#next conn
			} //#next i
			//#print("mi", madeImprovement)
			if(madeImprovement == false) { break
		} //#next roundTry
		if(ret) { print("Improvement: %3.3f" % (V_init - minV))
		return ret
	} //#end def rearrangeGrid(grid, pointPosMap, width=40, nonConns=NULL)
	
	def sortBest(self, width=40, rearrange=true):
		"""
		sortGrid, sortBranch, and sortLoop, pick whichever has lowest potential.
		"""
		from sww_optimize1 import LJ_Potential, V_SmallWideWorld, V_Harmonic, V_Harmonic_Angle
		lj_conf = LJ_Potential()
		x0, conns = self.getX0Conns()
		//#print(x0)
		//#if(verbose) {
		//#	print("atom conns:", conns)
		} //#end if
		nonConns = getNonConns(conns, true)

		//# Not necessary to get spherical coordinates of the original coordinates.
		//#output = [spherical2.cartesianToSpherical(v) for v in molecule.x0]
		//#print("Spherical coordinates of atoms:")
		//#print(output)
		
		//# This may be used eventually
		//#sphericalCoords, atom_name_hash = molecule.toSpherical(verbose)
		self.sortBranch(width)
		x_out = self.getX0Conns()[0]
		V_branch = V_SmallWideWorld(x_out, conns, width, lj_conf, nonConns)

		self.sortLinear(width)
		self.sortLoop(width)
		x_out = self.getX0Conns()[0]
		V_loop = V_SmallWideWorld(x_out, conns, width, lj_conf, nonConns)
		
		dV = V_branch - V_loop
		if(dV < 0) {
			bestSort = 'branch'
			V_best = V_branch
		} else {
			bestSort = 'loop'
			V_best = V_loop
		} //#end if

		self.sortLattice(width)
		x_out = self.getX0Conns()[0]
		V_lattice = V_SmallWideWorld(x_out, conns, width, lj_conf, nonConns)
		if(V_best > V_lattice) {
			bestSort = 'lattice'
			V_best = V_lattice
		} //#end if
		
		self.sortSpiral(width)
		x_out = self.getX0Conns()[0]
		V_spiral = V_SmallWideWorld(x_out, conns, width, lj_conf, nonConns)
		if(V_best > V_spiral) {
			bestSort = 'spiral'
			V_best = V_spiral
		} //#end if
		
		self.sortGrid(width, rearrange)
		x_out = self.getX0Conns()[0]
		V_grid = V_SmallWideWorld(x_out, conns, width, lj_conf, nonConns)
		//#print("Branch Potential:", V_branch)
		//#print("Grid dt: %3.3f seconds" % grid_dt)
		//#print("Grid Potential:", V_grid)
		dV = (V_best - V_grid)
		//#print("Branch - Grid: %3.3f" % dV)
		if(dV < 0) {
			//#print("Note: Branch is lower, see %s" % base_filename)
			if(bestSort == 'loop') {
				self.sortLinear(width)
				self.sortLoop(width)
			else if(bestSort == 'lattice') {
				self.sortLattice(width)
			else if(bestSort == 'spiral') {
				self.sortSpiral(width)
			} else {
				self.sortBranch(width)
			} //#end if
		} //#end if
		self.centerBound()
	} //#end def sortBest(width=40, rearrange=true)

	def findRoute(self, p, q, avoid=NULL, level=0):
		"""
		Given two connected Points, find a route not directly connecting the two.
		Returns a list of Points or NULL in case of failure.
		A* would rock, but I'm just a hacker, not a computer scientist. j/k
		"""
		if(avoid == NULL) {
			avoid = [p]
		} else {
			avoid = [p] + avoid
		} //#end if
		foundRoute = NULL
		for(conn in p.conns) {
			//# We don't want to go backwards, infinite recursion.
			if(conn in avoid) { continue
			if(conn != q) {
				v = self.findRoute(conn, q, avoid, level + 1)
				if(v) { 
					//#print('found route', ', '.join([str(x) for x in v]))
					foundRoute = [p] + v
				} //#end if
			} else {
				//# We got a path, but we might not want it.
				//# Check if we just started.
				if(level > 0) {
					return [p, q]
				else if(foundRoute == NULL) {
					//# TODO: Verify this somehow...
					foundRoute = [q]
				} //#end if
			} //#end if
		} //#next conn
		return foundRoute
	} //#end def findRoute(p, q, avoid=NULL)
	
	def squash(self):
		"""
		FIXME: Broken function.
		"""
		mean = Vector2(0,0)
		for(point in points) {
			//#mean[0] += point.x
			//#mean[1] += point.y
			mean += point
		} //#next point
		//#mean[0] /= (1.0 * points.size())
		//#mean[1] /= (1.0 * points.size())
		mean /= (1.0 * points.size())
		//#goto = [self.width/2.0, self.height/2.0]
		goto = Vector2(self.width/2.0, self.height/2.0)
		//#diff = [goto[0]-mean[0], goto[1]-mean[1]]
		diff = goto - mean
		for(point in points) {
			//#point.x += diff[0]
			//#point.y += diff[1]
			for(conn in point.conns) {
				//# FIXME: This doesn't make any sense.
				point.x += diff.x
				point.y += diff.y
			} //#next conn
		} //#next point
	} //#end def squash()
	
	def split(self):
		"""
		Return a list of independent graphs.
		"""
		output = [set() for p in points]
		revMap = {}
		for(i = 0; i < points.size()); i++) {
			revMap[points[i]] = i
		} //#next i
		//# First step: merge all connections together.
		for(i = 0; i < points.size()); i++) {
			ourSlot = NULL
			for(conn in points[i].conns) {
				j = revMap[conn]
				if(len(output[j]) > 0) {
					if(ourSlot != NULL) {
						//# Incorporate their useless map.
						output[ourSlot].update(output[j])
						output[j] = set()
					} else {
						ourSlot = j
					} //#end if
					//#print(points[i].name, 'going to', points[j].name)
				} //#end if
			} //#next conn
			if(ourSlot == NULL) {
				output[i].update([points[i]] + points[i].conns)
			} else {
				output[ourSlot].update([points[i]] + points[i].conns)
			} //#end if
		} //#next i
		//# this leaves us with at most a handful of groups that need to be merged.
		//# Look for things to merge.
		for(i = 0; i < points.size()); i++) {
			if(len(output[i]) > 0) {
				toMerge = []
				for(p in output[i]) {
					//#j = revMap[p]
					for(k = 0; k < points.size()); k++) {
						//# Don't try to merge with yourself.
						if(i == k) { continue
						if(k in toMerge) { continue
						if(p in output[k]) {
							//# Incorporate their useless map.
							toMerge.push_back(k)
						} //#end if
					} //#next k
				} //#next p
				for(k in toMerge) {
					output[i].update(output[k])
					output[k] = set()
				} //#next k
			} //#end if
		} //#next i
		while set() in output:
			output.remove(set())
		} //#loop
		return output
	} //#end def split()

	def loadJson(self, jsonData):
		"""
		load JSON string
		Difficult because of the design, but it works.
		No error handling, this should be silly.
		"""
		//# TODO: Decide if this needs to be a wider dependency.
		import json
		try:
			x = json.loads(jsonData)
		except ValueError as ve:
			//#print("Error: Invalid JSON", ve)
			raise Exception("Invalid JSON: " + repr(ve))
		//#end try
		nameObjMap = {}
		for(n in x['nodes']) {
			//# n is in the format [name, x, y]
			nameObjMap[n[0]] = points.size()
			points.push_back(Point(n[0], NULL, n[1], n[2]))
		} //#next n
		for(c in x['conns']) {
			//# c in in the format [name, name]
			objIndex1 = nameObjMap[c[0]]
			objIndex2 = nameObjMap[c[1]]
			//# Dupes easily.
			if(points[objIndex2] in points[objIndex1].conns) { continue
			points[objIndex1].addConn(points[objIndex2])
		} //#next c
	} //#end def loadJson(self, jsonData)

	def toPolar(self, verbose=false):
		"""
		Convert cartesian coordinates to polar dr coordinates. This 
		means that for each atom, we subtract a neighboring atom's 
		position and then convert to polar coordinates. This gives 
		a repeatable method of showing relative angles and bond 
		lengths. To ensure that you have the correct drs in the correct 
		order, ensure that atom_name_hash is the same for the same 
		molecule. If the hash is different, that means you have a 
		different list of polar dr and so they cannot be compared.
		
		Returns:
		[(polar_dr), ...], atom_name_hash
		
		To get the hex digest from atom_name_hash, use atom_name_hash.hexdigest()
		To get the binary digest, use atom_name_hash.digest()
		"""
		dr = [(0, 0)]
		atom_name_hash = sha256()
		atom_name_data = ''
		//# We need sorted connections to fix a strange bug related to export/import, so I choose name.
		for(p in points) {
			p.conns.sort(key=lambda x: x.name)
		} //#next p
		//# Make sure they don't use the reverse like N3 -- CT, CT -- N3 because 
		//# that produces no useful data.
		reverses = [(0, 0)]
		known = [0]
		unconns = []
		for(round_n = 0; round_n < 1000); round_n++) {
			//#print('round_n', round_n)
			for(i, point in enumerate(points)) {
				if(i in known) { continue
				//# This algorithm requires each molecule to order its atoms in 
				//# the same way. We hash the names to ensure that the data is 
				//# the same.
				atom_name = point.name
				dr_x = NULL
				i_x = NULL
				for(x in point.conns) {
					i_x_poss = points.index(x)
					if(i_x_poss not in known) { continue
					//#print('I think %i is okay' % i_x_poss)
					i_x = i_x_poss
					if(x.conns.size() > 1) {
						if((i_x, i) not in reverses) {
							//# We found one I think...
							break
						} //#end if
					} //#end if
				} //#next x
				if(point.conns.size() == 0) {
					//# Unconnected atom
					unconns.push_back(point)
					continue
				} //#end if
				if(i_x == NULL) { continue
				dr_x = point - points[i_x]
				dr.push_back(dr_x)
				//#print('hi2', i)
				atom2_name = points[i_x].name
				atom_name_hash.update((atom_name + ('[%i]' % i) + "--" + atom2_name + ('[%i]' % i_x)).encode('utf-8'))
				atom_name_data += (atom_name + "--" + atom2_name) + '\n'
				reverses.push_back((i, i_x))
				known.push_back(i)
	        } //#next i, atom
			//# Connect unconnect atoms in a row to the last known.
			if(unconns.size() > 0 and known.size() == 0) {
				known.push_back(points.index(unconns[0]))
			} //#end if
			for(unc in unconns) {
				i = points.index(unc)
				i_x = known[-1]
				reverses.push_back((i, i_x))
				known.push_back(i)
			} //#next unc
			
			//# We are done when all have connections.
			if(reverses.size() >= points.size()) { break
		} //#next round_n
		if(dr.size() != points.size()) {
			print("Error: toPolar failed, expect failure elsewhere.", dr.size(), points.size())
		} //#end if
		//#print(atom_name_data)
		return [polar1.cartesianToPolar(dv) for dv in dr], atom_name_hash
	} //#end def toPolar(verbose=false)

*/

Reverse::Reverse(int a_init, int parent_init)
{
	a = a_init;
	parent = parent_init;
}
Reverse::~Reverse()
{
}

Connectivity::Connectivity()
{
}
Connectivity::~Connectivity()
{
}

/*
 * A very common piece of code
 * points.index(x)
 */
int pointIndex(Points *points, Point *x)
{
	return x - &(*(points->begin()));
}

int findReverse(Connectivity *reverses, int a, int parent)
{
	Connectivity::iterator r;
	int i = 0;
	for(r = reverses->begin(); r != reverses->end(); ++r, i++)
	{
		if((*r).a == a && (*r).parent == parent)
		{
			return i;
		}
	}
	return -1;
}

Connectivity Map::getPolarConn()
{
	/*"""
	Same code as toPolar sadly can't fix that.
	returns a list of index pairs [(i, j), ...] and an atom_name_hash object.
	"""*/
	//#dr = []
	// TODO: SHA256
	//atom_name_hash = sha256();
	string atom_name_data = "";
	//# We need sorted connections to fix a strange bug related to export/import, so I choose name.
	// TODO: Sort
	/*for(p in points) {
		p.conns.sort(key=lambda x: x.name)
	} //#next p*/
	//# Make sure they don't use the reverse like N3 -- CT, CT -- N3 because 
	//# that produces no useful data.
	int knownsNotIncremented = 0;
	Connectivity reverses; //= [(0, 0)]
	reverses.push_back(Reverse(0, 0));
	std::vector<int> known;
	known.push_back(0);
	Pointps unconns;
	size_t i;
	int i_x;
	size_t round_n;
	for(round_n = 0; round_n < (points.size() * 2); round_n++) {
		bool knownsIncremented = false;
		for(i = 0; i < points.size(); i++) {
			if(std::find(known.begin(), known.end(), i) != known.end()) { continue; }
			//# This algorithm requires each molecule to order its atoms in 
			//# the same way. We hash the names to ensure that the data is 
			//# the same.
			int dr_x = 0;
			i_x = -1;
			Pointps::iterator x;
			for(x = points[i].conns.begin(); x != points[i].conns.end(); x++) {
				//# No need for dr_x, just set to true.
				dr_x = 1; //#point - x
				int i_x_poss = pointIndex(&points, *x);
				// if(i_x_poss not in known)
				if(std::find(known.begin(), known.end(), i_x_poss) == known.end()) { continue; }
				i_x = i_x_poss;
				if((*x)->conns.size() > 1) {
					//if((i_x, i) not in reverses) {
					// TODO: Test this very odd code.
					if(findReverse(&reverses, i_x, i) == -1) {
						//# We found one I think...
						std::cout << "Yay" << std::endl;
						break;
					} //#end if
					std::cout << "Nope" << std::endl;
				} //#end if
			} //#next x
			if(dr_x == 0) {
				//# Unconnected atom
				unconns.push_back(&points[i]);
				continue;
			} //#end if
			if(i_x == -1) { continue; }
			//#dr.push_back(dr_x)
			//atom_name_hash.update((points[i].name + ('[%i]' % i) + "--" + points[i_x].name + ('[%i]' % i_x)).encode('utf-8'));
			atom_name_data += (points[i].name + "--" + points[i_x].name) + '\n';
			//#reverses.push_back((point, x))
			reverses.push_back(Reverse(i, i_x));
			known.push_back(i);
			knownsIncremented = true;
		} //#next i, atom
		
		//# Connect unconnect atoms in a row to the last known.
		if((unconns.size() > 0) && (known.size() == 0)) {
			known.push_back(pointIndex(&points, unconns[0]));
		} //#end if
		Pointps::iterator unc;
		for(unc = unconns.begin(); unc != unconns.end(); unc++) {
			i = pointIndex(&points, *unc);
			i_x = known[known.size() - 1];
			reverses.push_back(Reverse(i, i_x));
			known.push_back(i);
		} //#next unc
		
		//# We are done when all have connections.
		if(reverses.size() >= points.size()) { break; }
		
		//# We are in trouble if knowns didn't increase. That means two separate structures.
		if(!knownsIncremented) {
			//#print('not knownsIncremented on round', round_n)
			knownsNotIncremented++;
			if(knownsNotIncremented >= 2) {
				//#print('knownsNotIncremented >= 2 on round', round_n)
				for(i = 0; i < points.size(); i++) {
					//if(i not in known) {
					if(std::find(known.begin(), known.end(), i) == known.end()) {
						//# Connect it to the last known.
						reverses.push_back(Reverse(known[known.size() - 1], i));
						known.push_back(i);
						//# Reset the counter.
						knownsNotIncremented = 0;
						break;
					} //#end if
				} //#next i
				//#break
			} //#end if
		} else {
			knownsNotIncremented = 0;
		} //#end if
	} //#next round_n
	//#print(round_n+1, "rounds to do getPolarConn")
	if(reverses.size() != points.size()) {
		std::cout << "Error: getPolarConn failed, expect failure elsewhere." << std::endl;
		// TODO: 
		//print(reverses)
		std::cout << atom_name_data << std::endl;
	} //#end if
	//#print(atom_name_data)
	// TODO: Use hash instead of data obviously...
	reverses.anh = atom_name_data; //atom_name_hash;
	return reverses;
} //#end def getPolarConn()

void Map::setCartesian(std::vector<PolarVector> *dr)
{
	/*"""
	Unlike the name suggests, we are not setting the atom's 
	positions to a list of cartesian coordinates.
	dr is a list of relative polar coordinates.
	"""*/
	if(dr->size() != points.size()) {
		cout << "Not right number of dr. " << dr->size() << " " << points.size() << std::endl;
		return;
	} //#end if
	
	//# The format of drs is [(atom_0_index, atom_i_index), ...]
	//# For example methane is [(0, 1), (1, 0), (2, 0), (3, 0), (4, 0)]
	//# That is to say carbon is relative to hydrogen1.
	//# hydrogen1 is relative to carbon (circular I know, necessary)
	//# hydrogen2 is relative to carbon.
	//# ...
	Connectivity drs = getPolarConn();
	//#print(atom_hash.hexdigest())
	
	//# The first one is always our starting point.
	//# We could set it to zero, but let's leave it where it is.
	//# We know the position of drs[0][0].
	std::vector<int> knowns;
	knowns.push_back(drs[0].a);
	//#print(points[drs[0][0]].name, 'assumed')
	//#self.atoms[drs[0][0]].pos = self.atoms[drs[0][0]].pos
	size_t i;
	size_t j = 0;
	while (knowns.size() < points.size()) {
		//# i starts at 0 and goes to N like it should.
		for(i = 0; i < drs.size(); i++) {
			Reverse &spc = drs[i];
			if(std::find(knowns.begin(), knowns.end(), spc.a) != knowns.end()) {
				if(std::find(knowns.begin(), knowns.end(), spc.parent) != knowns.end()) {
					//# We know both, so skip this dr...
					continue;
				} //#end if
				//#print(points[spc.parent].name, 'relative to', points[spc.a].name, dr[i])
				knowns.push_back(spc.parent);
				points[spc.parent].setPos(points[spc.a] + \
					polarToCartesian(&((*dr)[i])));
			} else if(std::find(knowns.begin(), knowns.end(), spc.parent) != knowns.end()) {
				//# spc.a not in knowns
				//#print(points[spc.a].name, 'relative to', points[spc.parent].name, dr[i])
				knowns.push_back(spc.a);
				points[spc.a].setPos(points[spc.parent] + \
					polarToCartesian(&((*dr)[i])));
			} //#end if
		} //#next i, spc
		j++;
		if(j > 1000) {
			// << " " << knowns
			std::cout << "rounds has passed 1000, we give up. " << knowns.size() << " " << points.size() << std::endl;
			//std::cout << drs << std::endl;
			break;
		} //#end if
	} //#loop
} //#end def setCartesian(dr)


Rect Map::bounds()
{
	/*"""
	Find the maximum extents.
	returns (x_min, y_min, x_max, y_max)
	"""*/
	if(points.size() == 0)
	{
		return Rect();
	} //#end if
	//# x_min, y_min, x_max, y_max
	Vector2 minp = points[0];
	Vector2 maxp = minp;
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		// Is min()/max() faster?
		if(points[i].x < minp.x) minp.x = points[i].x;
		if(points[i].y < minp.y) minp.y = points[i].y;
		if(points[i].x > maxp.x) maxp.x = points[i].x;
		if(points[i].y > maxp.y) maxp.y = points[i].y;
	} //#next point
	return Rect(minp.x, minp.y, maxp.x, maxp.y);
} //#end def bounds()

void Map::center()
{
	Vector2 mean = Vector2(0,0);
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		//#mean[0] += point.x
		//#mean[1] += point.y
		mean += points[i];
	}
	//#next point
	//#mean[0] /= (1.0 * len(self.points))
	//#mean[1] /= (1.0 * len(self.points))
	mean /= (1.0 * points.size());
	//#goto = [self.width/2.0, self.height/2.0]
	Vector2 goto_pos = Vector2((int)(width / 2.0), (int)(height / 2.0));
	//#diff = [goto[0]-mean[0], goto[1]-mean[1]]
	Vector2 diff = goto_pos - mean;
	//#print 'center diff', diff
	for(i = 0; i < points.size(); i++)
	{
		//#point.x += diff[0]
		//#point.y += diff[1]
		points[i].x += diff.x;
		points[i].y += diff.y;
	}
	//#next point
}
//#end void center()

void Map::centerBound()
{
	/*"""
	Instead of averaging, this puts the center based on the mean of the 
	bounds. This works with pretty much all graphs.
	"""*/
	if(points.size() == 0)
	{
		return;
	} //#end if
	Rect map_bounds = bounds();
	Vector2 mean = (Vector2(map_bounds.min_x, map_bounds.min_y) + Vector2(map_bounds.max_x, map_bounds.max_y)) / 2.0;
	Vector2 goto_pos = Vector2((int)(width / 2.0), (int)(height / 2.0));
	Vector2 diff = goto_pos - mean;
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		points[i].x += diff.x;
		points[i].y += diff.y;
	}
}//#end def centerBound()

void Map::squash()
{
	Vector2 mean = Vector2(0, 0);
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		//#mean[0] += point.x
		//#mean[1] += point.y
		mean += points[i];
	}
	//#next point
	//#mean[0] /= (1.0 * len(self.points))
	//#mean[1] /= (1.0 * len(self.points))
	mean /= (1.0 * points.size());
	//#goto = [self.width/2.0, self.height/2.0]
	Vector2 goto_pos = Vector2((int)(width/2.0), (int)(height/2.0));
	//#diff = [goto[0]-mean[0], goto[1]-mean[1]]
	Vector2 diff = goto_pos - mean;
	for(i = 0; i < points.size(); i++)
	{
		//#point.x += diff[0]
		//#point.y += diff[1]
		// Note that this is a bug in center, but I'm leaving it in as squash.
		size_t j;
		for(j = 0; j < points[i].conns.size(); j++)
		{
			points[i].x += diff.x;
			points[i].y += diff.y;
		}
		//#next conn
	}
	//#next point
}
//#end void squash()
	
void Map::move(float x, float y)
{
	/*"""
	Move everything on the map to offset x, y.
	"""*/
	if (points.size() == 0)
	{
		return;
	}
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		points[i].x += x;
		points[i].y += y;
	} //#next point
} //#end def move(x, y)

string Map::str()
{
	return drawSvg();
}
//#void __str__(self):

string itostr(int a)
{
	string o;
	stringstream s;
	s << a;
	s >> o;
	return o;
}

string ftostr(float a)
{
	string o;
	stringstream s;
	s << a;
	s >> o;
	return o;
}

string itohex(int a, int width)
{
	string o;
	stringstream s;
	s << std::hex << std::setfill('0');
	s << std::setw(width) << a;
	s >> o;
	return o;
}

string str_replace(const std::string& subject,
					const std::string& search,
					const std::string& replace)
{
	std::string str = subject;
	size_t pos = 0;
	while((pos = str.find(search, pos)) != std::string::npos)
	{
		str.replace(pos, search.length(), replace);
		pos += replace.length();
	}
	return str;
}

string Map::drawSvg()
{
	string output = "<?xml version=\"1.0\" standalone=\"no\"?>\n"
		"<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">";
	output += "\n<svg width=\"" + itostr(width) + "\" height=\"" + itostr(height) + "\" viewBox=\"0 0 " + itostr(width) + " " + itostr(height) + "\" xmlns=\"http://www.w3.org/2000/svg\" version=\"1.1\">\n";
	// XML entity encode.
	output += "<title>" + str_replace(name, "&", "&amp;") + "</title>\n";
	output += "<desc>" + str_replace(desc, "&", "&amp;") + "</desc>\n";
	output += "<rect x=\"1\" y=\"1\" width=\"" + itostr(width-2) + "\" height=\"" + itostr(height-2) + "\" fill=\"none\" stroke=\"blue\" />\n";
	
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		output += points[i].drawSvg(line_width);
	}
	//#next point
	//# A layer of text above the connections.
	output += "<g>\n";
	for(i = 0; i < points.size(); i++)
	{
		Vector2 tpos = points[i] + Vector2(8, 8);
		if (points[i].url != "")
		{
			// url.replace("&", "&amp;") is awful in C++.
			output += "<a xlink:href=\"" + str_replace(points[i].url, "&", "&amp;") + "\">";
		} //#end if
		output += "<text x=\"" + itostr(tpos.x) + "\" y=\"" + itostr(tpos.y) + "\">" + points[i].name + "</text>\n";
		//output += "<text x='" + ('%3.3f' % tpos.x) + "' y='" + ('%3.3f' % tpos.y) + "'>" + point.name + "</text>\n";
		if (points[i].url != "")
		{
			output += "</a>\n";
		}
	}
	//#next point
	output += "</g>\n";
	output += "</svg>\n";
	return output;
}
// #end void drawSvg()

/*
 * json.dumps in a few lines.
 * TODO: Unicode...
 * json.dumps(''.join([chr(i) for i in range(256)]))
 * "\u0000\u0001\u0002\u0003\u0004\u0005\u0006\u0007\b\t\n\u000b\f\r\u000e\u000f\u0010\u0011\u0012\u0013\u0014\u0015\u0016\u0017\u0018\u0019\u001a\u001b\u001c\u001d\u001e\u001f !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~\u007f\u0080\u0081\u0082\u0083\u0084\u0085\u0086\u0087\u0088\u0089\u008a\u008b\u008c\u008d\u008e\u008f\u0090\u0091\u0092\u0093\u0094\u0095\u0096\u0097\u0098\u0099\u009a\u009b\u009c\u009d\u009e\u009f\u00a0\u00a1\u00a2\u00a3\u00a4\u00a5\u00a6\u00a7\u00a8\u00a9\u00aa\u00ab\u00ac\u00ad\u00ae\u00af\u00b0\u00b1\u00b2\u00b3\u00b4\u00b5\u00b6\u00b7\u00b8\u00b9\u00ba\u00bb\u00bc\u00bd\u00be\u00bf\u00c0\u00c1\u00c2\u00c3\u00c4\u00c5\u00c6\u00c7\u00c8\u00c9\u00ca\u00cb\u00cc\u00cd\u00ce\u00cf\u00d0\u00d1\u00d2\u00d3\u00d4\u00d5\u00d6\u00d7\u00d8\u00d9\u00da\u00db\u00dc\u00dd\u00de\u00df\u00e0\u00e1\u00e2\u00e3\u00e4\u00e5\u00e6\u00e7\u00e8\u00e9\u00ea\u00eb\u00ec\u00ed\u00ee\u00ef\u00f0\u00f1\u00f2\u00f3\u00f4\u00f5\u00f6\u00f7\u00f8\u00f9\u00fa\u00fb\u00fc\u00fd\u00fe\u00ff"
 */
//const char hex[] = "0123456789abcdef";
string json_dumps(string s)
{
	string output = "\"";
	size_t i;
	for(i = 0; i < s.length(); i++)
	{
		unsigned char v = s[i];
		//cout << "v: " << v << ", " << (int)v << endl;
		if(v < 0x20) {
			// Dirty hex conversion =)
			output += "\\u00" + itohex(v, 2); //(hex[(v & 0xf0) >> 4]) + (hex[v & 0xf]);
			continue;
		}
		if(v == '\"' || v == '\\')
		{
			output += "\\";
		}
		output += v;
	}
	return output + "\"";
}

string Map::saveJson()
{
	/*"""
	save JSON output
	Difficult because of the design, but it works.
	We use float instead of integer representation because the potential 
	energy depends on precise placement. If you want integer, modify this 
	function.
	"""*/
	//# TODO: Decide if this needs to be a wider dependency.
	//import json
	string desc_out = "";
	if(desc != "") {
		desc_out = "\"desc\":" + json_dumps(desc) + ", ";
	} //#end if
	string name_out = "";
	if(name != "") {
		name_out = "\"name\":" + json_dumps(name) + ", ";
	} //#end if
	string output = "{" + name_out + desc_out + "\"nodes\":[";
	string conns;
	std::set<Point *> alreadyConn;
	size_t i;
	//for(point in points) {
	for(i = 0; i < points.size(); i++)
	{
		output += "[" + json_dumps(points[i].name) + ", " + ftostr(points[i].x) + ", " + ftostr(points[i].y) + "], ";
		Pointps::iterator conn;
		for(conn = points[i].conns.begin(); conn != points[i].conns.end(); conn++) {
			//# Don't add a duplicate connection.
			//# NOTE: This assumes dual connection, but that MUST be true.
			//# TODO: A better way to do this is i < j
			if(alreadyConn.find(*conn) != alreadyConn.end()) { continue; }
			conns += "[" + json_dumps(points[i].name) + ", " + json_dumps((*conn)->name) + "], ";
		} //#next conn
		alreadyConn.insert(&points[i]);
	} //#next point
	//# Strip comma.
	output = output.substr(0, output.length() - 2);
	output += "], \"conns\":[" + conns.substr(0, conns.length() - 2);
	//# Add end bracket.
	output += "]}";
	return output;
} //#end def saveJson(self)

bool Map::setPos(float *v, size_t v_len)
{
	/*"""
	Take an x0 properly reshaped and load into points.
	Example:
	m = np.matrixlib.matrix(x0)
	m = m.reshape(len(x0)//2, 2)
	map1.setPos([np.array(xa)[0] for xa in m])
	"""*/
	// Debug code.
	if(v_len > 2 * points.size())
	{
		return false;
	}
	size_t i;
	//for i, xa in enumerate(v):
	for(i = 0; i < v_len/2; i++)
	{
		points[i].x = v[2*i];
		points[i].y = v[2*i+1];
	} //#next i, xa
	return true;
}//#end def setPos(v)

float *Map::getX0()
{
	/*"""
	Returns x0
	x0: the 1d array of point [x, y, x, y ...] and the 
	This is used in sww_optimize1, sortLoop and other things.
	"""*/
	//# Get the xy values into a 2D array.
	float *x0 = new float[points.size()];
	
	size_t i;
	for(i = 0; i < points.size(); i++)
	{
		x0[2*i] = points[i].x;
		x0[2*i+1] = points[i].y;
	}
	return x0;
} //#end def getX0()

ConnList *Map::getConns()
{
	/*"""
	conns: the 2d array of connections [[1], [0, 2], ...]
	x0 is a numpy array, conns is a Python array.
	"""*/
	ConnList *conns = new ConnList();
	Points::iterator p;
	//for p in self.points:
	for(p = points.begin(); p != points.end(); ++p) {
		Conns pConns;
		//for conn in p.conns:
		Pointps::iterator conn;
		for(conn = (*p).conns.begin(); conn != (*p).conns.end(); ++conn) {
			Point *q = *conn;
			pConns.indices.push_back(q-&(*points.begin()));
		} //#next conn
		conns->push_back(pConns);
	} //#next p
	return conns;
} //#end def getConns()

// #end class Map

//class MapGroup:
//def __init__(self):
MapGroup::MapGroup()
{
	//self.points = []
	left_bracket = NULL;
	right_bracket = NULL;
	finished = false;
} //#end def MapGroup()

MapGroup::~MapGroup()
{}

void MapGroup::leaf(Point *p)
{
	/*"""
	Make this group a single node leaf.
	"""*/
	finished = true;
	points.clear();
	points.push_back(p);
	left_bracket = NULL;
	right_bracket = NULL;
} //#end def leaf(p)

void MapGroup::addPoint(Point *p)
{
	/*"""
	Append a point to points.
	"""*/
	points.push_back(p);
} //#end def addPoint(p)

Pointps MapGroup::cull()
{
	/*"""
	Remove any nodes that aren't part of a loop.
	For example: a-b-c-d-e-f-g, d-a. Remove e, f, and g.
	Return culled nodes. Remove nodes from self.points.
	FIXME: It doesn't work when a whole group is consumed by another group.
	"""*/
	//# Obviously unnecessary to cull groups of 0, 1, or 2.
	Pointps v;
	if (points.size() < 3) return v;
	//# TODO: Build a cache so that we can use it to be quicker.
	//cache = []
	Pointps to_cull;
	bool foundCull = true;
	while(foundCull) {
		foundCull = false;
		Pointps::iterator p;
		for(p = points.begin(); p != points.end(); ++p) {
			int conns_in_G = 0;
			Pointps::iterator q;
			//for (q in (*p)->conns) {
			for(q = (*p)->conns.begin(); q != (*p)->conns.end(); ++q) {
				if (std::find(points.begin(), points.end(), *q) != points.end()) {
					conns_in_G++;
				} //#end if
			} //#next q
			if (conns_in_G < 2) {
				to_cull.push_back(*p);
				foundCull = true;
				break;
			} //#end if
		} //#next p
		if (foundCull) {
			points.erase(to_cull.begin() + (to_cull.size() - 1));
		} //#end if
	} //#loop
	return to_cull;
} //#end def cull()

float MapGroup::minX()
{
	/*"""
	A simple method to find the x value of the minimum x value among points.
	This allows necessary sorting.
	Example:
	groups.sort(key=lambda g:g.minX())
	"""*/
	if (points.size() == 0) return 0;
	float min_x = points[0]->x;
	size_t i;
	for(i = 1; i < points.size(); i++)
	{
		if(points[i]->x < min_x) {
			min_x = points[i]->x;
		}
	}
	return min_x;
	//return min([p.x for p in self.points])
} //#end def minX()

/*MapGroup::__repr__(self):
	"""
	"""
	return '<MapGroup %r>' % ([p.name for p in self.points])
//#end def __repr__()*/

//#end class MapGroup

//class Vector2:
// x=380, y=180
Vector2::Vector2(float init_x, float init_y)
{
	x = init_x;
	y = init_y;
}

Vector2::Vector2(Point *p)
{
	x = p->x;
	y = p->y;
}

Vector2::Vector2()
{
	x = 0;
	y = 0;
}

Vector2::~Vector2()
{
}

string Vector2::str()
{
	string s;
	//('%3.3f' % self.x) + ', ' + ('%3.3f' % self.y);
	s = itostr(x) + ", " + itostr(y);
	return s;
}
//#end void __str__()

Vector2::operator string ()
{
	return str();
}

Vector2 Vector2::operator +(Vector2 v)
{
	return Vector2(x + v.x, y + v.y);
}

Vector2 Vector2::operator -(Vector2 v)
{
	return Vector2(x - v.x, y - v.y);
}

Vector2 Vector2::operator *(float f)
{
	return Vector2((x * f), (y * f));
}

Vector2 Vector2::operator /(float f)
{
	return Vector2((x / f), (y / f));
}

Vector2 &Vector2::operator +=(Vector2 v)
{
	x += v.x;
	y += v.y;
	return *this;
}

Vector2 &Vector2::operator -=(Vector2 v)
{
	x -= v.x;
	y -= v.y;
	return *this;
}

Vector2 &Vector2::operator *=(float f)
{
	x *= f;
	y *= f;
	return *this;
}

Vector2 &Vector2::operator /=(float f)
{
	x /= f;
	y /= f;
	return *this;
}

float Vector2::dot(Vector2 v)
{
	return (x * v.x) + (y * v.y);
}

//#void __iadd__(vec):
//#	v = Vector2(self.x + vec.x, self.y + vec.y)
//#	return v

//#void __idiv__(flt):
//#	v = Vector2(self.x / flt, self.y / flt)
//#	return v

float Vector2::length()
{
	return sqrt(x * x + y * y);
}

float Vector2::lensq()
{
	return x * x + y * y;
}

//#end class Vector2

//class Point(Vector2):
//name, conns=None, x=380, y=180, color='//#0000ff'
Point::Point(string init_name, float init_x, float init_y, Pointps *init_conns, string init_color)
: Vector2(init_x, init_y)
{
	//Vector2::Vector2(init_x, init_y);
	name = init_name;
	if(init_conns != NULL)
	{
		conns = *init_conns;
	}
	color = init_color;
	//if conns == None: self.conns = []
}

/* *shrug* */
Point::Point()
{
}

Point::~Point()
{
}

string Point::str()
{
	string s;
	//cout << name << endl;
	s = name + " " + itostr(x) + ", " + itostr(y); //Vector2::str();
	return s;
}
//#end void __str__()

string Point::drawSvg(int line_width)
{
	string output;
	//Vector2 tpos = *this + Vector2(8, 8);
	//# Draw each point
	//output = "<!--<text x='" + itostr(tpos.x) + "' y='" + itostr(tpos.y) + "'>" + name + "</text>-->\n";
	output += "<circle cx='" + itostr(x) + "' cy='" + itostr(y) + "' r='4' fill='" + color + "' stroke='black' stroke-width='1' />\n";
	//# Draw each conn
	size_t j;
	for(j = 0; j < conns.size(); j++)
	{
		output += "<!-- " + this->str() + " to " + conns[j]->str() + "-->\n";
		output += "<path d='M " + itostr(x) + " " + itostr(y) + " L " + itostr(conns[j]->x) + " " + itostr(conns[j]->y) + "' fill='none' stroke='blue' stroke-width='" + itostr(line_width) + "' />\n";
	}
	//#next conn
	return output;
}

// conn, reConn=True
void Point::addConn(Point *conn, bool reConn)
{
	//#print 'adding conn?', self.name, conn
	conns.push_back(conn);
	if(reConn)
	{
		conn->addConn(this, false);
	}
}

void Point::setColor(string value)
{
	// Set color to a hex value like //#ff0000 //
	color = value;
}

void Point::setPos(Vector2 v)
{
	x = v.x;
	y = v.y;
}
//#end class Point(Vector2)

void gridToPos(std::vector <Pointps> *grid, float width)
{
	/*"""
	Simple function does the positioning of nodes in grid.
	Assumes a proper grid of points.
	TODO: When multiple nodes are in one cell, they should be spread out as 
	much as possible.
	"""*/
	size_t x, y;
	for (y = 0; y < grid->size(); y++) {
		for (x = 0; x < (*grid)[0].size(); x++) {
			if ((*grid)[y][x]) {
				/* TODO: implement another gridToPos for different grid.
				if (type(grid[y][x]) == list) {
					for p in grid[y][x]:
						p.setPos(Vector2(x * width, y * width));
					} //#end if
				} else {*/
					//#print(x, y, grid[y][x].name)
				(*grid)[y][x]->setPos(Vector2(x * width, y * width));
				//} //#end if
			} //#end if
		} //#next x
	} //#next y
} //#end def gridToPos(grid)

Rect::Rect(float init_min_x, float init_min_y, float init_max_x, float init_max_y)
{
	min_x = init_min_x;
	min_y = init_min_y;
	max_x = init_max_x;
	max_y = init_max_y;
}

Rect::~Rect()
{
}

#ifdef TEST_SWW

int main(int argc, char **argv)
{
	bool toSVG = false;
	int i;
	// TODO: getopt or something.
	for(i = 1; i < argc; i++)
	{
		if(strcmp(argv[argc], "--svg") == 0) {
			toSVG = true;
		}
	}
	cout << "Vector2" << endl;
	Vector2 v;
	cout << v.str() << endl;
	v = Vector2(62, 99);
	cout << v.str() << endl;
	v += Vector2(22, 11);
	cout << v.str() << endl;

	cout << "Point + Vector2" << endl;
	Point p("p", 0, 0, NULL, "#000000");
	cout << p.str() << endl;
	cout << v.str() << endl;
	p += v;
	cout << p.str() << endl;
	cout << v.str() << endl;
	
	cout << "Points" << endl;
	Points s;
	s.push_back(Point("a", 123, 456));
	s.push_back(Point("b", 789, 234));
	s.push_back(Point("c", 567, 890));
	s.push_back(Point("d", 917, 490));
	cout << s[0].str() << endl;
	cout << s[1].str() << endl;
	cout << s[2].str() << endl;

	s[0] += v;
	cout << s[0].str() << endl;
	
	Map map1(&s);
	//map1.name = "\"\\asdf1234\x01\x12\x81\x13\x82";
	map1.points[0].addConn(&(map1.points[1]));
	map1.points[0].addConn(&(map1.points[2]));
	map1.sortBranch();
	if(toSVG)
	{
		cout << map1.drawSvg();
	}
	else
	{
		cout << map1.saveJson() << std::endl;
	}
	Connectivity rev = map1.getPolarConn();
	Connectivity::iterator r;
	//int i = 0;
	for(r = rev.begin(); r != rev.end(); ++r, i++)
	{
		std::cout << "Polar Conn: " << (*r).a << " " << (*r).parent << std::endl;
	}
	map1.sortLinear();
	if(toSVG)
	{
		cout << map1.drawSvg();
	}
	else
	{
		cout << map1.saveJson() << std::endl;
	}
	map1.sortCircle();
	if(toSVG)
	{
		cout << map1.drawSvg();
	}
	else
	{
		cout << map1.saveJson() << std::endl;
	}
	map1.sortSpiral();
	if(toSVG)
	{
		cout << map1.drawSvg();
	}
	else
	{
		cout << map1.saveJson() << std::endl;
	}
	map1.sortLattice();
	map1.centerBound();
	if(toSVG)
	{
		cout << map1.drawSvg();
	}
	else
	{
		cout << map1.saveJson() << std::endl;
	}
	std::vector<MapGroup> mgs = map1.groupNodes();
	std::vector<MapGroup>::iterator group;
	for(group = mgs.begin(); group != mgs.end(); ++group)
	{
		Pointps::iterator p;
		for(p = (*group).points.begin(); p != (*group).points.end(); ++p)
		{
			cout << (*p)->name << ", ";
		}
		cout << endl;
	}
	return 0;
}


#endif // TEST_SWW
