/*
 * Small Wide World
 * by Joel R. Voss aka. Javantea
 * Copyright 2007 AltSci Concepts
 * June 28, 2007
 *
 * Port to C++ from Python.
 */

#ifndef SWW_H_
#define SWW_H_

#include <vector>
#include <string>
using std::vector;
using std::string;

#define PI 3.1415927f
#define TAU 2*PI
// TODO: Use something better for SMALL.
#define SMALL 1e-16

class Point;

class Vector2 {
	public:
	float x, y;
	
	Vector2 operator -(Vector2 v);
	Vector2 operator +(Vector2 v);
	Vector2 operator *(float f);
	Vector2 operator /(float f);
	Vector2 &operator +=(Vector2 v);
	Vector2 &operator -=(Vector2 v);
	Vector2 &operator *=(float f);
	Vector2 &operator /=(float f);
	float dot(Vector2 v);
	float length();
	float lensq();
	string str();
	operator string ();
	
	Vector2();
	Vector2(float x, float y);
	Vector2(Point *p);
	~Vector2();
};

class PolarVector {
	public:
	float r, theta;
	
	operator string ();
	
	PolarVector();
	PolarVector(float r, float theta);
	PolarVector(Vector2 *p);
	~PolarVector();
};

// TODO: This could be class methods.
PolarVector polar_add(PolarVector *v1, PolarVector *v2);
PolarVector polar_sub(PolarVector *v1, PolarVector *v2);
Vector2 polarToCartesian(PolarVector *v);
PolarVector cartesianToPolar(Vector2 *v);

typedef vector<Point> Points;
typedef vector<Point *> Pointps;

class Point : public Vector2 {
	public:
	string name;
	string color;
	string url;
	Pointps conns;
	
	void setColor(string new_color);
	void setPos(Vector2 v);
	void addConn(Point *point, bool reConn=true);
	string str();
	string drawSvg(int line_width);
	
	Point(string init_name, float init_x, float init_y, Pointps *init_conns = NULL, string init_color="#0000ff");
	Point();
	~Point();
	
};

class Conns {
	public:
	std::vector<int> indices;
};

// Hmm 2d array, just like python...
typedef std::vector<Conns> ConnList;

class Rect {
	public:
	float min_x;
	float min_y;
	float max_x;
	float max_y;
	Rect(float init_min_x=0, float init_min_y=0, float init_max_x=0, float init_max_y=0);
	~Rect();
};

class Reverse
{
	public:
	int a;
	int parent;
	Reverse(int a_init, int parent_init);
	~Reverse();
};

class Connectivity : public std::vector<Reverse>
{
	public:
	string anh;
	Connectivity();
	~Connectivity();
};

class MapGroup
{
	public:
	Pointps points;
	Point *left_bracket;
	Point *right_bracket;
	bool finished;
	void leaf(Point *p);
	void addPoint(Point *p);
	Pointps cull();
	float minX();
	MapGroup();
	~MapGroup();
};

class Map {
	public:
	string name;
	string desc;
	string line_color;
	Points points;
	int width;
	int height;
	int line_width;
	Point *selectedPoint;
	
	void addPoint(Point point);
	void addPoints(Points &add_points);
	void randomize(int amount=180);
	void click(int x, int y);
	bool drag(int x, int y);
	void sort1();
	void sort2(float dist1=70, float f=0.5);
	void sort3();
	void sort4();
	void center();
	void squash();
	string str();
	string drawSvg();
	string saveJson();
	bool setPos(float *v, size_t v_len);
	float *getX0();
	ConnList *getConns();
	Rect bounds();
	void move(float x, float y);
	void centerBound();
	void sortBranch(float r0=40, bool *knownPos=NULL);
	void sortLinear(float b0=40);
	void sortCircle(float b0=40);
	void sortSpiral(float b0=40);
	void sortLattice(float b0=40);
	MapGroup *addNodeToGroup(Point *p, Pointps *left, Pointps *right, std::vector<MapGroup> *groups, MapGroup *currGroup);
	std::vector<MapGroup> groupNodes();
	Connectivity getPolarConn();
	void setCartesian(std::vector<PolarVector> *dr);
	
	Map(Points *init_points=NULL, int init_width=640, int init_height=480, int init_line_width=2, string init_line_color="#8888ff");
	~Map();
	
};

int randint(int min, int max);
string itostr(int a);
string itohex(int a, int width);
string ftostr(float a);
string json_dumps(string s);

void gridToPos(std::vector <Pointps> *grid, float width=40);

#endif // SWW_H_
