#!/usr/bin/env python3
"""
Grid Global Minimization Algorithm for SWW
by Javantea
Feb 9, 2016
Based on Global Minimization Algorithm modified for SWW
by Javantea
Nov 9, 2015
Based on Global Minimization Algorithm
by Javantea
Nov 26, 2014 - Dec 1, 2014
Based on Convert PDB to Spherical Coordinates Oct 28, 2014 - Nov 26, 2014
Based on metropolis_min1.py by Javantea, July 17-22, 2014
Based on s_4atom.py by Javantea, June 24, 2014
Based on basinhopping_min1.py by Javantea, July 23, 2014

This algorithm creates a simple extensible grid adding points based on a loose interpretation of the Branch sort algorithm. If this works better than Branch sort or loop sort, it will be merged into Map class and made the default because it should produce prettier graphs.

1000 graphs on 4 octocore computers.
mkdir grid1za2 && cd $_ && python3 ../grid1.py -m -t -s 0 -e 35 >grid1testz1.txt &
mkdir grid1zb2 && cd $_ && python3 ../grid1.py -m -t -s 35 -e 70 >grid1testz2.txt &
mkdir grid1zc2 && cd $_ && python3 ../grid1.py -m -t -s 70 -e 105 >grid1testz3.txt &
mkdir grid1zd2 && cd $_ && python3 ../grid1.py -m -t -s 105 -e 140 >grid1testz4.txt &
mkdir grid1ze2 && cd $_ && python3 ../grid1.py -m -t -s 140 -e 175 >grid1testz5.txt &
mkdir grid1zf2 && cd $_ && python3 ../grid1.py -m -t -s 175 -e 210 >grid1testz6.txt &
mkdir grid1zg2 && cd $_ && python3 ../grid1.py -m -t -s 210 -e 245 >grid1testz7.txt &

mkdir grid1sa && cd $_ && python3 ../grid1.py -m -t -s 245 -e 280 >grid1tests1.txt &
mkdir grid1sb && cd $_ && python3 ../grid1.py -m -t -s 280 -e 315 >grid1tests2.txt &
mkdir grid1sc && cd $_ && python3 ../grid1.py -m -t -s 315 -e 350 >grid1tests3.txt &
mkdir grid1sd && cd $_ && python3 ../grid1.py -m -t -s 350 -e 385 >grid1tests4.txt &
mkdir grid1se && cd $_ && python3 ../grid1.py -m -t -s 385 -e 420 >grid1tests5.txt &
mkdir grid1sf && cd $_ && python3 ../grid1.py -m -t -s 420 -e 455 >grid1tests6.txt &
mkdir grid1sg && cd $_ && python3 ../grid1.py -m -t -s 455 -e 490 >grid1tests7.txt &

mkdir grid1ha && cd $_ && python3 ../grid1.py -m -t -s 490 -e 525 >grid1testh1.txt &
mkdir grid1hb && cd $_ && python3 ../grid1.py -m -t -s 525 -e 560 >grid1testh2.txt &
mkdir grid1hc && cd $_ && python3 ../grid1.py -m -t -s 560 -e 595 >grid1testh3.txt &
mkdir grid1hd && cd $_ && python3 ../grid1.py -m -t -s 595 -e 630 >grid1testh4.txt &
mkdir grid1he && cd $_ && python3 ../grid1.py -m -t -s 630 -e 665 >grid1testh5.txt &
mkdir grid1hf && cd $_ && python3 ../grid1.py -m -t -s 665 -e 700 >grid1testh6.txt &
mkdir grid1hg && cd $_ && python3 ../grid1.py -m -t -s 700 -e 735 >grid1testh7.txt &

mkdir grid1da && cd $_ && python3 ../grid1.py -m -t -s 735 -e 770 >grid1testd1.txt &
mkdir grid1db && cd $_ && python3 ../grid1.py -m -t -s 770 -e 805 >grid1testd2.txt &
mkdir grid1dc && cd $_ && python3 ../grid1.py -m -t -s 805 -e 840 >grid1testd3.txt &
mkdir grid1dd && cd $_ && python3 ../grid1.py -m -t -s 840 -e 875 >grid1testd4.txt &
mkdir grid1de && cd $_ && python3 ../grid1.py -m -t -s 875 -e 910 >grid1testd5.txt &
mkdir grid1df && cd $_ && python3 ../grid1.py -m -t -s 910 -e 945 >grid1testd6.txt &
mkdir grid1dg && cd $_ && python3 ../grid1.py -m -t -s 945 -e 980 >grid1testd7.txt &

mkdir grid1zh && cd $_ && python3 ../grid1.py -m -t -s 980 -e 1000 >grid1testz8.txt &

"""
from __future__ import print_function
import os
from math import pi
import random
import time
import string
import numpy as np
from scipy.optimize import minimize
from sww_optimize1 import LJ_Potential, V_SmallWideWorld, getNonConns, createMap4, ConnList
import mapmake1
tau = 2*pi
trueRand = random.SystemRandom()
Vector2 = mapmake1.Vector2
gridToPos = mapmake1.gridToPos

def usage():
	"Print the usage info."
	print(main.__doc__)
#end def usage()

class LJ_Potential2:
	"""
	Proper Lennard-Jones 6 12 Potential, input is r_sq instead of r.
	"""
	def __init__(self, r0=40, scale_factor=0.055):
		self.r0 = r0
		self.scale_factor = scale_factor
		self.A = 2 * ((r0)**6) * scale_factor
		self.B = ((r0)**12) * scale_factor
	#end def __init__()
	
	def V_LJ(self, r_sq):
		"""
		Attempt at a Lennard-Jones 6 12 potential
		The potential is very high when the two nodes are close.
		The potential is low when two nodes are far away.
		The potential is lowest when two nodes are at the r0 distance, not too 
		close, not too far away.
		Non-Connected nodes should have a LJ potential.
		r0 is the value for which this LJ potential is optimized.
		"""
		# This is a divide by zero protection.
		if r_sq < 1e-6:
			return self.V_LJ(1e-6)
		#end if
		r6 = 1/(r_sq ** 3)
		return ((-self.A*r6) + (self.B*r6*r6))
	#end def V_LJ(r)
#end class LJ_Potential

def GridPotentialMore(w, h, lj_conf2, b0=40):
	"""
	Experimental potential to replace V_SmallWideWorld
	Step 1: Precompute grid non-bonded potentials for all time
	"""
	b0_sq = (b0//4) * (b0//4)
	Pot = []
	for x in range(w*4*2):
		Pot.append([0] * h*4*2)
	#next x
	for x in range(w*4):
		for y in range(h*4):
			r_sq = b0_sq * ((x * x) + (y * y))
			V = lj_conf2.V_LJ(r_sq)
			Pot[w+x][h+y] = V
			Pot[w-x][h-y] = V
		#next y
	#next x
	return Pot
#end def GridPotential(w, h, lj_conf2, b0=40)

def GridPotential(w, h, lj_conf2, b0=40):
	"""
	Experimental potential to replace V_SmallWideWorld
	Step 1: Precompute grid non-bonded potentials for all time
	"""
	b0_sq = (b0) * (b0)
	Pot = []
	for x in range(w*2):
		Pot.append([0] * h*2)
	#next x
	for x in range(w):
		for y in range(h):
			r_sq = b0_sq * ((x * x) + (y * y))
			V = lj_conf2.V_LJ(r_sq)
			Pot[w+x][h+y] = V
			Pot[w-x][h-y] = V
		#next y
	#next x
	return Pot
#end def GridPotential(w, h, lj_conf2, b0=40)

def V_Grid_LJ(pointToPos, nonConns, Pot, w, h):
	"""
	Step 2: Take the list of nonConns and add potential from the cache.
	Instead of non-conns being a lookup into points or x, it should be coordinates.
	How do we do that when coordinates are always changing?
	We could use pointToPos.
	Note: this variable is pointPosMap in sortGrid/rearrangeGrid.
	TODO: Make a modified L-BFGS-B and Metropolis that can use this potential.
	TODO: Create a system that reduces this to O(N) or O(1) or preferrably O(0).
	"""
	Potential = 0.0
	for i in range(len(nonConns)):
		x1, y1 = pointToPos[i]
		for j in nonConns[i]:
			# i is not connected to j
			x2, y2 = pointToPos[j]
			dx = x2 - x1
			dy = y2 - y1
			# One of these will be 0, the other will be correct.
			# Note: it doesn't matter dx first or dy first, we're talking lengths.
			# This could be made into Potential += Pot[dx*dx + dy*dy] if it was faster.
			Potential += Pot[w+dx][h+dy]
		#next j
	#next i
	return Potential
#end def V_Grid_LJ(pointToPos, nonConns, Pot, w, h)

def testGrid(seed=9999, start=0, end=10000):
	"""
	Check important assumptions about grid vs other algorithms.
	At this point there are 343 bad values and 50 or so pretty bad ones.
	907 have branch lower which means that there are bugs in many of the solutions.
	However, it also means that 90% of the graphs are better from grid. Win.
	"""
	lj_conf = LJ_Potential()
	b0 = 40
	rn = trueRand.randint(0, 0xffffffff)
	for i in range(0, end):
		print("seed:", seed)
		nodes = mapmake1.randint(12, 50)
		conns = mapmake1.randint(1, 10)
		branches = mapmake1.randint(0, 20)
		print("Map: %i %i %i" % (nodes, conns, branches))
		map1 = createMap4(nodes, conns, branches, seed, False)
		# Use a different seed to get different maps. Deterministic.
		seed = mapmake1.randint(0, 0xffffffff)
		# If we don't do createMap4, the seed will be different, expensive but not as expensive as sorting/optimizing by a long shot.
		if i < start: continue
		#map1.sortLinear()
		#map1.sortLoop()
		#map1.centerBound()
		map1.desc = "SWW Optimize map with {} nodes and {} conns [results]".format(nodes, conns)
		# FIXME: rare overwrites
		base_filename = 'grid1_%i_%i.json' % (rn, i)
		x0, conns = map1.getX0Conns(True)
		conns_cpp = ConnList(conns)
		#print(x0)
		#if verbose:
		#	print("atom conns:", conns)
		#end if
		nonConns = ConnList(getNonConns(conns, True))

		# Not necessary to get spherical coordinates of the original coordinates.
		#output = [spherical2.cartesianToSpherical(v) for v in molecule.x0]
		#print("Spherical coordinates of atoms:")
		#print(output)
		
		# This may be used eventually
		#sphericalCoords, atom_name_hash = molecule.toSpherical(verbose)
		start_t = time.time()
		map1.sortBranch(b0)
		branch_dt = (time.time() - start_t)
		x_out = map1.getX0Conns()[0]
		V_branch = V_SmallWideWorld(x_out, conns_cpp, b0, lj_conf, nonConns)
		print("Branch dt: %3.3f seconds" % branch_dt)

		start2_t = time.time()
		map1.sortGrid(b0)
		x_out = map1.getX0Conns()[0]
		V_grid = V_SmallWideWorld(x_out, conns_cpp, b0, lj_conf, nonConns)
		grid_dt = (time.time() - start2_t)
		print("Branch Potential:", V_branch)
		print("Grid dt: %3.3f seconds" % grid_dt)
		print("Grid Potential:", V_grid)
		dV = (V_branch - V_grid)
		print("Branch - Grid: %3.3f" % dV)
		best = 'grid'
		V_best = V_grid
		if dV < 0:
			map1.centerBound()
			open(base_filename.replace('.json', 'branch.json'), 'w').write(map1.saveJson())
			best = 'branch'
			V_best = V_branch
		#end if
		# Spiral
		start3_t = time.time()
		map1.sortSpiral(b0)
		x_out = map1.getX0Conns()[0]
		V_spiral = V_SmallWideWorld(x_out, conns_cpp, b0, lj_conf, nonConns)
		spiral_dt = (time.time() - start3_t)
		print("Spiral dt: %3.3f seconds" % spiral_dt)
		print("Spiral Potential:", V_spiral)
		if V_spiral < V_best:
			print("Note: Spiral is lower, see %s" % base_filename)
			map1.centerBound()
			open(base_filename.replace('.json', 'spiral.json'), 'w').write(map1.saveJson())
			best = 'spiral'
			V_best = V_spiral
		#end if
		# Lattice
		start4_t = time.time()
		map1.sortLattice(b0)
		x_out = map1.getX0Conns()[0]
		V_lattice = V_SmallWideWorld(x_out, conns_cpp, b0, lj_conf, nonConns)
		lattice_dt = (time.time() - start4_t)
		print("Lattice dt: %3.3f seconds" % lattice_dt)
		print("Lattice Potential:", V_lattice)
		if V_lattice < V_best:
			print("Note: Lattice is lower, see %s" % base_filename)
			map1.centerBound()
			open(base_filename.replace('.json', 'lattice.json'), 'w').write(map1.saveJson())
			best = 'lattice'
			V_best = V_lattice
		#end if
		# Linear
		start5_t = time.time()
		map1.sortLinear(b0)
		x_out = map1.getX0Conns()[0]
		V_linear = V_SmallWideWorld(x_out, conns_cpp, b0, lj_conf, nonConns)
		linear_dt = (time.time() - start5_t)
		print("Linear dt: %3.3f seconds" % linear_dt)
		print("Linear Potential:", V_linear)
		if V_linear <= V_best:
			print("Warning: Linear is lower, see %s" % base_filename)
			map1.centerBound()
			open(base_filename.replace('.json', 'linear.json'), 'w').write(map1.saveJson())
			best = 'linear'
			V_best = V_linear
		#end if
		# NOTE: Loop must follow linear or else you need to run map1.sortLinear(b0) before map1.sortLoop(b0).
		# Loop
		start6_t = time.time()
		map1.sortLoop(b0)
		x_out = map1.getX0Conns()[0]
		V_loop = V_SmallWideWorld(x_out, conns_cpp, b0, lj_conf, nonConns)
		loop_dt = (time.time() - start6_t)
		print("Loop dt: %3.3f seconds" % loop_dt)
		print("Loop Potential:", V_loop)
		if V_loop <= V_grid:
			map1.centerBound()
			open(base_filename.replace('.json', 'loop.json'), 'w').write(map1.saveJson())
			best = 'loop'
			V_best = V_loop
		#end if
		# Right Loop
		start7_t = time.time()
		map1.sortRightLoop(b0)
		x_out = map1.getX0Conns()[0]
		V_rightloop = V_SmallWideWorld(x_out, conns_cpp, b0, lj_conf, nonConns)
		rightloop_dt = (time.time() - start7_t)
		print("Right Loop dt: %3.3f seconds" % rightloop_dt)
		print("Right Loop Potential:", V_spiral)
		if V_rightloop < V_best:
			print("Note: Right Loop is lower, see %s" % base_filename)
			map1.centerBound()
			open(base_filename.replace('.json', 'rightloop.json'), 'w').write(map1.saveJson())
			best = 'rightloop'
			V_best = V_rightloop
		#end if
		# Random
		start8_t = time.time()
		for p in map1.points:
			p.setPos(Vector2(0, 0))
		#next i
		# FIXME: Pick a better equation for random factor.
		map1.randomize(b0 * nodes * 0.3)
		x_out = map1.getX0Conns()[0]
		V_random = V_SmallWideWorld(x_out, conns_cpp, b0, lj_conf, nonConns)
		random_dt = (time.time() - start8_t)
		print("Random dt: %3.3f seconds" % random_dt)
		print("Random Potential:", V_random)
		if V_random <= V_best:
			print("Warning: Random is lower, see %s" % base_filename)
			map1.centerBound()
			open(base_filename.replace('.json', 'random.json'), 'w').write(map1.saveJson())
			best = 'random'
			V_best = V_random
		#end if
		map1.sortFruchtermanReingoldLoop(b0, iterations=200, dt=0.98)
		x_out = map1.getX0Conns()[0]
		V_FruchtermanReingold = V_SmallWideWorld(x_out, conns_cpp, b0, lj_conf, nonConns)
		FruchtermanReingold_dt = (time.time() - start8_t)
		print("FruchtermanReingold dt: %3.3f seconds" % FruchtermanReingold_dt)
		print("FruchtermanReingold Potential:", V_FruchtermanReingold)
		if V_FruchtermanReingold <= V_best:
			print("Warning: FruchtermanReingold is lower, see %s" % base_filename)
			map1.centerBound()
			open(base_filename.replace('.json', 'FruchtermanReingold.json'), 'w').write(map1.saveJson())
			best = 'FruchtermanReingold'
			V_best = V_FruchtermanReingold
		#end if
		print("{0} is lowest".format(best))
		if best != 'grid':
			print("see %s" % base_filename.replace('.json', best + '.json'))
		#end if
	#next i
#end def testGrid(seed=9999, start=0, end=10000)


def testGridMinimized(seed=9999, start=0, end=1000):
	"""
	Check important assumptions about grid vs other algorithms.
	At this point there are 343 bad values and 50 or so pretty bad ones.
	907 have branch lower which means that there are bugs in many of the solutions.
	However, it also means that 90% of the graphs are better from grid. Win.
	"""
	lj_conf = LJ_Potential()
	b0 = 40
	gtol = 1e-2
	rn = trueRand.randint(0, 0xffffffff)
	for i in range(0, end):
		print("seed:", seed)
		#nodes = mapmake1.randint(150, 300)
		#conns = mapmake1.randint(1, 60)
		#branches = mapmake1.randint(0, 120)
		#nodes = mapmake1.randint(5, 15)
		#branches = mapmake1.randint(0, 3)
		nodes = mapmake1.randint(12, 150)
		conns = mapmake1.randint(1, 30)
		branches = mapmake1.randint(0, 60)
		total_nodes = nodes + branches
		if total_nodes == 5:
			maxconns = 5
		elif total_nodes == 6:
			maxconns = 9
		elif total_nodes == 7:
			maxconns = 14
		elif total_nodes == 8:
			maxconns = 20
		elif total_nodes == 9:
			maxconns = 27
		else:
			maxconns = 30
		#end if
		#conns = mapmake1.randint(1, maxconns)
		if conns > maxconns: conns = maxconns
		#nodes = mapmake1.randint(12, 50)
		#conns = mapmake1.randint(1, 10)
		#branches = mapmake1.randint(0, 20)
		print("Map: %i %i %i" % (nodes, conns, branches))
		map1 = createMap4(nodes, conns, branches, seed, False)
		# Use a different seed to get different maps. Deterministic.
		seed = mapmake1.randint(0, 0xffffffff)
		# If we don't do createMap4, the seed will be different, expensive but not as expensive as sorting/optimizing by a long shot.
		if i < start: continue
		#map1.sortLinear()
		#map1.sortLoop()
		#map1.centerBound()
		map1.desc = "SWW Optimize map with {} nodes and {} conns [results]".format(nodes, conns)
		# FIXME: rare overwrites
		base_filename = 'grid1_%i_%i.json' % (rn, i)
		x0, conns = map1.getX0Conns(True)
		conns_cpp = ConnList(conns)
		#print(x0)
		#if verbose:
		#	print("atom conns:", conns)
		#end if
		nonConns = ConnList(getNonConns(conns, True))
		minimizer_opts = {"method": "L-BFGS-B", 'args':(conns_cpp, b0, lj_conf, nonConns), 'options':{'gtol':gtol, 'eps':1e-4}}

		# Not necessary to get spherical coordinates of the original coordinates.
		#output = [spherical2.cartesianToSpherical(v) for v in molecule.x0]
		#print("Spherical coordinates of atoms:")
		#print(output)
		
		# This may be used eventually
		#sphericalCoords, atom_name_hash = molecule.toSpherical(verbose)
		start_t = time.time()
		map1.sortBranch(b0)
		x_out = map1.getX0Conns()[0]
		#V_branch = V_SmallWideWorld(x_out, conns_cpp, b0, lj_conf, nonConns)
		resb = minimize(V_SmallWideWorld, x_out, **minimizer_opts)
		V_branch = V_SmallWideWorld(resb.x, conns_cpp, b0, lj_conf, nonConns)
		branch_dt = (time.time() - start_t)
		print("Branch dt: %3.3f seconds" % branch_dt)

		start2_t = time.time()
		map1.sortGrid(b0)
		x_out = map1.getX0Conns()[0]
		res = minimize(V_SmallWideWorld, x_out, **minimizer_opts)
		V_grid = V_SmallWideWorld(res.x, conns_cpp, b0, lj_conf, nonConns)
		grid_dt = (time.time() - start2_t)
		print("Branch Potential:", V_branch)
		print("Grid dt: %3.3f seconds" % grid_dt)
		print("Grid Potential:", V_grid)
		dV = (V_branch - V_grid)
		print("Branch - Grid: %3.3f" % dV)
		best = 'grid'
		V_best = V_grid
		if dV < 0:
			m = np.matrixlib.matrix(resb.x)
			m = m.reshape(len(map1.points), 2)
			map1.setPos([np.array(xa)[0] for xa in m])
			map1.centerBound()
			open(base_filename.replace('.json', 'branch.json'), 'w').write(map1.saveJson())
			best = 'branch'
			V_best = V_branch
		else:
			# Save grid.json to save only if best == grid.
			# gridJson is only used if best == grid which isn't true in the other case.
			m = np.matrixlib.matrix(res.x)
			m = m.reshape(len(map1.points), 2)
			map1.setPos([np.array(xa)[0] for xa in m])
			map1.centerBound()
			gridJson = map1.saveJson()
		#end if
		# Spiral
		start3_t = time.time()
		map1.sortSpiral(b0)
		x_out = map1.getX0Conns()[0]
		res = minimize(V_SmallWideWorld, x_out, **minimizer_opts)
		V_spiral = V_SmallWideWorld(res.x, conns_cpp, b0, lj_conf, nonConns)
		spiral_dt = (time.time() - start3_t)
		print("Spiral dt: %3.3f seconds" % spiral_dt)
		print("Spiral Potential:", V_spiral)
		if V_spiral < V_best:
			m = np.matrixlib.matrix(res.x)
			m = m.reshape(len(map1.points), 2)
			map1.setPos([np.array(xa)[0] for xa in m])
			map1.centerBound()
			open(base_filename.replace('.json', 'spiral.json'), 'w').write(map1.saveJson())
			best = 'spiral'
			V_best = V_spiral
		#end if
		# Lattice
		start4_t = time.time()
		map1.sortLattice(b0)
		x_out = map1.getX0Conns()[0]
		res = minimize(V_SmallWideWorld, x_out, **minimizer_opts)
		V_lattice = V_SmallWideWorld(res.x, conns_cpp, b0, lj_conf, nonConns)
		lattice_dt = (time.time() - start4_t)
		print("Lattice dt: %3.3f seconds" % lattice_dt)
		print("Lattice Potential:", V_lattice)
		if V_lattice < V_best:
			m = np.matrixlib.matrix(res.x)
			m = m.reshape(len(map1.points), 2)
			map1.setPos([np.array(xa)[0] for xa in m])
			map1.centerBound()
			open(base_filename.replace('.json', 'lattice.json'), 'w').write(map1.saveJson())
			best = 'lattice'
			V_best = V_lattice
		#end if
		# Linear
		start5_t = time.time()
		map1.sortLinear(b0)
		x_out = map1.getX0Conns()[0]
		res = minimize(V_SmallWideWorld, x_out, **minimizer_opts)
		V_linear = V_SmallWideWorld(res.x, conns_cpp, b0, lj_conf, nonConns)
		linear_dt = (time.time() - start5_t)
		print("Linear dt: %3.3f seconds" % linear_dt)
		print("Linear Potential:", V_linear)
		if V_linear <= V_best:
			m = np.matrixlib.matrix(res.x)
			m = m.reshape(len(map1.points), 2)
			map1.setPos([np.array(xa)[0] for xa in m])
			map1.centerBound()
			open(base_filename.replace('.json', 'linear.json'), 'w').write(map1.saveJson())
			best = 'linear'
			V_best = V_linear
		#end if
		# Loop
		start6_t = time.time()
		map1.sortLinear(b0)
		map1.sortLoop(b0)
		x_out = map1.getX0Conns()[0]
		res = minimize(V_SmallWideWorld, x_out, **minimizer_opts)
		V_loop = V_SmallWideWorld(res.x, conns_cpp, b0, lj_conf, nonConns)
		loop_dt = (time.time() - start6_t)
		print("Loop dt: %3.3f seconds" % loop_dt)
		print("Loop Potential:", V_loop)
		if V_loop <= V_grid:
			m = np.matrixlib.matrix(res.x)
			m = m.reshape(len(map1.points), 2)
			map1.setPos([np.array(xa)[0] for xa in m])
			map1.centerBound()
			open(base_filename.replace('.json', 'loop.json'), 'w').write(map1.saveJson())
			best = 'loop'
			V_best = V_loop
		#end if
		# Right Sort
		start7_t = time.time()
		map1.sortRightLoop(b0)
		x_out = map1.getX0Conns()[0]
		res = minimize(V_SmallWideWorld, x_out, **minimizer_opts)
		V_rightsort = V_SmallWideWorld(res.x, conns_cpp, b0, lj_conf, nonConns)
		rightsort_dt = (time.time() - start7_t)
		print("Right Sort dt: %3.3f seconds" % rightsort_dt)
		print("Right Sort Potential:", V_rightsort)
		if V_rightsort < V_best:
			m = np.matrixlib.matrix(res.x)
			m = m.reshape(len(map1.points), 2)
			map1.setPos([np.array(xa)[0] for xa in m])
			map1.centerBound()
			open(base_filename.replace('.json', 'rightsort.json'), 'w').write(map1.saveJson())
			best = 'rightsort'
			V_best = V_rightsort
		#end if
		# Random
		start8_t = time.time()
		for p in map1.points:
			p.setPos(Vector2(0, 0))
		#next i
		# FIXME: Pick a better equation for random factor.
		map1.randomize(b0 * nodes * 0.3)
		x_out = map1.getX0Conns()[0]
		res = minimize(V_SmallWideWorld, x_out, **minimizer_opts)
		V_random = V_SmallWideWorld(res.x, conns_cpp, b0, lj_conf, nonConns)
		random_dt = (time.time() - start8_t)
		print("Random dt: %3.3f seconds" % random_dt)
		print("Random Potential:", V_random)
		if V_random <= V_best:
			m = np.matrixlib.matrix(res.x)
			m = m.reshape(len(map1.points), 2)
			map1.setPos([np.array(xa)[0] for xa in m])
			map1.centerBound()
			open(base_filename.replace('.json', 'random.json'), 'w').write(map1.saveJson())
			best = 'random'
			V_best = V_random
		#end if
		# TODO: Try different inputs to Fruchterman-Reingold
		map1.sortFruchtermanReingoldLoop(b0, iterations=200, dt=0.98)
		x_out = map1.getX0Conns()[0]
		res = minimize(V_SmallWideWorld, x_out, **minimizer_opts)
		V_FruchtermanReingold = V_SmallWideWorld(res.x, conns_cpp, b0, lj_conf, nonConns)
		FruchtermanReingold_dt = (time.time() - start8_t)
		print("FruchtermanReingold dt: %3.3f seconds" % FruchtermanReingold_dt)
		print("FruchtermanReingold Potential:", V_FruchtermanReingold)
		if V_FruchtermanReingold <= V_best:
			m = np.matrixlib.matrix(res.x)
			m = m.reshape(len(map1.points), 2)
			map1.setPos([np.array(xa)[0] for xa in m])
			map1.centerBound()
			open(base_filename.replace('.json', 'FruchtermanReingold.json'), 'w').write(map1.saveJson())
			best = 'FruchtermanReingold'
			V_best = V_FruchtermanReingold
		#end if
		print("{0} is lowest".format(best))
		if best != 'grid':
			print("see %s" % base_filename.replace('.json', best + '.json'))
		else:
			open(base_filename.replace('.json', 'grid.json'), 'w').write(gridJson)
		#end if
	#next i
#end def testGridMinimized(seed=9999, start=0, end=1000)

def main():
	"""Usage: python3 grid1.py [-v] [-h] [-d] [-t] [filename.json] [...]
	Runs AltSci Grid Global Minimization Algorithm on a JSON file.
	-v | --verbose Print the name combinations that produce the dr
	-h | --help    Print this help.
	-d | --data    Write a dat file as output instead of a array. Dat files 
	               are very easy to parse with numpy or gnuplot.
	-t | --test    Run a quick test to ensure that Grid sort always beats Branch sort.
	-n | --nodes   How many nodes in the backbone [default 20]
	-c | --conns   How many connections between the backbone [default 4]
	-b | --branches How many branches to add to the backbone [default 0]
	-r | --random  Use a random seed instead of a deterministic seed id
	-x | --seed    Deterministic random number generator seed [default 9999]
	"""
	import sys
	import getopt
	
	# Deterministic
	mapmake1.seed(12345)
	
	# Do not switch to scientific notation even though you enjoy it.
	# We're trying to do science here!
	np.set_printoptions(suppress=True)

	filenames = ['']
	verbose = False
	datOutput = False
	#if len(sys.argv) > 1:
	#	filename = sys.argv[1]
	#end if
	start = 0
	end = None
	actualRandom = False
	seed = 9999
	nodes = 20
	conns = 4
	branches = 0
	minimizeOut = False
	test = False
	experiment = False
	experiment2 = False
	funcName = ''
	#steps = 20
	gtol = 1e-2
	try:
		flags, extra_args = getopt.getopt(sys.argv[1:], 'vhmdtpqs:e:n:c:b:rug:x:f:', 
		['verbose', 'help', 'min', 'data', 'test', 'experiment', 'experiment2', 'funcname=', 'start=', 
		'end=', 'nodes=', 'conns=', 'branches=', 'random', 'shuffle', 'gtol=', 'seed='])
	except getopt.GetoptError as goe:
		print(goe)
		return 1
	#end try
	for flag in flags:
		if flag[0] == '-v' or flag[0] == '--verbose':
			verbose = True
		elif flag[0] == '-m' or flag[0] == '--min':
			minimizeOut = True
		elif flag[0] == '-d' or flag[0] == '--data':
			datOutput = True
		elif flag[0] == '-t' or flag[0] == '--test':
			test = True
		elif flag[0] == '-p' or flag[0] == '--experiment':
			experiment = True
		elif flag[0] == '-q' or flag[0] == '--experiment2':
			experiment2 = True
		elif flag[0] == '-f' or flag[0] == '--funcname':
			funcName = flag[1]
		elif flag[0] == '-h' or flag[0] == '--help':
			usage()
			return 1
		elif flag[0] == '-s' or flag[0] == '--start':
			start = int(flag[1])
		elif flag[0] == '-e' or flag[0] == '--end':
			end = int(flag[1])
		elif flag[0] == '-n' or flag[0] == '--nodes':
			nodes = int(flag[1])
		elif flag[0] == '-c' or flag[0] == '--conns':
			conns = int(flag[1])
		elif flag[0] == '-b' or flag[0] == '--branches':
			branches = int(flag[1])
		elif flag[0] == '-r' or flag[0] == '--random':
			actualRandom = True
		elif flag[0] == '-u' or flag[0] == '--shuffle':
			shuffle = True
		elif flag[0] == '-g' or flag[0] == '--gtol':
			gtol = float(flag[1])
		elif flag[0] == '-x' or flag[0] == '--seed':
			seed = int(flag[1])
		#end if
	#next flag
	
	if extra_args != []:
		filenames = extra_args
	#end if
	
	lj_conf = LJ_Potential()
	
	# Necessary for the functions we call, optional for mapmake1.
	mapmake1.loadPolar()
	b0 = 40
	
	if test:
		if minimizeOut:
			return testGridMinimized(seed, start, end)
		else:
			return testGrid(seed, start, end)
		#end if
	#end if
	if experiment:
		if len(filenames) > 0 and filenames[0] != '':
			filename = filenames[0]
		else:
			filename = 'sww_optimize1out21121.json'
		#end if
		map1 = mapmake1.Map()
		map1.loadJson(open(filename, 'r').read())
		grid = map1.sortGrid()
		x0, conns = map1.getX0Conns(True)
		conns_cpp = ConnList(conns)
		nonConns = getNonConns(conns, True)
		
		lj_conf2 = LJ_Potential2()
		Pot = GridPotential(len(grid[0]), len(grid), lj_conf2)
		print('\n'.join([', '.join(['%3.4f' % x for x in v if x < -0.0001 or x > 0.0001]) for v in Pot]))
		pointToPos = {}
		for y, row in enumerate(grid):
			for x, col in enumerate(row):
				if col:
					i = map1.points.index(col)
					pointToPos[i] = (x, y)
				#end if
			#next x, col
		#next y, row
		Potential = V_Grid_LJ(pointToPos, nonConns, Pot, len(grid[0]), len(grid))
		"""
		# FIXME: The potential we get back is wrong for sww_optimize1out21121.json *shrug*
		GridPotential: Potential: 35374309.79898411
		grid1.py: 176666.60562368366
		grid JS: 219713.70853792407
		First, solve GridPotential, then solve grid JS.
		"""
		print("Potential:", Potential)
		return
	#end if
	if experiment2:
		if len(filenames) > 0 and filenames[0] != '':
			filename = filenames[0]
			base_filename = filename
			map1 = mapmake1.Map()
			map1.loadJson(open(filename, 'r').read())
		else:
			# They didn't give a filename must want us to generate one.
			if actualRandom:
				# TODO: Reduces the entropy to very low.
				seed = mapmake1.randint(0, 0xffffffff)
			#end if
			# Don't sort, we'll use loop sort.
			map1 = createMap4(nodes, conns, branches, seed, False)
			#map1.sortLinear()
			#map1.sortLoop()
			#map1.centerBound()
			map1.desc = "SWW #sortfunc# Experiment 2 map with {} nodes and {} conns [results]".format(nodes, conns)
			# FIXME: rare overwrites
			base_filename = 'grid1_%i.json' % trueRand.randint(0, 0xffffffff)
		#end if
		sortFunc = map1.sortGrid
		# TODO: Validate that this isn't any sort of privilege escalation.
		if hasattr(map1, 'sort' + funcName):
			sortFunc = getattr(map1, 'sort' + funcName)
		elif hasattr(map1, 'sort' + string.capwords(funcName)):
			sortFunc = getattr(map1, 'sort' + string.capwords(funcName))
		elif funcName != '':
			print("Warning: sort function does not exist.", funcName)
			exit(1)
		#end if
		map1.desc = map1.desc.replace('#sortfunc#', sortFunc.__name__)
		map1.sortOrder(sortFunc, 1000)
		open(base_filename.replace('.json', 'out_%s.json' % sortFunc.__name__), 'w').write(map1.saveJson())
		return
	#end if
	
	for filename in filenames:
		if filename != '':
			base_filename = filename.split(os.sep)[-1]
			map1 = mapmake1.Map()
			map1.loadJson(open(filename, 'r').read())
		else:
			# They didn't give a filename must want us to generate one.
			if actualRandom:
				# TODO: Reduces the entropy to very low.
				seed = mapmake1.randint(0, 0xffffffff)
			#end if
			# Don't sort, we'll use loop sort.
			map1 = createMap4(nodes, conns, branches, seed, False)
			#map1.sortLinear()
			#map1.sortLoop()
			#map1.centerBound()
			map1.desc = "SWW Grid map with {} nodes and {} conns [results]".format(nodes, conns)
			# FIXME: rare overwrites
			base_filename = 'grid1_%i.json' % trueRand.randint(0, 0xffffffff)
		#end if
		x0, conns = map1.getX0Conns(True)
		conns_cpp = ConnList(conns)
		#print(x0)
		if verbose:
			print("atom conns:", conns)
		#end if
		nonConns = ConnList(getNonConns(conns, True))
		V_init = V_SmallWideWorld(x0, conns_cpp, b0, lj_conf, nonConns)

		# Not necessary to get spherical coordinates of the original coordinates.
		#output = [spherical2.cartesianToSpherical(v) for v in molecule.x0]
		#print("Spherical coordinates of atoms:")
		#print(output)
		
		# This may be used eventually
		#sphericalCoords, atom_name_hash = molecule.toSpherical(verbose)
		
		minimizer_opts = {"method": "L-BFGS-B", 'args':(conns_cpp, b0, lj_conf, nonConns), 'options':{'gtol':gtol, 'eps':1e-4}}

		start_t = time.time()
		map1.sortGrid()
		map1.centerBound()
		x_out = map1.getX0Conns()[0]
		V_out = V_SmallWideWorld(x_out, conns_cpp, b0, lj_conf, nonConns)
		print("Initial Potential:", V_init)
		print("Output Potential:", V_out)
		print("Algorithm Step 1: %3.3f seconds" % (time.time() - start_t))
		"""
		if datOutput:
			datFilename_i = 0
			while os.path.exists('grid1_Vt%i.dat' % datFilename_i):
				datFilename_i += 1
			#loop
			t_min_dat = [t[0]]
			V_min_dat = [V[0]]
			curr_V_min = V[0]
			for t_i, V_i in zip(t, V):
				if V_i < curr_V_min:
					t_min_dat.append(t_i)
					V_min_dat.append(V_i)
					curr_V_min = V_i
				#end if
			#next t_i, V_i
			with open('grid1_Vt%i.dat' % datFilename_i, 'w') as dat_file:
				dat_file.write('\n'.join(['%3.7f %3.7f' % x for x in zip(t_min_dat, V_min_dat)]))
			#end with
			print('grid1_Vt%i.dat' % datFilename_i)
		#end if
		"""
		
		start2_t = time.time()
		
		x_i = map1.getX0Conns()[0]
		#V_i = V_SmallWideWorld(x_i, conns, b0, lj_conf, nonConns)
		#print('Potential Recalc:', min_i, V_i)
		open(base_filename.replace('.json', 'out.json'), 'w').write(map1.saveJson())
		#open(base_filename.replace('.json', 'out.svg'), 'w').write(map1.drawSvg())
		print(base_filename.replace('.json', 'out.json'))
		
		if not minimizeOut: continue

		res = minimize(V_SmallWideWorld, x_i, **minimizer_opts)
		print(res)
		Vmin = V_SmallWideWorld(res.x, conns_cpp, b0, lj_conf, nonConns)
		print('Local Minimized:', Vmin)
		m = np.matrixlib.matrix(res.x)
		m = m.reshape(len(map1.points), 2)
		map1.setPos([np.array(xa)[0] for xa in m])
		map1.centerBound()
		open(base_filename.replace('.json', '') + ('out.min.json'), 'w').write(map1.saveJson())
		open(base_filename.replace('.json', '') + ('out.min.svg'), 'w').write(map1.drawSvg())
		print("Algorithm Step 2: %3.3f seconds" % (time.time() - start2_t))
	#next filename
	
	"""
	Using minimize:
	x0 = np.array([0.,0.,0., 0.,0.,0.99])
	conns = [[1], [0]]
	xmin = scipy.optimize.minimize(algorithm1.V_molecular_mult, x0, method='L-BFGS-B', args=(conns,))
	v = xmin.x
	v[3] -= v[0]
	v[4] -= v[1]
	v[5] -= v[2]
	v[0] = 0
	v[1] = 0
	v[2] = 0
	print(v)
	[ 0.          0.          0.          0.          0.          0.97718752]
	This shows us that the optimal distance between two connected items in 
	the potential (as it currently is) is 0.97718752 angstroms.
	In an optimized methane molecule, the distance is usually 1.0 angstroms.
	"""
	
	# return the last in case it's being called by a tester.
	return map1
#end def main()

if __name__ == '__main__':
	main()
#end if
