#!/usr/bin/env python3
"""
Natural Language Parsing
by Javantea
Aug 31, 2016

Based on An Oversimplification of the American-led intervention in Syria
by Javantea
Aug 30, 2016

Inputs subject verb object additional
Outputs a graph of all relationships.

additional always starts with because for now.
"""
import sys
import mapmake1
import nltk

def NaturalLanguageToGraph(data):
	"""
	Convert a simple explanation of relationships to a graph.
	"""
	lines = data.split('\n')
	#verbs = 'fights,supports,opposes,condemned,condemns,condemn,does not support,stopped fighting,fight'.split(',')
	#['fights', 'fight', 'supports', 'opposes', 'condemn', 'condemned', 'does not support', 'stopped fighting']
	all_verbs = []
	add = 'because'.split(',')

	rel = []

	for line in lines:
		words = nltk.word_tokenize(line)
		tagged_sent = nltk.pos_tag(words)
		if ('.', '.') in tagged_sent:
			tagged_sent.remove(('.', '.'))
		#end if
		#print(tagged_sent)
		verb = []
		# This is an ugly grammar, but it works for syria1.txt. So you need to work with that assumption.
		# subject verb object additional
		acceptable_to_verb = False
		acceptable_to_add = False
		additional = []
		for i, (word, tag) in enumerate(tagged_sent):
			# No verbs allowed at the end of a sentence.
			if i == (len(tagged_sent) - 1):
				acceptable_to_verb = False
			#end if
			if acceptable_to_verb and (tag.startswith('V') or tag == 'DOZ' or tag.startswith('BE') or word == 'not'):
				verb.append(word)
				continue
			#end if
			if not acceptable_to_verb:
				if i == 0: acceptable_to_verb = True
			else:
				if verb:
					acceptable_to_verb = False
					acceptable_to_add = True
				#end if
			#end if
			if acceptable_to_add and (word == 'because' or additional):
				additional.append(word)
			#end if
		#next i, (word, tag)
		subject = []
		object1 = []
		if len(verb) == 0:
			if len(tagged_sent) == 3:
				subject, verb, object1 = [word for word, tag in tagged_sent]
				subject = [subject]
				verb = [verb]
				object1 = [object1]
			else:
				for word, tag in tagged_sent:
					if word in all_verbs:
						verb.append(word)
					#end if
				#next word, tag
				if len(verb) == 0:
					print("Unparsable sentence", tagged_sent)
					continue
				#end if
			#end if
		#end if
		if len(subject) == 0:
			res = subject
			verb_index = 0
			for word, tag in tagged_sent:
				if verb_index < len(verb) and word == verb[verb_index]:
					res = object1
					verb_index += 1
					continue
				#end if
				if res == object1 and len(additional) > 0 and word == additional[0]:
					break
				#end if
				res.append(word)
			#next word, tag
		#end if
		all_verbs.extend(verb)
		if len(subject) == 0 or len(object1) == 0:
			
			print("Warning: failed to get subject and object", tagged_sent)
			continue
		#end if
		print('Result:', subject, verb, object1)
		rel.append((subject, verb, object1, additional))
	#next line

	print(rel)

	points = []
	nodeName = {}
	for subject, verb, object1, add_text in rel:
		subject_str = ' '.join(subject)
		object1_str = ' '.join(object1)
		if subject_str not in nodeName:
			points.append(mapmake1.Point(subject_str))
			nodeName[subject_str] = points[-1]
		#end if
		if object1_str not in nodeName:
			points.append(mapmake1.Point(object1_str))
			nodeName[object1_str] = points[-1]
		#end if
		nodeName[subject_str].addConn(nodeName[object1_str])
	#next subject, verb, object1, add_text
	map1 = mapmake1.Map()
	map1.addPoints(points)
	return map1
#end def NaturalLanguageToGraph(data)

def main():
	filename = 'syria1.txt'
	if len(sys.argv) > 1:
		filename = sys.argv[1]
	#end if
	data = open(filename,'r').read().strip()
	map1 = NaturalLanguageToGraph(data)
	map1.sortBest()
	jsonText = map1.saveJson()
	if jsonText[-1] == '}':
		jsonText = jsonText[:-1] + ', "showText":true}'
	#end if
	out_file = filename.replace('.txt', '2.json')
	print('Wrote', out_file)
	open(out_file, 'w').write(jsonText)
#end def main()

if __name__ == '__main__':
	main()
#end if
