#!/usr/bin/env python3
"""
Polar Coordinates
by Javantea
Nov 10, 2015
Based on AltSci Algorithm Spherical Coordinates Take 2
by Javantea
Oct 26, 2014

It turns out that polar coordinates are very easy. Why did I act foolish about it?
http://en.wikipedia.org/wiki/Spherical_coordinate_system#Cartesian_coordinates

NOTE: Vector Subtraction and Vector Addition in Spherical coordinates does 
_NOT_ work the same way it does it Cartesian coordinates. Do not attempt to 
add or subtract polar coordinates.
To multiply or divide a polar coordinate just multiply or divide the r value.
"""
from math import sqrt, atan2, sin, cos

# FIXME: SMALL needs to be based on divide by zero and the a-a=0 problem.
SMALL = 1e-12
SMALL_SQ = 1e-24

def cartesianToPolar(v):
	"""
	Input vector (x, y)
	Output vector (r, theta)
	r = |v|
	phi = atan2(y, x)
	"""
	if isinstance(v, tuple):
		x,y = v
	else:
		x,y = v.x, v.y
	# r = |v|
	r = sqrt(x*x + y*y)

	# Fixed!
	theta = atan2(y, x)
	
	return (r, theta)
#end def cartesianToPolar(v)

def polarToCartesian(v):
	"""
	Input vector (r, theta)
	Output vector (x, y)
	See cartesianToPolar for formulae or the source.
	"""
	r, theta = v
	x = r * cos(theta)
	y = r * sin(theta)
	#print('ptc %3.3f %3.3f' % (x, y))
	return (x, y)
#end def polarToCartesian(v)

def vec_sub(v1, v2):
	"""
	NOTE: Don't use this function, use a better function.
	"""
	return (v1[0]-v2[0], v1[1]-v2[1])
#end def vec_sub(v1, v2)

def polar_sub(v1, v2):
	"""
	NOTE: Don't use this function, use cartesian coordinates from the get 
	go, then convert to polar.
	"""
	vc1 = polarToCartesian(v1)
	vc2 = polarToCartesian(v2)
	return cartesianToPolar((vc1[0]-vc2[0], vc1[1]-vc2[1]))
#end def polar_sub(v1, v2)

def polar_add(v1, v2):
	"""
	NOTE: Don't use this function, use cartesian coordinates from the get 
	go, then convert to polar.
	"""
	vc1 = polarToCartesian(v1)
	vc2 = polarToCartesian(v2)
	return cartesianToPolar((vc1[0]+vc2[0], vc1[1]+vc2[1]))
#end def polar_add(v1, v2)

def main():
	import random
	random.seed('abcdefg')
	cartesian_vectors = [(random.random() * 10, random.random() * 10) for i in range(100)]
	polar_vectors = [cartesianToPolar(v) for v in cartesian_vectors]
	print('\n'.join(['%3.3f %3.3f' % v for v in polar_vectors]))
	dvs = vec_sub(polar_vectors[0], polar_vectors[1])
	dvc = vec_sub(cartesian_vectors[0], cartesian_vectors[1])
	diff1 = vec_sub(dvs, cartesianToPolar(dvc))
	diff2 = vec_sub(polarToCartesian(dvs), dvc)
	# NOTE: Subtraction in Polar coordinates is not the same as 
	# subtraction in cartesian coordinates!
	# For example, (0, 0) - (r, theta) = (r, theta+pi)
	print("not same", diff1, diff2)
	
	# This works:
	dvs = polar_sub(polar_vectors[0], polar_vectors[1])
	dvc = vec_sub(cartesian_vectors[0], cartesian_vectors[1])
	diff1 = polar_sub(dvs, cartesianToPolar(dvc))
	diff2 = vec_sub(polarToCartesian(dvs), dvc)
	print('same', diff1, diff2)
	assert(diff1[0] < SMALL) # r should be 0
	assert(diff2[0] < SMALL) # x should be 0
	assert(diff2[1] < SMALL) # y should be 0
#end def main()

if __name__ == '__main__':
	main()
#end if
