#!/usr/bin/env python3
"""
pwmap2.py
by Javantea
Dec 4, 2007

Maps apis on ProgrammableWeb.com

Each api has a list of mashups that use it. Each mashup has a list of apis it 
uses. This is a perfect system to map.

One grok (http transfer) gives us a full map. Sweet.

youtube -- youtube-flicker-mashups -- flicker
 \-- youtube-google-maps-mashups -- google-maps
 \-- youtube-*-mashups -- *
"""
import urllib.request, urllib.error, urllib.parse
from random import seed
import os
from xml.etree import ElementTree
from sys import path
path.append('.')
import mapmake1

# We use urlopen and file depending on protocol.
# http://api.programmableweb.com/

class ApiMashupMap(mapmake1.Map):
	def __init__(self, url='/apis/youtube/mashups', seed_id = 234, pages=1, width=800, height=600, title='Small Wide World ProgrammableWeb Mashup'):
		seed(seed_id)
		# Width, height, line_width = 1.
		mapmake1.Map.__init__(self, None, width, height, 1, '#9999ff')
		self.title = title
		self.p = []
		self.i = 0
		self.names = {}
		self.apikey = '083671210bad9fe22c1d6d183b47fc89'
		
		# Grab a given number of pages from API.
		for pageno in range(pages):
			self.parseUrl(url, seed_id, pageno+1)
		#next pageno
		
		# Change the size of each point depending on how many links it has.
		for point in self.points:
			size = len(point.conns) * 2
			if size > 8:
				# in: out, 8: 8, 9: 8.125, 20: 9.5
				size = 7 + size/8.0
			#end if
			point.size = size
		#next point
		
		# Organize pushes the points into position depending on their connections.
		self.organize()
	#def __init__(seed_id = 234, levels=1)

	def parseUrl(self, url='/apis/youtube/mashups', seed_id = 234, pageno=1):
		p_new = self.i
		pageqs = ''
		if pageno != 1: pageqs = '&page=' + str(pageno)
		filename = 'data/' + url.replace('/', '_')[1:] + str(pageno) + '.xml'
		apikeyqs = '?apikey=' + self.apikey
		if '?' in url: apikeyqs = '&apikey=' + self.apikey
		#print '<!-- Grabbing', 'http://api.programmableweb.com' + url, '-->'
		if not os.path.exists(filename):
			# Need an absolute url.
			try:
				data = urllib.request.urlopen('http://api.programmableweb.com' + url + apikeyqs + pageqs).read()
				open(filename, 'wb').write(data)
			except urllib.error.URLError as e:
				print('Warning: Could not grab api data', url, ':', e)
				#skip over adding this friend's friends.
				return
			#end try
		#end if
		
		# Use XML
		# Thanks to Yahoo Developer for the idea:
		# http://developer.yahoo.com/python/python-xml.html
		tree1 = ElementTree.parse(filename)
		root = tree1.getroot()
		#print root[0].tag, root[0].text
		NS   = "http://www.w3.org/2005/Atom"
		PWNS = "http://www.programmableweb.com/api/opensearch/1.0/"
		urls = [i.get('href') for i in root.findall('{%s}entry/{%s}link' % (NS, NS))]
		mashups = [i.text for i in root.findall('{%s}entry/{%s}title' % (NS, NS))]
		apis2d = [[j.text for j in i.findall('{%s}content/{%s}mashup/{%s}apis/{%s}api/{%s}name' % (NS, PWNS, NS, NS, NS))] for i in root.findall('{%s}entry' % (NS,))]
		api_urls2d = [[j.text for j in i.findall('{%s}content/{%s}mashup/{%s}apis/{%s}api/{%s}url' % (NS, PWNS, NS, NS, NS))] for i in root.findall('{%s}entry' % (NS,))]
		
		name_pts = []
		
		j = 0
		for mashup in mashups:
			if mashup not in list(self.names.keys()):
				#print 'm', mashup
				self.p.append(mapmake1.Point(mashup, color='#99ccff', url=urls[j]))
				self.names[mashup] = self.i
				name_pts.append(self.i)
				self.i += 1
			else:
				name_pts.append(self.names[mashup])
			#end if
			# There are two links per mashup, we want the first
			j += 2
		#next mashup
		
		j = 0
		for apis in apis2d:
			k = 0
			for api in apis:
				# Only add them if they are new.
				if api not in list(self.names.keys()):
					#print 'a', api
					self.p.append(mapmake1.Point(api, color='#cccc99', url=api_urls2d[j][k]))
					self.names[api] = self.i
					api_id = self.i
					self.i += 1
				else:
					api_id = self.names[api]
				#end if
				self.p[name_pts[j]].addConn(self.p[api_id])
				k += 1
			#next tag
			j += 1
		#next tags
		
		# The new ones in self.p are from p_new to the end
		# verified, fixed a bug.
		#print p_new, len(self.p)
		self.addPoints(self.p[p_new:])
	#end def parseUrl([url], [seed_id], levels)
	
	def sort2(self, dist1 = 100, f = 0.5):
		# We can override sort2 to do dist1 = 170 instead of 70.
		mapmake1.Map.sort2(self, dist1, f)
	#end def sort2(self, dist1 = 70, f = 0.5)
	
	def organize(self):
		self.randomize(int((self.width + self.height) / 1.4))
		for i in range(40):
			self.sort2((self.width + self.height) / 15)
			if i % 30 == 0:
				self.sort3()
			#end if
			if i % 32 == 0:
				self.sort4()
			#end if
			if i % 25 == 0:
				self.randomize(200)
			#end if
			#a.sort1()
			#a.sort1()
			#a.sort1()
			#a.sort1()
		#next i
		for i in range(25):
			self.sort4()
		#next i
		#m.sort3()
		self.center()
	#def organize()
	
	#def __str__(self):
	#	return str(self.m)
	#end def __str__()
#end class ApiMashupMap(mapmake1.Map)

def main(seed_id = 4723):
	url = '/apis/youtube/mashups?sort=popular'
	from sys import argv
	if len(argv) > 1:
		url = argv[1]
	#end if
	m = ApiMashupMap(url, seed_id, 2, 1200, 1200, 'Top 20 Youtube Mashups')
	#for point in m.points:
	#	print 'p', point.name
	#next point
	print(m)
#end def main()

if __name__ == '__main__':
	main()
#end if
