#!/usr/bin/env python3
"""
routemap1.py
by Javantea

Traceroute Mapper test.

"""
from __future__ import print_function
import sys
import mapmake1
import re
from random import seed
try:
	import sww_optimize1
except ImportError:
	print("Warning: SWW Optimization has missing dependencies. Try running sww_optimize1.py to find out what.", file=sys.stderr)
	sww_optimize1 = None
#end try

def mapRouteFile(filename):
	a = open(filename, 'r').read()
	functions = []
	data1 = a.split('\n')
	return mapRoute(data1)
#end def mapRouteFile(filename):


def mapRoute(data):
	"""
	mapRoute takes a dump from traceroute and turns it into a useful list of 
	routes that they make.
	"""
	test = r' ?(\d+)  ([^ ]+) \(([^\)]+)\) +(\d*\.?\d* ?m?s?) +(\d*\.?\d* ?m?s?) +(\d*\.?\d* ?m?s?)'
	
	prog = re.compile(test)
	
	links = []
	
	for b in data:
		c = prog.finditer(b)
		for d in c:
			links.append(d.groups())
		#next d
	#next d
	
	"""for line in data:
		func_re = re.match(r'^([0-9a-f]{8}) \<([^\>]+)\>:', line)
		if func_re != None:
			functions.append(list(func_re.groups()))
			continue
		#end if
		call_re = re.match(r'^ +([0-9a-f]+):\t.{21}\tcall   ([0-9a-f]+) \<([^\>]+)\>', line)
		if call_re != None:
			functions[-1].append(call_re.groups())
			continue
		#end if
	#next line"""
	
	return links
#end def def mapRoute(data)

def createRouteMap(filename = 'traceroute1b3.txt', seed_id = 234, optimize=False):
	seed(seed_id)
	m = mapmake1.Map(None, 500, 500)
	r = mapRouteFile(filename)
	
	"""# the format of functions is:
	# [address, name, (call addr, call pos, call name), ...]
	for func in f:
		print func[1] + ': (',
		for call in func[2:]:
			print call[2] + ',',
		#next call
		print ')'
	#next func
	return"""
		
	i = 0
	p = []
	names = {}
	for route in r:
		if route[1] in list(names.keys()):
			continue
		#end if
		p.append(mapmake1.Point(route[1]))
		names[route[1]] = i
		i += 1
	#next route
	i = 0
	prev_d = None
	prev_p = None
	for route in r:
		p_i = p[names[route[1]]]
		if prev_d != None:
			# TODO: This is really difficult. Please do on paper.
			#if route[1] in names.keys():
			#	rel = names.keys().index(route[1])
			#else:
			#	rel = i - 1
			#end if
			#print route[0], '>?', prev[0]
			if int(route[0]) > int(prev_d[0]):
				p_i.addConn(prev_p)
			#else:
			#	print '<!-- new route -->'
			#end if
		#end if
		prev_d = route
		prev_p = p_i
		i += 1
	#next route
	
	m.addPoints(p)
	#m.randomize(300)
	m.sortBest()
	if sww_optimize1 and optimize:
		for i in range(300):
			m.sort2()
			if i % 13 == 0:
				m.sort3()
			#end if
			#a.sort1()
			#a.sort1()
			#a.sort1()
			#a.sort1()
		#next i
		#m.sort3()
		# This is really useful to make a tough traceroute viewable.
		lj_conf = sww_optimize1.LJ_Potential(120)
		sww_optimize1.OptimizeBasinHopping(m, lj_conf, 100, 3)
	else:
		m.centerBound()
	#end if
	return m
#end def def mapRouteSvg([filename], [seed_id], [optimize])

def main(seed_id = 234):
	filename = 'traceroute1b3.txt'
	from sys import argv
	output = 'json'
	optimize = False
	if '-svg' in argv:
		output = 'svg'
		argv.pop('-svg')
	#end if
	if len(argv) > 1:
		filename = argv[1]
	#end if
	m = createRouteMap(filename, seed_id, optimize)
	if output == 'svg':
		print(m)
	else:
		print(m.saveJson())
	#end if
#end def main()

if __name__ == '__main__':
	main()
#end if



